package rasmus_torkel.test.config;

import java.io.File;
import java.io.FileNotFoundException;

import rasmus_torkel.config.ConfigFileReader;
import rasmus_torkel.config.ConfigProcessor;
import rasmus_torkel.config.exception.ConfigArgumentException;
import rasmus_torkel.text.read.exception.ReadException;

public class ConfigFileReaderArgsTest
{

    protected final File[] _inputFiles;
    
    public
    ConfigFileReaderArgsTest(String[] args) throws ReadException
    {
        if (args.length < 1)
        {
            throw new ConfigArgumentException("There must be one or more arguments for the configuration files");
        }
        _inputFiles = new File[args.length];
        for (int i = 0; i < _inputFiles.length; i++)
        {
            _inputFiles[i] = new File(args[i]);
        }
    }
    
    public String
    makeConfigString(ConfigProcessor processor)
    {
        return processor.makeConfigString();
    }
    
    public void
    test() throws ReadException, FileNotFoundException
    {
        ConfigFileReader reader = new ConfigFileReader(_inputFiles);
        System.out.println(reader.makeTrace());
        System.out.println("-------------------------------------");
        String configString = makeConfigString(reader);
        System.out.println(configString);
    }
    
    public static void
    processThrowable(Throwable e)
    {
        if (e instanceof ReadException)
        {
            System.err.println(e.getMessage());
        }
        else if (e instanceof RuntimeException || e instanceof Error)
        {
            e.printStackTrace(System.err);
        }
        else
        {
            System.err.println(e.toString());
        }
    }
    
    public static void
    main(String[] args)
    {
        try
        {
            ConfigFileReaderArgsTest simplifier = new ConfigFileReaderArgsTest(args);
            simplifier.test();
        }
        catch (Throwable e)
        {
            processThrowable(e);
        }
    }
}
