package rasmus_torkel.test.config;

import java.io.File;
import java.util.Properties;

import rasmus_torkel.config.PropertiesView;
import rasmus_torkel.config.PropertySourceDescription;
import rasmus_torkel.config.exception.PropertyDoesNotMatchTypeException;
import rasmus_torkel.config.exception.PropertyNotFoundException;
import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.text.read.exception.SuppliedClassNotEnumException;
import rasmus_torkel.text.read.exception.SuppliedClassNotStringMappedException;
import rasmus_torkel.text.type.strings.Ip4Address;

public class PropertiesViewTester extends UnitTest
{
    private PropertiesView _view;
    
    public
    PropertiesViewTester(PropertiesView view)
    {
        _view = view;
        String viewString = view.toString();
        System.out.print(viewString);
    }
    
    public void
    testIntN(String  name,
             Integer expectedValue)
    {
        testIntN(name, expectedValue, null);
    }
    
    public void
    testIntN(String   name,
             Class<?> expectedExceptionClass)
    {
        testIntN(name, null, expectedExceptionClass);
    }
    
    private void
    testIntN(String   name,
             Integer  expectedValue,
             Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testIntN: " + name);
        try
        {
            Integer value = _view.intN(name);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testIntE(String  name,
             int     expectedValue)
    {
        testIntE(name, expectedValue, null);
    }
    
    public void
    testIntE(String   name,
             Class<?> expectedExceptionClass)
    {
        testIntE(name, -1, expectedExceptionClass);
    }
    
    private void
    testIntE(String   name,
             int      expectedValue,
             Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testIntE: " + name);
        try
        {
            int value = _view.intE(name);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testIntD(String  name,
             int     defaultValue,
             int     expectedValue)
    {
        testIntD(name, defaultValue, expectedValue, null);
    }
    
    public void
    testIntD(String   name,
             int      defaultValue,
             Class<?> expectedExceptionClass)
    {
        testIntD(name, defaultValue, -1, expectedExceptionClass);
    }
    
    private void
    testIntD(String   name,
             int      defaultValue,
             int      expectedValue,
             Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testIntD: " + name);
        try
        {
            int value = _view.intD(name, defaultValue);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    private void
    processValue(int      value,
                 int      expectedValue,
                 Class<?> expectedExceptionClass)
    {
        System.out.println("value = " + value);
        if (expectedExceptionClass != null)
        {
            fail("Expected exception");
        }
        if (value != expectedValue)
        {
            fail("Expected " + expectedValue);
        }
    }
    
    public void
    testLongN(String  name,
              Long    expectedValue)
    {
        testLongN(name, expectedValue, null);
    }
    
    public void
    testLongN(String   name,
              Class<?> expectedExceptionClass)
    {
        testLongN(name, null, expectedExceptionClass);
    }
    
    private void
    testLongN(String   name,
              Long  expectedValue,
              Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testLongN: " + name);
        try
        {
            Long value = _view.longN(name);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testLongE(String  name,
              long     expectedValue)
    {
        testLongE(name, expectedValue, null);
    }
    
    public void
    testLongE(String   name,
              Class<?> expectedExceptionClass)
    {
        testLongE(name, -1, expectedExceptionClass);
    }
    
    private void
    testLongE(String   name,
              long      expectedValue,
              Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testLongE: " + name);
        try
        {
            long value = _view.longE(name);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testLongD(String  name,
              long     defaultValue,
              long     expectedValue)
    {
        testLongD(name, defaultValue, expectedValue, null);
    }
    
    public void
    testLongD(String   name,
              long      defaultValue,
              Class<?> expectedExceptionClass)
    {
        testLongD(name, defaultValue, -1, expectedExceptionClass);
    }
    
    private void
    testLongD(String   name,
              long      defaultValue,
              long      expectedValue,
              Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testLongD: " + name);
        try
        {
            long value = _view.longD(name, defaultValue);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    private void
    processValue(long     value,
                 long     expectedValue,
                 Class<?> expectedExceptionClass)
    {
        System.out.println("value = " + value);
        if (expectedExceptionClass != null)
        {
            fail("Expected exception");
        }
        if (value != expectedValue)
        {
            fail("Expected " + expectedValue);
        }
    }
    
    public void
    testFloatN(String  name,
               Float    expectedValue)
    {
        testFloatN(name, expectedValue, null);
    }
    
    public void
    testFloatN(String   name,
               Class<?> expectedExceptionClass)
    {
        testFloatN(name, null, expectedExceptionClass);
    }
    
    private void
    testFloatN(String   name,
               Float  expectedValue,
               Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testFloatN: " + name);
        try
        {
            Float value = _view.floatN(name);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testFloatE(String  name,
               float     expectedValue)
    {
        testFloatE(name, expectedValue, null);
    }
    
    public void
    testFloatE(String   name,
             Class<?> expectedExceptionClass)
    {
        testFloatE(name, -1, expectedExceptionClass);
    }
    
    private void
    testFloatE(String   name,
               float      expectedValue,
               Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testFloatE: " + name);
        try
        {
            float value = _view.floatE(name);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testFloatD(String  name,
               float     defaultValue,
               float     expectedValue)
    {
        testFloatD(name, defaultValue, expectedValue, null);
    }
    
    public void
    testFloatD(String   name,
               float      defaultValue,
               Class<?> expectedExceptionClass)
    {
        testFloatD(name, defaultValue, -1, expectedExceptionClass);
    }
    
    private void
    testFloatD(String   name,
               float      defaultValue,
               float      expectedValue,
               Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testFloatD: " + name);
        try
        {
            float value = _view.floatD(name, defaultValue);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    private void
    processValue(float     value,
                 float     expectedValue,
                 Class<?> expectedExceptionClass)
    {
        System.out.println("value = " + value);
        if (expectedExceptionClass != null)
        {
            fail("Expected exception");
        }
        if (value != expectedValue)
        {
            fail("Expected " + expectedValue);
        }
    }
    
    public void
    testDoubleN(String  name,
                Double    expectedValue)
    {
        testDoubleN(name, expectedValue, null);
    }
    
    public void
    testDoubleN(String   name,
                Class<?> expectedExceptionClass)
    {
        testDoubleN(name, null, expectedExceptionClass);
    }
    
    private void
    testDoubleN(String   name,
                Double  expectedValue,
                Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testDoubleN: " + name);
        try
        {
            Double value = _view.doubleN(name);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testDoubleE(String  name,
                double     expectedValue)
    {
        testDoubleE(name, expectedValue, null);
    }
    
    public void
    testDoubleE(String   name,
                Class<?> expectedExceptionClass)
    {
        testDoubleE(name, -1, expectedExceptionClass);
    }
    
    private void
    testDoubleE(String   name,
                double      expectedValue,
                Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testDoubleE: " + name);
        try
        {
            double value = _view.doubleE(name);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testDoubleD(String  name,
                double     defaultValue,
                double     expectedValue)
    {
        testDoubleD(name, defaultValue, expectedValue, null);
    }
    
    public void
    testDoubleD(String   name,
                double      defaultValue,
                Class<?> expectedExceptionClass)
    {
        testDoubleD(name, defaultValue, -1, expectedExceptionClass);
    }
    
    private void
    testDoubleD(String   name,
                double      defaultValue,
                double      expectedValue,
                Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testDoubleD: " + name);
        try
        {
            double value = _view.doubleD(name, defaultValue);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    private void
    processValue(double     value,
                 double     expectedValue,
                 Class<?> expectedExceptionClass)
    {
        System.out.println("value = " + value);
        if (expectedExceptionClass != null)
        {
            fail("Expected exception");
        }
        if (value != expectedValue)
        {
            fail("Expected " + expectedValue);
        }
    }
    
    private void
    processValue(Object value,
                 Object expectedValue,
                 Class<?> expectedExceptionClass)
    {
        System.out.println("value = " + value);
        if (expectedExceptionClass != null)
        {
            fail("Expected exception");
        }
        checkValue(value, expectedValue);
    }
    
    private void
    checkValue(Object value,
               Object expectedValue)
    {
        checkValueForNull(value, expectedValue);
        if (value != null)
        {
            if (!value.equals(expectedValue))
            {
                fail("expected " + expectedValue);
            }
        }
    }
    
    private void
    checkValueForNull(Object value,
                      Object expectedValue)
    {
    	if (   (value == null && expectedValue != null)
    	    || (value != null && expectedValue == null))
    	{
    	    fail("expected " + value);
    	}
    }
    
    private void
    checkSuccessVsExpectedException(Class<?> expectedExceptionClass)
    {
        if (expectedExceptionClass != null)
        {
            fail("Expected exception");
        }
    }
    
    public void
    testEnumN(String   name,
              Class<?> enumClass,
              Enum<?>  expectedValue)
    {
        testEnumN(name, enumClass, expectedValue, null);
    }
    
    public void
    testEnumN(String   name,
              Class<?> enumClass,
              Class<?> expectedExceptionClass)
    {
        testEnumN(name, enumClass, null, expectedExceptionClass);
    }
    
    private void
    testEnumN(String   name,
              Class<?> enumClass,
              Enum<?>  expectedValue,
              Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testEnumN: " + name);
        try
        {
            Enum<?> value = _view.enumN(name, enumClass);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testEnumE(String   name,
              Class<?> enumClass,
              Enum<?>  expectedValue)
    {
        testEnumE(name, enumClass, expectedValue, null);
    }
    
    public void
    testEnumE(String   name,
              Class<?> enumClass,
              Class<?> expectedExceptionClass)
    {
        testEnumE(name, enumClass, null, expectedExceptionClass);
    }
    
    private void
    testEnumE(String   name,
              Class<?> enumClass,
              Enum<?>  expectedValue,
              Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testEnumE: " + name);
        try
        {
            Enum<?> value = _view.enumE(name, enumClass);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testEnumD(String   name,
              Enum<?>  defaultValue,
              Class<?> enumClass,
              Enum<?>  expectedValue)
    {
        testEnumD(name, defaultValue, enumClass, expectedValue, null);
    }
    
    public void
    testEnumD(String   name,
              Enum<?>  defaultValue,
              Class<?> enumClass,
              Class<?> expectedExceptionClass)
    {
        testEnumD(name, defaultValue, enumClass, null, expectedExceptionClass);
    }
    
    private void
    testEnumD(String   name,
              Enum<?>  defaultValue,
              Class<?> enumClass,
              Enum<?>  expectedValue,
              Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testEnumD: " + name);
        try
        {
            Enum<?> value = _view.enumD(name, defaultValue, enumClass);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testStringMappedN(String   name,
                      Class<?> stringMappedClass,
                      Object   expectedValue)
    {
        testStringMappedN(name, stringMappedClass, expectedValue, null);
    }
    
    public void
    testStringMappedN(String   name,
                      Class<?> stringMappedClass,
                      Class<?> expectedExceptionClass)
    {
        testStringMappedN(name, stringMappedClass, null, expectedExceptionClass);
    }
    
    private void
    testStringMappedN(String   name,
                      Class<?> stringMappedClass,
                      Object   expectedValue,
                      Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testStringMappedN: " + name);
        try
        {
            Object value = _view.stringMappedN(name, stringMappedClass);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testStringMappedE(String   name,
                      Class<?> stringMappedClass,
                      Object   expectedValue)
    {
        testStringMappedE(name, stringMappedClass, expectedValue, null);
    }
    
    public void
    testStringMappedE(String   name,
                      Class<?> stringMappedClass,
                      Class<?> expectedExceptionClass)
    {
        testStringMappedE(name, stringMappedClass, null, expectedExceptionClass);
    }
    
    private void
    testStringMappedE(String   name,
                      Class<?> stringMappedClass,
                      Object   expectedValue,
                      Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testStringMappedE: " + name);
        try
        {
            Object value = _view.stringMappedE(name, stringMappedClass);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testStringMappedD(String   name,
                      Object   defaultValue,
                      Class<?> stringMappedClass,
                      Object   expectedValue)
    {
        testStringMappedD(name, defaultValue, stringMappedClass, expectedValue, null);
    }
    
    public void
    testStringMappedD(String   name,
                      Object   defaultValue,
                      Class<?> stringMappedClass,
                      Class<?> expectedExceptionClass)
    {
        testStringMappedD(name, defaultValue, stringMappedClass, null, expectedExceptionClass);
    }
    
    private void
    testStringMappedD(String   name,
                      Object   defaultValue,
                      Class<?> stringMappedClass,
                      Object   expectedValue,
                      Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testStringMappedD: " + name);
        try
        {
            Object value = _view.stringMappedD(name, defaultValue, stringMappedClass);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testFileN(String  name,
              File    expectedValue)
    {
        testFileN(name, expectedValue, null);
    }
    
    public void
    testFileN(String   name,
              Class<?> expectedExceptionClass)
    {
        testFileN(name, null, expectedExceptionClass);
    }
    
    private void
    testFileN(String   name,
              File     expectedValue,
              Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testFileN: " + name);
        try
        {
            File value = _view.fileN(name);
            checkValueForNull(value, expectedValue);
            System.out.println("value = " + value);
            checkSuccessVsExpectedException(expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testFileE(String   name,
              File     expectedValue)
    {
        testFileE(name, expectedValue, null);
    }
    
    public void
    testFileE(String   name,
              Class<?> expectedExceptionClass)
    {
        testFileE(name, null, expectedExceptionClass);
    }
    
    private void
    testFileE(String   name,
              File     expectedValue,
              Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testFileE: " + name);
        try
        {
            File value = _view.fileE(name);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    public void
    testFileD(String   name,
              File     defaultValue,
              File     expectedValue)
    {
        testFileD(name, defaultValue, expectedValue, null);
    }
    
    public void
    testFileD(String   name,
              File     defaultValue,
              Class<?> expectedExceptionClass)
    {
        testFileD(name, defaultValue, null, expectedExceptionClass);
    }
    
    private void
    testFileD(String   name,
              File     defaultValue,
              File     expectedValue,
              Class<?> expectedExceptionClass)
    {
        printTestPartSeparator();
        System.out.println("testFileD: " + name);
        try
        {
            File value = _view.fileD(name, defaultValue);
            processValue(value, expectedValue, expectedExceptionClass);
        }
        catch (Exception e)
        {
            processException(e, expectedExceptionClass);
        }
    }
    
    private void
    processException(Exception e,
                     Class<?>  expectedExceptionClass)
    {
        e.printStackTrace(System.out);
        if (expectedExceptionClass == null)
        {
            fail("Exception where success expected");
        }
        else
        {
            Class<?> exceptionClass = e.getClass();
            if (exceptionClass == expectedExceptionClass)
            {
                System.out.println("Exception was as expected");
            }
            else
            {
                fail("Expected different exception type " + expectedExceptionClass.getName());
            }
        }
    }
    
    public static enum Colour
    {
        red,
        green,
        blue
    }
    
    public static void
    main(String[] args)
    {
        Properties props = new Properties();
        float piF = (float)Math.PI;
        props.setProperty("twelve", "12");
        props.setProperty("float.pi", "" + piF);
        props.setProperty("double.pi", "" + Math.PI);
        props.setProperty("colour.strawberry", "red");
        props.setProperty("colour.sky", "blue");
        props.setProperty("colour.leaf", "green");
        props.setProperty("file1", "d:/data/xx.txt");
        props.setProperty("address1", "11.12.13.1");
        props.setProperty("biglong", "111222333444");
        PropertySourceDescription sourceDescription = new PropertySourceDescription("unit test properties");
        PropertiesView view = new PropertiesView(props, sourceDescription);
        PropertiesViewTester t = new PropertiesViewTester(view);
        t.announceTestClassStart();
        t.testIntN("twelve", 12);
        t.testIntN("eleven", (Integer)null);
        t.testIntN("colour.sky", PropertyDoesNotMatchTypeException.class);
        t.testIntE("twelve", 12);
        t.testIntE("eleven", PropertyNotFoundException.class);
        t.testIntE("colour.sky", PropertyDoesNotMatchTypeException.class);
        t.testIntD("twelve", 1212, 12);
        t.testIntD("eleven", 11, 11);
        t.testIntD("colour.sky", 12, PropertyDoesNotMatchTypeException.class);
        t.testLongN("biglong", 111222333444L);
        t.testLongN("eleven", (Long)null);
        t.testLongN("colour.sky", PropertyDoesNotMatchTypeException.class);
        t.testLongE("biglong", 111222333444L);
        t.testLongE("eleven", PropertyNotFoundException.class);
        t.testLongE("colour.sky", PropertyDoesNotMatchTypeException.class);
        t.testLongD("biglong", 1212, 111222333444L);
        t.testLongD("eleven", 11, 11);
        t.testLongD("colour.sky", 12, PropertyDoesNotMatchTypeException.class);
        t.testFloatN("float.pi", piF);
        t.testFloatN("eleven", (Float)null);
        t.testFloatN("colour.sky", PropertyDoesNotMatchTypeException.class);
        t.testFloatE("float.pi", piF);
        t.testFloatE("eleven", PropertyNotFoundException.class);
        t.testFloatE("colour.sky", PropertyDoesNotMatchTypeException.class);
        t.testFloatD("float.pi", 1212F, piF);
        t.testFloatD("eleven", 11F, 11F);
        t.testFloatD("colour.sky", 12, PropertyDoesNotMatchTypeException.class);
        t.testDoubleN("double.pi", Math.PI);
        t.testDoubleN("eleven", (Double)null);
        t.testDoubleN("colour.sky", PropertyDoesNotMatchTypeException.class);
        t.testDoubleE("double.pi", Math.PI);
        t.testDoubleE("eleven", PropertyNotFoundException.class);
        t.testDoubleE("colour.sky", PropertyDoesNotMatchTypeException.class);
        t.testDoubleD("double.pi", 1212D, Math.PI);
        t.testDoubleD("eleven", 11D, 11D);
        t.testDoubleD("colour.sky", 12D, PropertyDoesNotMatchTypeException.class);
        t.testEnumN("colour.strawberry", Colour.class, Colour.red);
        t.testEnumN("colour.sky", Colour.class, Colour.blue);
        t.testEnumN("colour.leaf", Colour.class, Colour.green);
        t.testEnumN("colour.sun", Colour.class, (Colour)null);
        t.testEnumN("colour.sky", File.class, SuppliedClassNotEnumException.class);
        t.testEnumN("twelve", Colour.class, PropertyDoesNotMatchTypeException.class);
        t.testEnumE("colour.strawberry", Colour.class, Colour.red);
        t.testEnumE("colour.sun", Colour.class, PropertyNotFoundException.class);
        t.testEnumE("twelve", Colour.class, PropertyDoesNotMatchTypeException.class);
        File file1 = new File("d:/data/xx.txt");
        File file2 = new File("d:/data/xx2.txt");
        t.testFileN("file1", file1);
        t.testFileN("file_xx", (File)null);
        t.testFileE("file1", file1);
        t.testFileE("file_xx", PropertyNotFoundException.class);
        t.testFileD("file1", file2, file1);
        t.testFileD("file_xx", file2, file2);
        Ip4Address address1 = new Ip4Address("11.12.13.1");
        Ip4Address address2 = new Ip4Address("101.103.103.2");
        t.testStringMappedN("address1", Ip4Address.class, address1);
        t.testStringMappedN("address_xx", Ip4Address.class, (Ip4Address)null);
        t.testStringMappedN("address1", Colour.class, SuppliedClassNotStringMappedException.class);
        t.testStringMappedN("twelve", Ip4Address.class, PropertyDoesNotMatchTypeException.class);
        t.testStringMappedE("address1", Ip4Address.class, address1);
        t.testStringMappedE("address_xx", Ip4Address.class, PropertyNotFoundException.class);
        t.testStringMappedD("address1", address2, Ip4Address.class, address1);
        t.testStringMappedD("address_xx", address2, Ip4Address.class, address2);
        t.testStringMappedN("file1", File.class, file1);
        t.testStringMappedN("file_xx", File.class, (File)null);
        t.testStringMappedE("file1", File.class, file1);
        t.testStringMappedE("file_xx", File.class, PropertyNotFoundException.class);
        t.testStringMappedD("file1", file2, File.class, file1);
        t.testStringMappedD("file_xx", file2, File.class, file2);
        t.announceTestClassPass();
    }
}
