/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2012/2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.set.chars;

import rasmus_torkel.set.chars.CharSet;
import rasmus_torkel.set.chars.DefinedCharSets;

public class CharSetTest
{
    private CharSet[] _charSets;
    
    public
    CharSetTest(CharSet... charSets)
    {
        _charSets = charSets;
    }
    
    public void
    test(int chi)
    {
        char ch = (char)chi;
        System.out.print(chi + ", '" + ch + "'" + ":   ");
        for (CharSet charSet : _charSets)
        {
            boolean in = charSet.isIn(ch);
            String inStr = in ? "in " : "out";
            System.out.print(charSet + " -> " + inStr + "   ");
        }
        if (Character.isWhitespace(ch))
        {
            System.out.print("  white space");
        }
        System.out.println("");
    }
    
    public static void
    test()
    {
        CharSetTest[] tests =
        {
            new CharSetTest(DefinedCharSets.BITS_7, DefinedCharSets.BITS_7_NON_NULL,
                            DefinedCharSets.BITS_8, DefinedCharSets.BITS_8_NON_NULL,
                            DefinedCharSets.BITS_16, DefinedCharSets.BITS_16_NON_NULL),
            new CharSetTest(DefinedCharSets.ENGLISH_KEYBOARD_SYMBOLS, DefinedCharSets.ENGLISH_KEYBOARD_NON_CONTROL,
                            DefinedCharSets.ENGLISH_KEYBOARD_UNIX_TEXT, DefinedCharSets.ENGLISH_KEYBOARD_GENERAL_TEXT),
            new CharSetTest(DefinedCharSets.DIGITS, DefinedCharSets.LATIN_LOWER_CASE_LETTERS, DefinedCharSets.LATIN_UPPER_CASE_LETTERS,
                            DefinedCharSets.LATIN_LETTERS, DefinedCharSets.VARIABLE_FIRST_CHARS, DefinedCharSets.VARIABLE_CHARS),
        };
        for (int testIndex = 0; testIndex < tests.length; testIndex++)
        {
            for (int i = 0; i <= 257; i++)
            {
                tests[testIndex].test(i);
            }
            tests[testIndex].test((int)Character.MAX_VALUE);
        }
    }
    
    public static void
    main(String[] args)
    {
        test();
    }
}
