package rasmus_torkel.test.set.doubles;

import rasmus_torkel.set.doubles.AllDoubles;
import rasmus_torkel.set.doubles.DoubleRange;
import rasmus_torkel.set.doubles.DoubleSet;
import rasmus_torkel.test_framework.UnitTest;

public class DoubleSetsTest extends UnitTest
{
    private void
    test(double    value,
         DoubleSet set,
         boolean   expectIn)
    {
        boolean isIn = set.isIn(value);
        if (isIn)
        {
            System.out.println(value + " is in DoubleSet " + set);
        }
        else
        {
            System.out.println(value + " is out of DoubleSet " + set);
        }
        if (isIn != expectIn)
        {
            fail("expectIn = " + expectIn);
        }
    }
    
    public static void
    main(String[] args)
    {
        DoubleSetsTest t = new DoubleSetsTest();
        t.test();
    }
    
    public void
    test()
    {
        announceTestClassStart();
        
        DoubleRange rangeAll = new DoubleRange(Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY);
        testAll(rangeAll);
        
        printTestPartSeparator();
        
        testAll(AllDoubles.INSTANCE);
        
        printTestPartSeparator();
        
        DoubleRange range1 = new DoubleRange(1.524f, 1.8288f);
        test(1.523999f, range1, false);
        test(1.524f, range1, true);
        test(1.8288f, range1, true);
        test(1.828801f, range1, false);
        
        printTestPartSeparator();
        
        DoubleRange range2 = new DoubleRange(-1e10f, 1e10f);
        test(-100001f * 100001f, range2, false);
        test(-100000f * 100000f, range2, true);
        test(100000f * 100000f, range2, true);
        test(100001f * 100001f, range2, false);
        
        announceTestClassPass();
    }
    
    public void
    testAll(DoubleSet set)
    {
        test(-Double.NEGATIVE_INFINITY, set, true);
        test(-Double.MAX_VALUE, set, true);
        test(-Float.MAX_VALUE, set, true);
        test(-10000000000d, set , true);
        test(-10000000, set , true);
        test(-10000, set , true);
        test(-10, set , true);
        test(-0.01, set , true);
        test(-Float.MIN_NORMAL, set, true);
        test(-Double.MIN_NORMAL, set, true);
        test(0, set, true);
        test(Double.MIN_NORMAL, set, true);
        test(Float.MIN_NORMAL, set, true);
        test(0.01, set , true);
        test(10, set , true);
        test(10000, set , true);
        test(10000000, set , true);
        test(10000000000d, set , true);
        test(Float.MAX_VALUE, set, true);
        test(Double.MAX_VALUE, set, true);
        test(Double.POSITIVE_INFINITY, set, true);
    }
}
