package rasmus_torkel.test.set.floats;

import rasmus_torkel.set.floats.AllFloats;
import rasmus_torkel.set.floats.FloatRange;
import rasmus_torkel.set.floats.FloatSet;
import rasmus_torkel.test_framework.UnitTest;

public class FloatSetsTest extends UnitTest
{
    private void
    test(float    value,
         FloatSet set,
         boolean  expectIn)
    {
        boolean isIn = set.isIn(value);
        if (isIn)
        {
            System.out.println(value + " is in FloatSet " + set);
        }
        else
        {
            System.out.println(value + " is out of FloatSet " + set);
        }
        if (isIn != expectIn)
        {
            fail("expectIn = " + expectIn);
        }
    }
    
    public static void
    main(String[] args)
    {
        FloatSetsTest t = new FloatSetsTest();
        t.test();
    }
    
    public void
    test()
    {
        announceTestClassStart();
        
        FloatRange rangeAll = new FloatRange(Float.NEGATIVE_INFINITY, Float.POSITIVE_INFINITY);
        testAll(rangeAll);
        
        printTestPartSeparator();
        
        testAll(AllFloats.INSTANCE);
        
        printTestPartSeparator();
        
        FloatRange range1 = new FloatRange(1.524f, 1.8288f);
        test(1.523999f, range1, false);
        test(1.524f, range1, true);
        test(1.8288f, range1, true);
        test(1.828801f, range1, false);
        
        printTestPartSeparator();
        
        FloatRange range2 = new FloatRange(-1e10f, 1e10f);
        test(-100001f * 100001f, range2, false);
        test(-100000f * 100000f, range2, true);
        test(100000f * 100000f, range2, true);
        test(100001f * 100001f, range2, false);
        
        announceTestClassPass();
    }
    
    public void
    testAll(FloatSet set)
    {
        test(Float.NEGATIVE_INFINITY, set, true);
        test(-Float.MAX_VALUE, set, true);
        test(-10000000000f, set , true);
        test(-10000000f, set , true);
        test(-10000f, set , true);
        test(-10f, set , true);
        test(-0.01f, set , true);
        test(-Float.MIN_NORMAL, set, true);
        test(0, set, true);
        test(Float.MIN_NORMAL, set, true);
        test(0.01f, set , true);
        test(10f, set , true);
        test(10000f, set , true);
        test(10000000f, set , true);
        test(10000000000f, set , true);
        test(Float.MAX_VALUE, set, true);
        test(Float.POSITIVE_INFINITY, set, true);
    }
}
