package rasmus_torkel.test.set.strings;

import rasmus_torkel.set.strings.CaseInsensitiveStringSet;
import rasmus_torkel.set.strings.ExplicitStringSet;
import rasmus_torkel.set.strings.StringLengthRange;
import rasmus_torkel.set.strings.StringPatternSet;
import rasmus_torkel.set.strings.StringSet;
import rasmus_torkel.set.strings.StringSetIntersection;
import rasmus_torkel.set.strings.StringSetUnion;
import rasmus_torkel.test_framework.UnitTest;

public class StringSetsTest extends UnitTest
{
    private void
    test(String    value,
         StringSet set,
         boolean   expectIn)
    {
        boolean isIn = set.isIn(value);
        if (isIn)
        {
            System.out.println("\"" + value + "\" is in StringSet " + set);
        }
        else
        {
            System.out.println("\"" + value + "\" is out of StringSet " + set);
        }
        if (isIn != expectIn)
        {
            fail("expectIn = " + expectIn);
        }
    }
    
    public static void
    main(String[] args)
    {
        StringSetsTest t = new StringSetsTest();
        t.test();
    }
    
    public void
    test()
    {
        announceTestClassStart();
        String doug = "Doug";
        String emily = "Emily";
        String floris = "Floris";
        String gerhard = "Gerhard";
        String hans_lee = "Hans Lee";
        String ingo_wong = "Ingo Wong";
        
        StringLengthRange lengthRange = new StringLengthRange(5, 8);
        test(doug, lengthRange, false);
        test(emily, lengthRange, true);
        test(floris, lengthRange, true);
        test(gerhard, lengthRange, true);
        test(hans_lee, lengthRange, true);
        test(ingo_wong, lengthRange, false);
        
        printTestPartSeparator();
        
        ExplicitStringSet directionSet = new ExplicitStringSet("north", "east", "south", "west");
        test("north", directionSet, true);
        test("east",  directionSet, true);
        test("south", directionSet, true);
        test("west",  directionSet, true);
        test("left",  directionSet, false);
        test("North", directionSet, false);
        
        printTestPartSeparator();
        
        CaseInsensitiveStringSet directionSetCI = new CaseInsensitiveStringSet("north", "east", "south", "west");
        test("North", directionSetCI, true);
        test("EAST",  directionSetCI, true);
        test("south", directionSetCI, true);
        test("wEST",  directionSetCI, true);
        test("left",  directionSetCI, false);
        
        printTestPartSeparator();
        
        StringPatternSet idSet = new StringPatternSet("^[a-zA-Z_]+[a-zA-Z0-9_]*$");
        test("_patternString0", idSet, true);
        test("_", idSet, true);
        test("", idSet, false);
        test("1a", idSet, false);
        test("_a,", idSet, false);
        test(doug, idSet, true);
        test(emily, idSet, true);
        test(floris, idSet, true);
        test(gerhard, idSet, true);
        test(hans_lee, idSet, false);
        test(ingo_wong, idSet, false);
        
        printTestPartSeparator();
        
        StringSetIntersection intersection = new StringSetIntersection(lengthRange, idSet);
        test(doug, intersection, false);
        test(emily, intersection, true);
        test(floris, intersection, true);
        test(gerhard, intersection, true);
        test(hans_lee, intersection, false);
        test(ingo_wong, intersection, false);
        
        printTestPartSeparator();
        
        StringSetUnion union = new StringSetUnion(lengthRange, idSet);
        test(doug, union, true);
        test(emily, union, true);
        test(floris, union, true);
        test(gerhard, union, true);
        test(hans_lee, union, true);
        test(ingo_wong, union, false);
        
        printTestPartSeparator();
        
        StringSetIntersection emptyIntersection = new StringSetIntersection();
        test(doug, emptyIntersection, true);
        
        printTestPartSeparator();
        
        StringSetUnion emptyUnion = new StringSetUnion();
        test(doug, emptyUnion, false);
        
        printTestPartSeparator();
        
        try
        {
            new StringPatternSet((String)null);
            fail("No exception for new StringPatternSet((String)null)");
        }
        catch (RuntimeException e)
        {
            System.out.println("After new StringPatternSet((String)null): " + e);
        }
        
        announceTestClassPass();
    }
}
