/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.text.type.strings;

import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.text.type.strings.Ip4Address;

public class Ip4AddressTest extends UnitTest
{
    private final Ip4Address cmpNumber = new Ip4Address("1.2.3.4");
    
    public void
    testOne(String string)
    {
        System.out.println("testOne: " + string);
        Ip4Address a_s = new Ip4Address(string);
        String s_s = a_s.allFieldsToString();
        System.out.println("from String: " + s_s);
        Ip4Address a_b = new Ip4Address(a_s._byte0, a_s._byte1, a_s._byte2, a_s._byte3);
        String s_b = a_b.allFieldsToString();
        System.out.println("from bytes: " + s_b);
        if (!s_b.equals(s_s))
        {
            fail("byte constructed address != string constructed address");
        }
        Ip4Address a_n = new Ip4Address(a_b._number);
        String s_n = a_n.allFieldsToString();
        System.out.println("from number: " + s_n);
        if (!s_n.equals(s_b))
        {
            fail("number constructed address != byte constructed address");
        }
        System.out.println("toString() = " + a_s.toString());
        System.out.println("hashCode() = " + a_s.hashCode());
        System.out.println("equals(" + cmpNumber + ") = " + a_s.equals(cmpNumber));
        System.out.println("compareTo(" + cmpNumber + ") = " + a_s.compareTo(cmpNumber));
        printTestPartSeparator();
    }
    
    public void
    testFail(String string)
    {
        try
        {
            new Ip4Address(string);
            fail(string + " did not trigger exception");
        }
        catch (RuntimeException e)
        {
            if (e.getClass() != Ip4Address.InitialisationException.class)
            {
                fail("Got wrong exception for " + string + ": " + e);
            }
            System.out.println("Correctly got exception for " + string + ": " + e);
        }
    }
    
    public void
    testFail(int byte0,
             int byte1,
             int byte2,
             int byte3)
    {
        String bytesStr = byte0 + ", " + byte1 + ", " + byte2 + ", " + byte3;
        try
        {
            new Ip4Address(byte0, byte1, byte2, byte3);
            fail(bytesStr + " did not trigger exception");
        }
        catch (RuntimeException e)
        {
            if (e.getClass() != Ip4Address.InitialisationException.class)
            {
                fail("Got wrong exception for " + bytesStr + ": " + e);
            }
            System.out.println("Correctly got exception for " + bytesStr + ": " + e);
        }
    }
    
    public void
    testFail(long number)
    {
        try
        {
            new Ip4Address(number);
            fail(number + " did not trigger exception");
        }
        catch (RuntimeException e)
        {
            if (e.getClass() != Ip4Address.InitialisationException.class)
            {
                fail("Got wrong exception for " + number + ": " + e);
            }
            System.out.println("Correctly got exception for " + number + ": " + e);
        }
    }
    
    public void
    test()
    {
        announceTestClassStart();
        testOne("0.0.0.0");
        testOne("255.255.255.255");
        testOne("1.2.3.4");
        testOne("1.2.4.8");
        testOne("16.32.64.128");

        testFail("");
        testFail(".");
        testFail("1.2.3");
        testFail("1.2.3.");
        testFail("1.2.3.4.");
        testFail("1.2.3..4");
        testFail("1..2.3.4");
        testFail("1.2.3.4x");
        testFail("1.2.3.4.5");
        testFail("1.2.3,4");
        testFail("1.2,3.4");
        testFail("1.x.3.4");
        testFail("256.2.3.4");
        testFail("1.256.3.4");
        testFail("1.2.256.4");
        testFail("1.2.3.256");
        testFail("1000.2.3.4");
        testFail("1.1000.3.4");
        testFail("1.2.1000.4");
        testFail("1.2.3.1000");
        printTestPartSeparator();
        
        testFail(-1, 0, 0, 0);
        testFail(0, -1, 0, 0);
        testFail(0, 0, -1, 0);
        testFail(0, 0, 0, -1);
        testFail(256, 0, 0, 0);
        testFail(0, 256, 0, 0);
        testFail(0, 0, 256, 0);
        testFail(0, 0, 0, 256);
        printTestPartSeparator();
        
        testFail(-1L);
        testFail(0x100000000L);
        printTestPartSeparator();
        
        announceTestClassPass();
    }
    
    public static void
    main(String[] args)
    {
        Ip4AddressTest t = new Ip4AddressTest();
        t.test();
    }
}
