package rasmus_torkel.test.xml_basic.oo;

import rasmus_torkel.misc.StringUtil;
import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.text.read.exception.MissingNodeException;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.factory.XmlFactoriesForStandardTypes;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;
import rasmus_torkel.xml_basic.write.XmlSink;

public class DifferentParametersTest extends UnitTest
{
    private static final String rootName = "root";
    private static final String arrayRelName = "array";
    private static final XmlNameSpace arrayNs = new XmlNameSpace("a", "ns_array");
    private static final TagNodeId arrayId = new TagNodeId(arrayRelName, arrayNs);
    private static final String elementRelName = "element";
    private static final XmlNameSpace elementNs = new XmlNameSpace("e", "ns_element");
    private static final TagNodeId elementId = new TagNodeId(elementRelName, elementNs);
    private static final String[] inStrings = { "aa", "bb" };
    private static final O[] inOs = { new O(2001), new O(2002) };
    
    public static void
    main(String[] args)
    {
        testEmpty();
        stringCycleRN();
        stringCycleR();
        stringCycleI();
        oCycleRN();
        oCycleR();
        oCycleI();
    }
    
    public static void
    testEmpty()
    {
        XmlSink sink = new XmlSink();
        sink.startNode(rootName);
        sink.closeNode();
        String emptyXml = sink.toString();
        TagNode rootNode;
        System.out.println("Testing with empty XML");
        System.out.println(emptyXml);
        printTestPartSeparator();
        
        rootNode = getRoot(emptyXml);
        try
        {
            XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentE(rootNode, arrayRelName, arrayNs, StringUtil.ARRAY_MAKER);
            System.out.println("Unexpectedly succeeded");
            throw new UnitTestError();
        }
        catch (MissingNodeException e)
        {
            System.out.println("Caught exception as expected: " + e);
        }
        printTestPartSeparator();
        
        rootNode = getRoot(emptyXml);
        try
        {
            XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentE(rootNode, arrayRelName, StringUtil.ARRAY_MAKER);
            System.out.println("Unexpectedly succeeded");
            throw new UnitTestError();
        }
        catch (MissingNodeException e)
        {
            System.out.println("Caught exception as expected: " + e);
        }
        printTestPartSeparator();
        
        rootNode = getRoot(emptyXml);
        try
        {
            XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentE(rootNode, arrayId, StringUtil.ARRAY_MAKER);
            System.out.println("Unexpectedly succeeded");
            throw new UnitTestError();
        }
        catch (MissingNodeException e)
        {
            System.out.println("Caught exception as expected: " + e);
        }
        printTestPartSeparator();
        
        rootNode = getRoot(emptyXml);
        try
        {
            XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentE(
                    rootNode, arrayRelName, arrayNs, elementRelName, elementNs, StringUtil.ARRAY_MAKER);
            System.out.println("Unexpectedly succeeded");
            throw new UnitTestError();
        }
        catch (MissingNodeException e)
        {
            System.out.println("Caught exception as expected: " + e);
        }
        printTestPartSeparator();
        
        rootNode = getRoot(emptyXml);
        try
        {
            XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentE(
                    rootNode, arrayRelName, elementRelName, StringUtil.ARRAY_MAKER);
            System.out.println("Unexpectedly succeeded");
            throw new UnitTestError();
        }
        catch (MissingNodeException e)
        {
            System.out.println("Caught exception as expected: " + e);
        }
        printTestPartSeparator();
        
        rootNode = getRoot(emptyXml);
        try
        {
            XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentE(
                    rootNode, arrayId, elementId, StringUtil.ARRAY_MAKER);
            System.out.println("Unexpectedly succeeded");
            throw new UnitTestError();
        }
        catch (MissingNodeException e)
        {
            System.out.println("Caught exception as expected: " + e);
        }
        printTestPartSeparator();
        
        String[] array;
        
        rootNode = getRoot(emptyXml);
        array = XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentN(rootNode, arrayRelName, arrayNs, StringUtil.ARRAY_MAKER);
        if (array == null)
        {
            System.out.println("Got null as expected");
        }
        else
        {
            System.out.println("Unexpectedly got non-null");
            throw new UnitTestError();
        }
        printTestPartSeparator();
        
        rootNode = getRoot(emptyXml);
        array = XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentN(rootNode, arrayRelName, StringUtil.ARRAY_MAKER);
        if (array == null)
        {
            System.out.println("Got null as expected");
        }
        else
        {
            System.out.println("Unexpectedly got non-null");
            throw new UnitTestError();
        }
        printTestPartSeparator();
        
        rootNode = getRoot(emptyXml);
        array = XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentN(rootNode, arrayId, elementId, StringUtil.ARRAY_MAKER);
        if (array == null)
        {
            System.out.println("Got null as expected");
        }
        else
        {
            System.out.println("Unexpectedly got non-null");
            throw new UnitTestError();
        }
        printTestPartSeparator();
        
        rootNode = getRoot(emptyXml);
        array = XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentN(
                  rootNode, arrayRelName, arrayNs, elementRelName, elementNs, StringUtil.ARRAY_MAKER);
        if (array == null)
        {
            System.out.println("Got null as expected");
        }
        else
        {
            System.out.println("Unexpectedly got non-null");
            throw new UnitTestError();
        }
        printTestPartSeparator();
        
        rootNode = getRoot(emptyXml);
        array = XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentN(
                  rootNode, arrayRelName, elementRelName, StringUtil.ARRAY_MAKER);
        if (array == null)
        {
            System.out.println("Got null as expected");
        }
        else
        {
            System.out.println("Unexpectedly got non-null");
            throw new UnitTestError();
        }
        printTestPartSeparator();
        
        rootNode = getRoot(emptyXml);
        array = XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentN(
                  rootNode, arrayId, elementId, StringUtil.ARRAY_MAKER);
        if (array == null)
        {
            System.out.println("Got null as expected");
        }
        else
        {
            System.out.println("Unexpectedly got non-null");
            throw new UnitTestError();
        }
        printEndTest();
    }
    
    private static TagNode
    getRoot(String xml)
    {
        return XmlReader.xmlStringToRoot(xml, "unit test", XmlReadOptions.DEFAULT, rootName);
    }
    
    private static void
    stringCycleRN()
    {
        System.out.println("RN string cycle");
        printStrings(inStrings);
        XmlSink sink = new XmlSink();
        sink.startNode(rootName);
        sink.sinkToStringArrayNode(arrayRelName, arrayNs, elementRelName, elementNs, inStrings);
        sink.closeNode();
        sink.abandon();
        String xml = sink.toString();
        System.out.println(xml);
        printTestPartSeparator();
        
        {
            System.out.println("RN N");
            TagNode rootNode = getRoot(xml);
            String[] stringsN =
                    XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentN(
                            rootNode, arrayRelName, arrayNs, elementRelName, elementNs, StringUtil.ARRAY_MAKER);
            checkStrings(stringsN);
        }
        
        {
            System.out.println("RN E");
            TagNode rootNode = getRoot(xml);
            String[] stringsE =
                    XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentE(
                            rootNode, arrayRelName, arrayNs, elementRelName, elementNs, StringUtil.ARRAY_MAKER);
            checkStrings(stringsE);
        }
        printEndTest();
    }
    
    private static void
    stringCycleR()
    {
        System.out.println("R string cycle");
        printStrings(inStrings);
        XmlSink sink = new XmlSink();
        sink.startNode(rootName);
        sink.sinkToStringArrayNode(arrayRelName, elementRelName, inStrings);
        sink.closeNode();
        sink.abandon();
        String xml = sink.toString();
        System.out.println(xml);
        printTestPartSeparator();
        
        {
            System.out.println("R N");
            TagNode rootNode = getRoot(xml);
            String[] stringsN =
                    XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentN(
                            rootNode, arrayRelName, elementRelName, StringUtil.ARRAY_MAKER);
            checkStrings(stringsN);
        }
        
        {
            System.out.println("R E");
            TagNode rootNode = getRoot(xml);
            String[] stringsE =
                    XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentE(
                            rootNode, arrayRelName, elementRelName, StringUtil.ARRAY_MAKER);
            checkStrings(stringsE);
        }
        printEndTest();
    }
    
    private static void
    stringCycleI()
    {
        System.out.println("I string cycle");
        printStrings(inStrings);
        XmlSink sink = new XmlSink();
        sink.startNode(rootName);
        sink.sinkToStringArrayNode(arrayId, elementId, inStrings);
        sink.closeNode();
        sink.abandon();
        String xml = sink.toString();
        System.out.println(xml);
        printTestPartSeparator();
        
        {
            System.out.println("I N");
            TagNode rootNode = getRoot(xml);
            String[] stringsN =
                    XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentN(
                            rootNode, arrayId, elementId, StringUtil.ARRAY_MAKER);
            checkStrings(stringsN);
        }
        
        {
            System.out.println("I E");
            TagNode rootNode = getRoot(xml);
            String[] stringsE =
                    XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentE(
                            rootNode, arrayId, elementId, StringUtil.ARRAY_MAKER);
            checkStrings(stringsE);
        }
        printEndTest();
    }
    
    private static void
    printStrings(String[] strings)
    {
        System.out.println(strings.length + " strings");
        for (String string : strings)
        {
            System.out.println(string);
        }
        printTestPartSeparator();
    }
    
    private static void
    checkStrings(String[] strings)
    {
        printStrings(strings);
        if (strings.length != inStrings.length)
        {
            System.out.println("Bad length");
            throw new UnitTestError();
        }
        for (int i = 0; i < strings.length; i++)
        {
            if (!strings[i].equals(inStrings[i]))
            {
                System.out.println("Wrong string at index " + i);
                throw new UnitTestError();
            }
        }
        System.out.println("Strings match");
        printTestPartSeparator();
    }
    
    private static void
    oCycleRN()
    {
        System.out.println("RN O cycle");
        printOs(inOs);
        XmlSink sink = new XmlSink();
        sink.startNode(rootName);
        sink.sinkArrayNode(arrayRelName, arrayNs, inOs);
        sink.closeNode();
        sink.abandon();
        String xml = sink.toString();
        System.out.println(xml);
        printTestPartSeparator();
        
        {
            System.out.println("RN N");
            TagNode rootNode = getRoot(xml);
            O[] os = O.FROM_XML_FACTORY.nextArrayFromParentN(rootNode, arrayRelName, arrayNs, O.ARRAY_MAKER);
            checkStrings(os);
        }
        
        {
            System.out.println("RN E");
            TagNode rootNode = getRoot(xml);
            O[] os = O.FROM_XML_FACTORY.nextArrayFromParentE(rootNode, arrayRelName, arrayNs, O.ARRAY_MAKER);
            checkStrings(os);
        }
        printEndTest();
    }
    
    private static void
    oCycleR()
    {
        System.out.println("R O cycle");
        printOs(inOs);
        XmlSink sink = new XmlSink();
        sink.startNode(rootName);
        sink.sinkArrayNode(arrayRelName, inOs);
        sink.closeNode();
        sink.abandon();
        String xml = sink.toString();
        System.out.println(xml);
        printTestPartSeparator();
        
        {
            System.out.println("R N");
            TagNode rootNode = getRoot(xml);
            O[] os = O.FROM_XML_FACTORY.nextArrayFromParentN(rootNode, arrayRelName, O.ARRAY_MAKER);
            checkStrings(os);
        }
        
        {
            System.out.println("R E");
            TagNode rootNode = getRoot(xml);
            O[] os = O.FROM_XML_FACTORY.nextArrayFromParentE(rootNode, arrayRelName, O.ARRAY_MAKER);
            checkStrings(os);
        }
        printEndTest();
    }
    
    private static void
    oCycleI()
    {
        System.out.println("I B cycle");
        printOs(inOs);
        XmlSink sink = new XmlSink();
        sink.startNode(rootName);
        sink.sinkArrayNode(arrayId, inOs);
        sink.closeNode();
        sink.abandon();
        String xml = sink.toString();
        System.out.println(xml);
        printTestPartSeparator();
        
        {
            System.out.println("I N");
            TagNode rootNode = getRoot(xml);
            O[] os = O.FROM_XML_FACTORY.nextArrayFromParentN(rootNode, arrayId, O.ARRAY_MAKER);
            checkStrings(os);
        }
        
        {
            System.out.println("I E");
            TagNode rootNode = getRoot(xml);
            O[] os = O.FROM_XML_FACTORY.nextArrayFromParentE(rootNode, arrayId, O.ARRAY_MAKER);
            checkStrings(os);
        }
        printEndTest();
    }
    
    private static void
    printOs(O[] os)
    {
        System.out.println(os.length + " O objects");
        for (O o : os)
        {
            System.out.println(o);
        }
        printTestPartSeparator();
    }
    
    private static void
    checkStrings(O[] os)
    {
        printOs(os);
        if (os.length != inOs.length)
        {
            System.out.println("Bad length");
            throw new UnitTestError();
        }
        for (int i = 0; i < os.length; i++)
        {
            if (!os[i].equals(inOs[i]))
            {
                System.out.println("Wrong O object at index " + i);
                throw new UnitTestError();
            }
        }
        System.out.println("O objects match");
        printTestPartSeparator();
    }
}
