/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.xml_basic.oo;

import rasmus_torkel.misc.ArrayMaker;
import rasmus_torkel.misc.StringUtil;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.factory.MainXmlObjectFactory;
import rasmus_torkel.xml_basic.read.factory.XmlFactoriesForStandardTypes;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;
import rasmus_torkel.xml_basic.write.XmlSink;
import rasmus_torkel.xml_basic.write.XmlSinkWritable;

public class PersonName implements XmlSinkWritable
{
    public  final String   _firstName;
    private final String[] _middleNames;
    public  final String   _surname;
    
    public static final TagNodeId NATURAL_TAG_NODE_ID = new TagNodeId("personName");
    
    public static MainXmlObjectFactory<PersonName> FROM_XML_FACTORY = new MainXmlObjectFactory<PersonName>(NATURAL_TAG_NODE_ID)
    {
        @Override
        public PersonName extractFromNode(TagNode node)
        {
            return new PersonName(node);
        }
    };
    
    public static final ArrayMaker<PersonName> ARRAY_MAKER = new ArrayMaker<PersonName>()
    {
        @Override
        public PersonName[]
        newArray(int size)
        {
            return new PersonName[size];
        }
    };
    
    public
    PersonName(TagNode node)
    {
        _firstName = node.nextTextFieldE("firstName");
        _middleNames =
                XmlFactoriesForStandardTypes.STRING_FACTORY.nextArrayFromParentN(
                        node, "middleNames", "middleName", StringUtil.ARRAY_MAKER);
        _surname = node.nextTextFieldE("surname");
    }
    
    public
    PersonName(String   firstName,
               String   surname)
    {
        this(firstName, (String[])null, surname);
    }
    
    public
    PersonName(String   firstName,
               String   middleName,
               String   surname)
    {
        this(firstName, new String[]{middleName}, surname);
    }
    
    public
    PersonName(String   firstName,
               String[] middleNames,
               String   surname)
    {
        _firstName = firstName;
        _middleNames = middleNames;
        _surname = surname;
    }
    
    public String
    toString()
    {
        StringBuilder buf = new StringBuilder();
        buf.append(_firstName);
        if (_middleNames != null)
        {
            for (String middleName : _middleNames)
            {
                buf.append(" ");
                buf.append(middleName);
            }
        }
        buf.append(" ");
        buf.append(_surname);
        return buf.toString();
    }

    @Override
    public TagNodeId naturalTagNodeId()
    {
        return NATURAL_TAG_NODE_ID;
    }

    @Override
    public void
    mostToXml(XmlSink      xmlSink,
              String       relativeName,
              XmlNameSpace nameSpace)
    {
        xmlSink.sinkSimpleNode("firstName", _firstName);
        xmlSink.sinkToStringArrayNode("middleNames", "middleName", _middleNames);
        xmlSink.sinkSimpleNode("surname", _surname);
    }
}
