package rasmus_torkel.test.xml_basic.oo;

import java.io.File;

import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.MiscXmlFunctions;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;
import rasmus_torkel.xml_basic.write.XmlSink;

public class PersonNameDemo extends UnitTest
{
    public static void
    main(String[] args)
    {
        announceTestClassStart(PersonNameDemo.class);
        demoSingle();
        demoSingleTagId();
        PersonName[] personNames =
        {
                new PersonName("Alexander", "Armstrong"),
                new PersonName("Bianca", "Bella", "Brown"),
                new PersonName("Christopher", new String[]{"Carlos", "Claus"}, "Cooper"),
        };
        demoArray(personNames);
        demoArrayTagIds(personNames);
        demoSingleLineXmlString();
        demoSingleLineXmlFile();
        demoSingleLineArrayNameXmlString();
        demoSingleLineArrayIdXmlString();
        demoSingleLineArrayNameXmlFile();
        demoSingleLineArrayIdXmlFile();
        announceTestClassPass(PersonNameDemo.class);
    }
    
    public static void
    demoSingle()
    {
        PersonName name = new PersonName("Alfred", new String[]{"Bernhard", "Carlos"}, "Dreyfuss");
        System.out.println("Doing " + name);
        printTestPartSeparator();
        
        XmlSink xmlSink = new XmlSink();
        xmlSink.sinkNode(name);
        String xml = xmlSink.toString();
        System.out.print(xml);
        printTestPartSeparator();
        
        MiscXmlFunctions.showXmlForHtml(xml);
        printTestPartSeparator();
        
        System.out.println("Using XmlStringifier");
        System.out.println(XmlSink.objectToString(name));
        printTestPartSeparator();

        System.out.println("Doing reverse");
        TagNode rootNode =
            XmlReader.xmlStringToRoot(xml, "demo", XmlReadOptions.DEFAULT, PersonName.NATURAL_TAG_NODE_ID);
        PersonName name2 = PersonName.FROM_XML_FACTORY.extractFromNode(rootNode);
        rootNode.verifyNoMoreChildren();
        String xml2 = XmlSink.objectToString(name2);
        System.out.print(xml2);
        boolean equals = xml2.equals(xml);
        System.out.println("equals: " + equals);
        if (!equals)
        {
            throw new UnitTestError(PersonNameDemo.class);
        }
        printEndTest();
    }
    
    public static void
    demoSingleTagId()
    {
        PersonName name = new PersonName("Alfred", new String[]{"Bernhard", "Carlos"}, "Dreyfuss");
        XmlNameSpace nameSpace = new XmlNameSpace("x", "http://rasmustorkel.com/ns1/");
        TagNodeId tagNodeId = new TagNodeId("xname", nameSpace);
        System.out.println("Doing " + name + ", using " + tagNodeId);
        printTestPartSeparator();
        
        XmlSink xmlSink = new XmlSink();
        xmlSink.sinkNode(tagNodeId, name);
        String xml = xmlSink.toString();
        System.out.print(xml);
        printTestPartSeparator();
        
        MiscXmlFunctions.showXmlForHtml(xml);
        printEndTest();

        System.out.println("Doing reverse");
        TagNode rootNode =
            XmlReader.xmlStringToRoot(xml, "demo", XmlReadOptions.DEFAULT, tagNodeId);
        PersonName name2 = PersonName.FROM_XML_FACTORY.extractFromNode(rootNode);
        XmlSink sink2 = new XmlSink();
        sink2.sinkNode(tagNodeId, name2);
        String xml2 = sink2.toString();
        System.out.print(xml2);
        boolean equals = xml2.equals(xml);
        System.out.println("equals: " + equals);
        if (!equals)
        {
            throw new UnitTestError(PersonNameDemo.class);
        }
        printEndTest();
    }
    
    public static void
    demoArray(PersonName[] personNames)
    {
        XmlSink xmlSink = new XmlSink();
        xmlSink.sinkArrayNode("personNames", personNames);
        String xml = xmlSink.toString();
        System.out.print(xml);
        printTestPartSeparator();
        
        MiscXmlFunctions.showXmlForHtml(xml);
        printTestPartSeparator();
        
        TagNode rootNode =
                XmlReader.xmlStringToRoot(xml, "demo", XmlReadOptions.DEFAULT, "personNames");
        PersonName[] personNames2 =
                PersonName.FROM_XML_FACTORY.extractArrayFromNode(rootNode, PersonName.ARRAY_MAKER);
        String xml2 = XmlSink.arrayToString("personNames", personNames2);
        System.out.print(xml2);
        boolean equals = xml2.equals(xml);
        System.out.println("equals: " + equals);
        if (!equals)
        {
            throw new UnitTestError(PersonNameDemo.class);
        }
        printEndTest();
    }
    
    public static void
    demoArrayTagIds(PersonName[] personNames)
    {
        XmlSink xmlSink = new XmlSink();
        XmlNameSpace nameSpace = new XmlNameSpace("x", "http://rasmustorkel.com/ns1/");
        TagNodeId arrayId = new TagNodeId("xnames", nameSpace);
        TagNodeId elementId = new TagNodeId("xname", nameSpace);
        xmlSink.sinkToStringArrayNode(arrayId, elementId, personNames);
        String xml = xmlSink.toString();
        System.out.print(xml);
        printTestPartSeparator();
        
        MiscXmlFunctions.showXmlForHtml(xml);
        printEndTest();
    }
    
    public static void
    demoSingleLineXmlString()
    {
        System.out.println("OneLineXml with string");
        PersonName name = new PersonName("Rasmus", "Torkel");
        String nameString = name.toString();
        System.out.println(nameString);
        printTestPartSeparator();
        String xmlString = XmlSink.objectToString(name);
        System.out.println(xmlString);
        printTestPartSeparator();
        PersonName name2 = PersonName.FROM_XML_FACTORY.stringToObject(xmlString, "demo");
        String nameString2 = name2.toString();
        System.out.println(nameString2);
        printTestPartSeparator();
        boolean equals = nameString2.equals(nameString);
        System.out.println("equals: " + equals);
        if (!equals)
        {
            throw new UnitTestError(PersonNameDemo.class);
        }
        printEndTest();
    }
    
    public static void
    demoSingleLineXmlFile()
    {
        File file = new File("./data/rasmus_torkel/test/xml_basic/oo/personName.xml");
        System.out.println("OneLineXml with file " + file);
        PersonName name = new PersonName("Rasmus", "Torkel");
        String nameString = name.toString();
        System.out.println(nameString);
        printTestPartSeparator();
        XmlSink.objectToFile(name, file);
        System.out.println("Written to file");
        printTestPartSeparator();
        PersonName name2 = PersonName.FROM_XML_FACTORY.fileToObject(file);
        System.out.println("Read name");
        String nameString2 = name2.toString();
        System.out.println(nameString2);
        printTestPartSeparator();
        boolean equals = nameString2.equals(nameString);
        System.out.println("equals: " + equals);
        if (!equals)
        {
            throw new UnitTestError(PersonNameDemo.class);
        }
        printEndTest();
    }
    
    public static void
    demoSingleLineArrayNameXmlString()
    {
        System.out.println("OneLineXml with string");
        PersonName[] names = makeSmallArray();
        String nameString = toString(names);
        System.out.println(nameString);
        printTestPartSeparator();
        String xmlString = XmlSink.arrayToString("array", names);
        System.out.println(xmlString);
        printTestPartSeparator();
        PersonName[] names2 = PersonName.FROM_XML_FACTORY.stringToArray(xmlString, "demo", PersonName.ARRAY_MAKER, "array");
        String nameString2 = toString(names2);
        System.out.println(nameString2);
        printTestPartSeparator();
        boolean equals = nameString2.equals(nameString);
        System.out.println("equals: " + equals);
        if (!equals)
        {
            throw new UnitTestError(PersonNameDemo.class);
        }
        printEndTest();
    }
    
    public static void
    demoSingleLineArrayIdXmlString()
    {
        System.out.println("OneLineXml with string");
        PersonName[] names = makeSmallArray();
        String nameString = toString(names);
        System.out.println(nameString);
        printTestPartSeparator();
        XmlNameSpace nameSpace = new XmlNameSpace("x", "http://rasmustorkel.com/ns1/");
        TagNodeId arrayId = new TagNodeId("array", nameSpace);
        String xmlString = XmlSink.arrayToString(arrayId, names);
        System.out.println(xmlString);
        printTestPartSeparator();
        PersonName[] names2 = PersonName.FROM_XML_FACTORY.stringToArray(xmlString, "demo", PersonName.ARRAY_MAKER, arrayId);
        String nameString2 = toString(names2);
        System.out.println(nameString2);
        printTestPartSeparator();
        boolean equals = nameString2.equals(nameString);
        System.out.println("equals: " + equals);
        if (!equals)
        {
            throw new UnitTestError(PersonNameDemo.class);
        }
        printEndTest();
    }
    
    public static void
    demoSingleLineArrayNameXmlFile()
    {
        File file = new File("./data/rasmus_torkel/test/xml_basic/oo/personNames_n.xml");
        System.out.println("OneLineXml with file " + file);
        PersonName[] names = makeSmallArray();
        String nameString = toString(names);
        System.out.println(nameString);
        printTestPartSeparator();
        XmlSink.arrayToFile("array", names, file);
        System.out.println("Wrote array");
        printTestPartSeparator();
        PersonName[] names2 = PersonName.FROM_XML_FACTORY.fileToArray(file, PersonName.ARRAY_MAKER, "array");
        String nameString2 = toString(names2);
        System.out.println(nameString2);
        printTestPartSeparator();
        boolean equals = nameString2.equals(nameString);
        System.out.println("equals: " + equals);
        if (!equals)
        {
            throw new UnitTestError(PersonNameDemo.class);
        }
        printEndTest();
    }
    
    public static void
    demoSingleLineArrayIdXmlFile()
    {
        File file = new File("./data/rasmus_torkel/test/xml_basic/oo/personNames_i.xml");
        System.out.println("OneLineXml with file " + file);
        PersonName[] names = makeSmallArray();
        String nameString = toString(names);
        System.out.println(nameString);
        printTestPartSeparator();
        XmlNameSpace nameSpace = new XmlNameSpace("a", "ns_a");
        TagNodeId arrayId = new TagNodeId("array", nameSpace);
        XmlSink.arrayToFile(arrayId, names, file);
        System.out.println("Wrote array");
        printTestPartSeparator();
        PersonName[] names2 = PersonName.FROM_XML_FACTORY.fileToArray(file, PersonName.ARRAY_MAKER, arrayId);
        String nameString2 = toString(names2);
        System.out.println(nameString2);
        printTestPartSeparator();
        boolean equals = nameString2.equals(nameString);
        System.out.println("equals: " + equals);
        if (!equals)
        {
            throw new UnitTestError(PersonNameDemo.class);
        }
        printEndTest();
    }
    
    private static PersonName[]
    makeSmallArray()
    {
        PersonName[] names = new PersonName[]
        {
                new PersonName("Alexander", "Armstrong"),
                new PersonName("Bianca", "Bella", "Brown"),
        };
        return names;
    }
    
    private static String
    toString(PersonName[] names)
    {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < names.length; i++)
        {
            if (i > 0)
            {
                buf.append(", ");
            }
            buf.append(names[i]);
        }
        return buf.toString();
    }
}
