package rasmus_torkel.test.xml_basic.oo.inheritance;

import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.text.read.exception.WrongNodeException;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.factory.ChoiceXmlObjectFactory;
import rasmus_torkel.xml_basic.read.factory.MainXmlObjectFactory;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.MiscXmlFunctions;
import rasmus_torkel.xml_basic.write.XmlSink;
import rasmus_torkel.xml_basic.write.XmlSinkWritable;

public class InheritanceChoiceTest extends UnitTest
{
    @SuppressWarnings("unchecked")
    public static void
    main(String[] args)
    {
        announceTestClassStart(InheritanceChoiceTest.class);
        Aa aa1 = new Aa(1001, 11001);
        Ab ab1 = new Ab(1002, 12002);
        Ac ac1 = new Ac(1003, 13003);
        Aa aa2 = new Aa(1004, 11004);
        B  b1  = new B(2005);
        X  x1  = new X("xenia", 9005);
        X  x2  = new X("xerxes", 9006);
        
        A[] as = new A[]{aa1, ab1, ac1, aa2};
        XmlSinkWritable[] objects = new XmlSinkWritable[]{aa1, b1, x1, x2};
        
        ChoiceXmlObjectFactory.Option optionAa = new ChoiceXmlObjectFactory.Option(Aa.FROM_XML_FACTORY);
        ChoiceXmlObjectFactory.Option optionB = new ChoiceXmlObjectFactory.Option(B.FROM_XML_FACTORY);
        ChoiceXmlObjectFactory.Option optionX1 = new ChoiceXmlObjectFactory.Option(x1._tagNodeId, X.FROM_XML_FACTORY);
        ChoiceXmlObjectFactory.Option optionX2 = new ChoiceXmlObjectFactory.Option(x2._tagNodeId, X.FROM_XML_FACTORY);
        
        ChoiceXmlObjectFactory<A> a_aa_ab =
                new ChoiceXmlObjectFactory<A>(
                        A.class,
                        new MainXmlObjectFactory[]{Aa.FROM_XML_FACTORY, Ab.FROM_XML_FACTORY});
        ChoiceXmlObjectFactory<A> a_aa_ab_ac =
                new ChoiceXmlObjectFactory<A>(
                        A.class,
                        new MainXmlObjectFactory[]{Aa.FROM_XML_FACTORY, Ab.FROM_XML_FACTORY, Ac.FROM_XML_FACTORY});
        ChoiceXmlObjectFactory<XmlSinkWritable> xmlSupporting_mixed =
                new ChoiceXmlObjectFactory<XmlSinkWritable>(
                        XmlSinkWritable.class,
                        new ChoiceXmlObjectFactory.Option[]{optionAa, optionB, optionX1, optionX2});
        try
        {
            ChoiceXmlObjectFactory.Option[] options = {optionAa, optionB};
            System.out.println("Attempting creation of choice factory for " + A.class.getName());
            System.out.println(XmlSink.arrayToString("options", options));
            new ChoiceXmlObjectFactory<A>(A.class, options);
            System.out.println("Created factory, expected exception");
            throw new UnitTestError(InheritanceChoiceTest.class);
        }
        catch (Exception e)
        {
            System.out.println("Got exception as expected " + e);
            printEndTest();
        }
        
        cycleTest((ChoiceXmlObjectFactory<XmlSinkWritable>)((ChoiceXmlObjectFactory<?>)a_aa_ab_ac), as);
        cycleTest(xmlSupporting_mixed, objects);
        cycleTestReadErr((ChoiceXmlObjectFactory<XmlSinkWritable>)((ChoiceXmlObjectFactory<?>)a_aa_ab), as);
        announceTestClassPass(InheritanceChoiceTest.class);
    }
    
    public static void
    cycleTest(ChoiceXmlObjectFactory<XmlSinkWritable> factory,
              XmlSinkWritable[]                       objects0)
    {
        System.out.println(XmlSink.objectToString(factory));
        printTestPartSeparator();
        String xml1 = objectsToXml(objects0);
        XmlSinkWritable[] objects1 = xmlToObjects(factory, xml1);
        String xml2 = objectsToXml(objects1);
        boolean equals = xml2.equals(xml1);
        System.out.println("equals: " + equals);
        if (!equals)
        {
            throw new UnitTestError(InheritanceChoiceTest.class);
        }
        printEndTest();
    }
    
    public static String
    objectsToXml(XmlSinkWritable[] objects)
    {
        String xml = XmlSink.arrayToString("elements", objects);
        System.out.println(xml);
        printTestPartSeparator();
        MiscXmlFunctions.showXmlForHtml(xml);
        return xml;
    }
    
    public static XmlSinkWritable[]
    xmlToObjects(ChoiceXmlObjectFactory<XmlSinkWritable> factory,
                 String                                  xmlString)
    {
        //TagNode rootNode =
        //        XmlReader.xmlStringToRoot(xml, "unit test", XmlReadOptions.DEFAULT, "elements");
        //XmlSinkWritable[] objects2 = factory.extractArrayFromNode(rootNode, XmlSinkWritable.ARRAY_MAKER);
        XmlSinkWritable[] objects2 = factory.stringToArray(xmlString, "demo", XmlSinkWritable.ARRAY_MAKER, "elements");
        System.out.println("loaded objects");
        printTestPartSeparator();
        return objects2;
    }
    
    public static void
    cycleTestReadErr(ChoiceXmlObjectFactory<XmlSinkWritable> factory,
                     XmlSinkWritable[]                       objects0)
    {
        System.out.println(XmlSink.objectToString(factory));
        printTestPartSeparator();
        String xml1 = objectsToXml(objects0);
        xmlToObjectsErr(factory, xml1);
        printEndTest();
    }
    
    public static void
    xmlToObjectsErr(ChoiceXmlObjectFactory<XmlSinkWritable> factory,
                    String                                  xml)
    {
        try
        {
            TagNode rootNode =
                    XmlReader.xmlStringToRoot(xml, "unit test", XmlReadOptions.DEFAULT, "elements");
            factory.extractArrayFromNode(rootNode, XmlSinkWritable.ARRAY_MAKER);
            System.out.println("extractArrayFromNode unexpectedly returned");
            throw new UnitTestError(InheritanceChoiceTest.class);
        }
        catch (WrongNodeException e)
        {
            System.out.println("Got exception as expected: " + e);
        }
    }
}
