package rasmus_torkel.test.xml_basic.oo.inheritance.class_of_object;

import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.factory.MainXmlObjectFactory;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;
import rasmus_torkel.xml_basic.write.XmlSink;
import rasmus_torkel.xml_basic.write.XmlSinkWritable;

public class Company implements XmlSinkWritable
{
    public final String     _name;
    public final Employee[] _employees;
    
    public static final TagNodeId NATURAL_TAG_NODE_ID = new TagNodeId("company");
    public static final MainXmlObjectFactory<Company> FROM_XML_FACTORY = new MainXmlObjectFactory<Company>(NATURAL_TAG_NODE_ID)
    {
        @Override
        public Company
        extractFromNode(TagNode node)
        {
            return new Company(node);
        }
    };
    
    public
    Company(TagNode node)
    {
        _name = node.nextTextFieldE("name");
        _employees = Employee.FROM_XML_FACTORY.nextArrayFromParentE(node, Employee.NATURAL_ARRAY_TAG_NODE_ID, Employee.ARRAY_MAKER);
    }
    
    public
    Company(String     name,
            Employee[] employees)
    {
        
        _name = name;
        _employees = employees;
    }

    @Override
    public TagNodeId
    naturalTagNodeId()
    {
        return NATURAL_TAG_NODE_ID;
    }

    @Override
    public void
    mostToXml(XmlSink      xmlSink,
              String       relativeName,
              XmlNameSpace nameSpace)
    {
        xmlSink.sinkClassOfObjectAttribute(this);
        xmlSink.sinkSimpleNode("name", _name);
        xmlSink.sinkArrayNode(Employee.NATURAL_ARRAY_TAG_NODE_ID, _employees);
    }
}
