package rasmus_torkel.test.xml_basic.oo.inheritance.class_of_object;

import rasmus_torkel.misc.ArrayMaker;
import rasmus_torkel.test.xml_basic.oo.PersonName;
import rasmus_torkel.text.type.factory.date.Date2;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.factory.MainXmlObjectFactory;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;
import rasmus_torkel.xml_basic.types.date.YmdHyphenPaddedDateXmlFactory;
import rasmus_torkel.xml_basic.write.XmlSink;
import rasmus_torkel.xml_basic.write.XmlSinkWritable;

public class Employee implements XmlSinkWritable
{

    public final long       _id;
    public final long       _bossId;
    public final PersonName _name;
    public final Date2      _birthDate;
    public final Date2      _joinDate;
    public final int        _salary;
    public final Gender     _gender;
    
    public static final TagNodeId NATURAL_TAG_NODE_ID = new TagNodeId("employee");
    public static final TagNodeId NATURAL_ARRAY_TAG_NODE_ID = new TagNodeId("employees");
    
    public static final MainXmlObjectFactory<Employee> FROM_XML_FACTORY = new MainXmlObjectFactory<Employee>(NATURAL_TAG_NODE_ID)
    {
        @Override
        public Employee extractFromNode(TagNode node)
        {
            return new Employee(node);
        }
    };
    
    public static ArrayMaker<Employee> ARRAY_MAKER = new ArrayMaker<Employee>()
    {
        @Override
        public Employee[]
        newArray(int size)
        {
            return new Employee[size];
        }
    };
    
    public
    Employee(TagNode node)
    {
        this(node, true);
    }
    
    protected
    Employee(TagNode node,
             boolean isBossIdExpected)
    {
        _id = node.nextLongFieldE("id");
        _bossId = bossIdFromNode(node, isBossIdExpected);
        _name = PersonName.FROM_XML_FACTORY.nextOnParentE(node, "name");
        _birthDate = YmdHyphenPaddedDateXmlFactory.INSTANCE.nextFieldE(node, "birthDate");
        _joinDate = YmdHyphenPaddedDateXmlFactory.INSTANCE.nextFieldE(node, "joinDate");
        _salary = node.nextIntFieldE("salary");
        _gender = (Gender)node.nextEnumFieldE("gender", Gender.class);
    }
    
    public
    Employee(long       id,
             long       bossId,
             PersonName name,
             Date2      birthDate,
             Date2      joinDate,
             int        salary,
             Gender     gender)
    {
        _id = id;
        _bossId = bossId;
        _name = name;
        _birthDate = birthDate;
        _joinDate = joinDate;
        _salary = salary;
        _gender = gender;
    }
    
    private static long
    bossIdFromNode(TagNode node,
                   boolean isBossIdExpected)
    {
        if (isBossIdExpected)
        {
            return node.nextLongFieldE("bossId");
        }
        else
        {
            return -1;
        }
    }
    
    public String
    toString()
    {
        return _id + "/" + _name;
    }

    @Override
    public TagNodeId naturalTagNodeId()
    {
        return NATURAL_TAG_NODE_ID;
    }
    
    @Override
    public void
    mostToXml(XmlSink      xmlSink,
              String       relativeName,
              XmlNameSpace nameSpace)
    {
        xmlSink.sinkClassOfObjectAttribute(this);
        xmlSink.sinkSimpleNode("id", _id);
        if (_bossId != -1)
        {
            xmlSink.sinkSimpleNode("bossId", _bossId);
        }
        xmlSink.sinkNode("name", _name);
        xmlSink.sinkSimpleNode("birthDate", _birthDate);
        xmlSink.sinkSimpleNode("joinDate", _joinDate);
        xmlSink.sinkSimpleNode("salary", _salary);
        xmlSink.sinkSimpleNode("gender", _gender);
    }
}
