package rasmus_torkel.test.xml_basic.oo.inheritance.class_of_object;

import rasmus_torkel.test.xml_basic.oo.PersonName;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.text.type.factory.date.Date2;
import rasmus_torkel.text.type.factory.date.YmdHyphenPaddedDateFactory;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.MiscXmlFunctions;
import rasmus_torkel.xml_basic.write.XmlSink;

public class EmployeeDemo
{
    public static Employee[]
    makeEmployeeSample()
    {
        Ceo ceo = new Ceo(
                1001,
                new PersonName("Alexander", "Armstrong"),
                new Date2(1961, 1, 11, YmdHyphenPaddedDateFactory.INSTANCE),
                new Date2(1991, 7, 12, YmdHyphenPaddedDateFactory.INSTANCE),
                105000,
                Gender.MALE,
                new Date2(1991, 7, 12, YmdHyphenPaddedDateFactory.INSTANCE),
                new Date2(1995, 7, 13, YmdHyphenPaddedDateFactory.INSTANCE));
        Manager manager1 = new Manager(
                1203,
                ceo._id,
                new PersonName("Bianca", "Bella", "Brown"),
                new Date2(1966, 2, 21, YmdHyphenPaddedDateFactory.INSTANCE),
                new Date2(1993, 8, 23, YmdHyphenPaddedDateFactory.INSTANCE),
                80000,
                Gender.FEMALE,
                new Date2(1993, 8, 23, YmdHyphenPaddedDateFactory.INSTANCE));
        Employee employee11 = new Employee(
                1203,
                manager1._id,
                new PersonName("Christopher", new String[]{"Carlos", "Claus"}, "Cooper"),
                new Date2(1972, 4, 4, YmdHyphenPaddedDateFactory.INSTANCE),
                new Date2(1997, 12, 1, YmdHyphenPaddedDateFactory.INSTANCE),
                100000,
                Gender.MALE);
        Employee employee12 = new Employee(
                1204,
                manager1._id,
                new PersonName("Dora", "Daisy", "Davidson"),
                new Date2(1971, 3, 26, YmdHyphenPaddedDateFactory.INSTANCE),
                new Date2(1995, 9, 27, YmdHyphenPaddedDateFactory.INSTANCE),
                100000,
                Gender.MALE);
        Employee[] employees = new Employee[]
        {
                ceo,
                manager1,
                employee11,
                employee12
        };
        return employees;
    }
    
    public static String
    employeesToXml(Employee[] employees)
    {
        XmlSink sink = new XmlSink();
        sink.sinkArrayNode("employees", employees);
        String xml = sink.toString();
        return xml;
    }
    
    public static Employee[]
    xmlToEmployees(String xml)
    {
        TagNode rootNode =
            XmlReader.xmlStringToRoot(xml, "unit test", XmlReadOptions.DEFAULT, "employees");
        Employee[] employees = Employee.FROM_XML_FACTORY.extractArrayFromNode(rootNode, Employee.ARRAY_MAKER);
        return employees;
    }
    
    public static void
    main(String[] args)
    {
        Employee[] employees0 = makeEmployeeSample();
        System.out.println("Employees to XML conversion");
        System.out.println("");
        String xml0 = employeesToXml(employees0);
        System.out.println(xml0);
        MiscXmlFunctions.showXmlForHtml(xml0);
        System.out.println("************************************************");
        System.out.println("XML to employees to XML conversion");
        System.out.println("");
        Employee[] employees1 = xmlToEmployees(xml0);
        String xml1 = employeesToXml(employees1);
        System.out.println(xml1);
        System.out.println("************************************************");
        boolean equals = xml1.equals(xml0);
        System.out.println("equals: " + equals);
        if (!equals)
        {
            throw new UnitTestError(EmployeeDemo.class);
        }
    }
}
