package rasmus_torkel.test.xml_basic.read.impl;

import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.text.read.exception.AlreadyClosedReadException;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;

public class XmlReaderCloseTest extends UnitTest
{
    private static final String XML = "<root><a>1</a><b>2</b></root>";
    private static final String SHORT_XML = "<root/>";
    
    public static void
    main(String[] args)
    {
        announceTestClassStart(XmlReaderCloseTest.class);
        notClosingAfterReadingToEnd(false);
        notClosingAfterReadingToEnd(true);
        writeAfterCloseTest();
        closeTwiceTest();
        closedAfterVerifyNoMoreChildren();
        closedAfterVerifyNoMoreChildrenOnShort();
        closedAfterReadRest();
        announceTestClassPass(XmlReaderCloseTest.class);
    }
    
    public static void
    writeAfterCloseTest()
    {
        System.out.println(XML);
        printTestPartSeparator();
        TagNode rootNode = XmlReader.xmlStringToRoot(XML, "unit test (closing)", XmlReadOptions.DEFAULT);
        int a = rootNode.nextIntFieldE("a");
        System.out.println("a = " + a);
        rootNode.closeReader();
        testOpen("After closing reader", rootNode, false);
        printTestPartSeparator();
        try
        {
            System.out.println("Getting next field");
            int b = rootNode.nextIntFieldE("b");
            System.out.println("Unexpectedly succeeded, b = " + b);
            throw new UnitTestError();
        }
        catch (AlreadyClosedReadException e)
        {
            System.out.println("Got exception as expected");
            e.printStackTrace(System.out);
        }
        printEndTest();
    }
    
    public static void
    closeTwiceTest()
    {
        System.out.println(XML);
        printTestPartSeparator();
        TagNode rootNode = XmlReader.xmlStringToRoot(XML, "unit test (closing)", XmlReadOptions.DEFAULT);
        int a = rootNode.nextIntFieldE("a");
        System.out.println("a = " + a);
        testOpen("Before closeReader", rootNode, true);
        rootNode.closeReader();
        testOpen("After closeReader", rootNode, false);
        printTestPartSeparator();
        System.out.println("calling rootNode.closeReader again");
        rootNode.closeReader();
        testOpen("after closeReader #2", rootNode, false);
        printTestPartSeparator();
        try
        {
            System.out.println("Getting next field");
            int b = rootNode.nextIntFieldE("b");
            System.out.println("Unexpectedly succeeded, b = " + b);
            throw new UnitTestError();
        }
        catch (AlreadyClosedReadException e)
        {
            System.out.println("Got exception as expected");
            e.printStackTrace(System.out);
        }
        printEndTest();
    }
    
    public static void
    closedAfterVerifyNoMoreChildren()
    {
        System.out.println(XML);
        printTestPartSeparator();
        TagNode rootNode = XmlReader.xmlStringToRoot(XML, "unit test (closing)", XmlReadOptions.DEFAULT);
        int a = rootNode.nextIntFieldE("a");
        System.out.println("a = " + a);
        int b = rootNode.nextIntFieldE("b");
        System.out.println("b = " + b);
        testOpen("After reading b", rootNode, true);
        rootNode.verifyNoMoreChildren();
        testOpen("After verifyNoMoreChildren", rootNode, false);
        rootNode.verifyNoMoreChildren();
        testOpen("After verifyNoMoreChildren #2", rootNode, false);
    }
    
    public static void
    closedAfterVerifyNoMoreChildrenOnShort()
    {
        System.out.println("closedAfterVerifyNoMoreChildrenOnShort");
        printTestPartSeparator();
        System.out.println(SHORT_XML);
        printTestPartSeparator();
        TagNode rootNode = XmlReader.xmlStringToRoot(SHORT_XML, "unit test (closing)", XmlReadOptions.DEFAULT);
        testOpen("After getting root node", rootNode, false);
        rootNode.verifyNoMoreChildren();
        testOpen("After verifyNoMoreChildren", rootNode, false);
        rootNode.verifyNoMoreChildren();
        testOpen("After verifyNoMoreChildren #2", rootNode, false);
        printEndTest();
    }
    
    public static void
    notClosingAfterReadingToEnd(boolean withTrailingSpaces)
    {
        String xml;
        if (withTrailingSpaces)
        {
            xml = XML + "                             ";
            System.out.println("We have trailing spaces");
        }
        else
        {
            xml = XML;
        }
        System.out.println(xml);
        printTestPartSeparator();
        TagNode rootNode = XmlReader.xmlStringToRoot(xml, "unit test (closing)", XmlReadOptions.DEFAULT);
        int a = rootNode.nextIntFieldE("a");
        System.out.println("a = " + a);
        int b = rootNode.nextIntFieldE("b");
        System.out.println("b = " + b);
        testOpen("After reading b", rootNode, true);
        TagNode nonNode = rootNode.nextChildN();
        System.out.println("nonNode is " + (nonNode == null ? "null" : nonNode.toString()));
        if (nonNode != null)
        {
            System.out.println("Expected null");
            throw new UnitTestError();
        }
        testOpen("After reading null node", rootNode, false);
        rootNode.closeReader();
        testOpen("After closeReader", rootNode, false);
        printEndTest();
    }
    
    public static void
    closedAfterReadRest()
    {
        System.out.println(XML);
        printTestPartSeparator();
        TagNode rootNode = XmlReader.xmlStringToRoot(XML, "unit test (closing)", XmlReadOptions.DEFAULT);
        testOpen("After getting root node", rootNode, true);
        rootNode.readRest();
        testOpen("After readRest", rootNode, false);
        printEndTest();
    }
    
    private static void
    testOpen(String  situation,
             TagNode node,
             boolean expectOpen)
    {
        boolean isOpen = node.isReaderOpen();
        System.out.println(situation + ", reader is " + (isOpen ? "open" : "closed"));
        if (expectOpen != isOpen)
        {
            System.out.println("expect " + (expectOpen ? "open" : "closed"));
            throw new UnitTestError();
        }
    }
}
