package rasmus_torkel.test.xml_basic.read.impl;

import rasmus_torkel.set.chars.DefinedCharSets;
import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.text.write.LineEndOption;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.exception.XmlCharSetException;
import rasmus_torkel.xml_basic.read.exception.XmlSyntaxException;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.MiscXmlFunctions;

public class XmlReaderSuiteChars extends UnitTest
{
    public static void
    main(String[] args)
    {
        announceTestClassStart(XmlReaderSuiteChars.class);
        XmlReadOptions optionsLower = XmlReadOptions.DEFAULT.diffCharSet(DefinedCharSets.LATIN_LOWER_CASE_LETTERS);
        XmlReadOptions optionsNamesLower = XmlReadOptions.DEFAULT.diffNameCharSet(DefinedCharSets.LATIN_LOWER_CASE_LETTERS);
        XmlReadOptions optionsValuesLower = XmlReadOptions.DEFAULT.diffValueCharSet(DefinedCharSets.LATIN_LOWER_CASE_LETTERS);
        String xmlLower = "<root aname=\"avalue\">text</root>";
        String xml1Root = "<1Root aname=\"avalue\">text</1Root>";
        String xmlRootBrackets = "<root[] aname=\"avalue\">text</root[]>";
        String xmlRoot1Upper = "<Root aname=\"avalue\">text</Root>";
        String xmlRootLastUpper = "<rooT aname=\"avalue\">text</rooT>";
        String xml1AttrName = "<root 1aname=\"avalue\">text</root>";
        String xmlAttrNameBrackets = "<root aname[]=\"avalue\">text</root>";
        String xmlAttrName1Upper = "<root Aname=\"avalue\">text</root>";
        String xmlAttrNameLastUpper = "<root anameE=\"avalue\">text</root>";
        String xmlAttrValue1Upper = "<root aname=\"Avalue\">text</root>";
        String xmlAttrValueLastUpper = "<root aname=\"avaluE\">text</root>";
        String xmlValue1Upper = "<root aname=\"avalue\">Text</root>";
        String xmlValueLastUpper = "<root aname=\"avalue\">texT</root>";
        String xmlValueControl = "<root aname=\"avalue\">control&#xA;</root>";
        String xmlUpperPrefix = "<X:root aname=\"avalue\" xmlns:X=\"ns\">text</X:root>";
        String xmlRelRoot1Upper = "<x:Root aname=\"avalue\" xmlns:x=\"ns\">text</x:Root>";
        String xml1Prefix = "<1X:root aname=\"avalue\" xmlns:1X=\"ns\">text</1X:root>";
        String xml1RelRoot = "<x:1Root aname=\"avalue\" xmlns:x=\"ns\">text</x:1Root>";
        String xmlUpperAttrPrefix = "<root X:aname=\"avalue\" xmlns:X=\"ns\">text</root>";
        String xmlRelAttr1Upper = "<root x:Aname=\"avalue\" xmlns:x=\"ns\">text</root>";
        String xml1AttrPrefix = "<root 1X:aname=\"avalue\" xmlns:1X=\"ns\">text</root>";
        String xml1RelAttr = "<root x:1Aname=\"avalue\" xmlns:x=\"ns\">text</root>";
        
        readXmlOk(xmlLower, XmlReadOptions.DEFAULT);
        
        readXmlOk(xmlRoot1Upper, XmlReadOptions.DEFAULT);
        readXmlErr(xml1Root, XmlReadOptions.DEFAULT, XmlSyntaxException.class);
        readXmlErr(xml1Root, optionsLower, XmlSyntaxException.class);
        readXmlErr(xmlRootBrackets, XmlReadOptions.DEFAULT, XmlSyntaxException.class);
        readXmlErr(xmlRootBrackets, optionsLower, XmlSyntaxException.class);
        readXmlErr(xmlRoot1Upper, optionsLower, XmlCharSetException.class);
        readXmlErr(xmlRootLastUpper, optionsLower, XmlCharSetException.class);
        readXmlErr(xmlRoot1Upper, optionsNamesLower, XmlCharSetException.class);
        readXmlOk(xmlRoot1Upper, optionsValuesLower);
        
        readXmlOk(xmlAttrName1Upper, XmlReadOptions.DEFAULT);
        readXmlErr(xml1AttrName, XmlReadOptions.DEFAULT, XmlSyntaxException.class);
        readXmlErr(xml1AttrName, optionsLower, XmlSyntaxException.class);
        readXmlErr(xmlAttrNameBrackets, XmlReadOptions.DEFAULT, XmlSyntaxException.class);
        readXmlErr(xmlAttrNameBrackets, optionsLower, XmlSyntaxException.class);
        readXmlErr(xmlAttrName1Upper, optionsLower, XmlCharSetException.class);
        readXmlErr(xmlAttrNameLastUpper, optionsLower, XmlCharSetException.class);

        readXmlOk(xmlAttrValue1Upper, XmlReadOptions.DEFAULT);
        readXmlErr(xmlAttrValue1Upper, optionsLower, XmlCharSetException.class);
        readXmlErr(xmlAttrValueLastUpper, optionsLower, XmlCharSetException.class);

        readXmlOk(xmlValue1Upper, XmlReadOptions.DEFAULT);
        readXmlErr(xmlValue1Upper, optionsLower, XmlCharSetException.class);
        readXmlErr(xmlValueLastUpper, optionsLower, XmlCharSetException.class);
        readXmlErr(xmlValueControl, optionsLower, XmlCharSetException.class);
        
        readXmlOk(xmlUpperPrefix, XmlReadOptions.DEFAULT);
        readXmlErr(xmlUpperPrefix, optionsLower, XmlCharSetException.class);
        readXmlOk(xmlRelRoot1Upper, XmlReadOptions.DEFAULT);
        readXmlErr(xmlRelRoot1Upper, optionsLower, XmlCharSetException.class);
        readXmlErr(xml1Prefix, optionsLower, XmlSyntaxException.class);
        readXmlErr(xml1RelRoot, optionsLower, XmlSyntaxException.class);
        
        readXmlOk(xmlUpperAttrPrefix, XmlReadOptions.DEFAULT);
        readXmlErr(xmlUpperAttrPrefix, optionsLower, XmlCharSetException.class);
        readXmlOk(xmlRelAttr1Upper, XmlReadOptions.DEFAULT);
        readXmlErr(xmlRelRoot1Upper, optionsLower, XmlCharSetException.class);
        readXmlErr(xml1AttrPrefix, optionsLower, XmlSyntaxException.class);
        readXmlErr(xml1RelAttr, optionsLower, XmlSyntaxException.class);
        
        announceTestClassPass(XmlReaderSuiteChars.class);
    }
    
    public static void
    readXmlOk(String         xmlString,
              XmlReadOptions readOptions)
    {
        try
        {
            readXml(xmlString, readOptions);
            printEndTest();
        }
        catch (RuntimeException e)
        {
            e.printStackTrace(System.out);
            System.out.println("Exception was unexpected");
            throw new UnitTestError(XmlReaderSuiteChars.class);
        }
    }
    
    public static void
    readXmlErr(String         xmlString,
               XmlReadOptions readOptions,
               Class<?>       exceptionClass)
    {
        try
        {
            readXml(xmlString, readOptions);
            System.out.println("Expected exception");
            throw new UnitTestError(XmlReaderSuiteChars.class);
        }
        catch (RuntimeException e)
        {
            if (!e.getClass().equals(exceptionClass))
            {
                e.printStackTrace(System.out);
                System.out.println("Expected exception in class " + exceptionClass.getName());
                throw new UnitTestError(XmlReaderSuiteChars.class);
            }
            System.out.println(e);
            printEndTest();
        }
    }
    
    public static void
    readXml(String         xmlString,
            XmlReadOptions readOptions)
    {
        MiscXmlFunctions.showXmlForHtml(xmlString + LineEndOption.SYSTEM_PROPERTY._lineEndStr);
        System.out.println(xmlString);
        printRuler(100);
        System.out.println(readOptions);
        TagNode node = XmlReader.xmlStringToRoot(xmlString, "Unit Test", readOptions);
        System.out.println("nodeName = " + node._id._relativeName);
        System.out.println("attributeName = " + node.attribute(0)._name);
        System.out.println("attributeValue = " + node.attribute(0)._value);
        System.out.println("value = " + node.onlyText());
    }
}
