/*
 * Copyright Rasmus Torkel 2012
 * Author    Rasmus Torkel 2012
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.xml_basic.read.impl;

import rasmus_torkel.set.chars.DefinedCharSets;
import rasmus_torkel.set.doubles.DoubleRange;
import rasmus_torkel.set.floats.FloatRange;
import rasmus_torkel.set.ints.IntRange;
import rasmus_torkel.set.longs.LongRange;
import rasmus_torkel.text.read.exception.MissingNodeException;
import rasmus_torkel.text.read.exception.SuppliedClassNotEnumException;
import rasmus_torkel.text.read.exception.SuppliedClassNotStringMappedException;
import rasmus_torkel.text.read.exception.WrongNodeException;
import rasmus_torkel.text.type.strings.Ip4Address;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.exception.XmlAttributeNotInSetException;
import rasmus_torkel.xml_basic.read.exception.XmlMissingAttributeException;
import rasmus_torkel.xml_basic.read.exception.XmlTextFieldDoesNotMatchTypeException;
import rasmus_torkel.xml_basic.read.exception.XmlTextFieldNotInSetException;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;

public class XmlReaderSuiteTypes extends XmlReaderSuite
{
    public static final String N_MIXED_DATA = "mixedData";
    public static final String N_STRING = "s";
    public static final String N_INT = "i";
    public static final String N_LONG = "l";
    public static final String N_BOOL = "b";
    public static final String N_FLOAT = "f";
    public static final String N_DOUBLE = "d";
    public static final String N_ENUM = "e";
    public static final String N_STRING_MAPPED = "m";
    public static final String N_CONST = "c";
    
    public static final String[] FIELD_NAMES = { N_STRING, N_INT, N_LONG, N_BOOL, N_FLOAT, N_DOUBLE, N_ENUM, N_STRING_MAPPED, N_CONST };
    
    public static final String  V_STRING = "ss";
    public static final int     V_INT =    (int)Short.MAX_VALUE + 1;
    public static final long    V_LONG =   (long)Integer.MAX_VALUE + 1;
    public static final boolean V_BOOL =   true;
    public static final float   V_FLOAT =  1.5f;
    public static final double     V_DOUBLE = 4.5E38;
    public static final Colour     V_ENUM =   Colour.Red;
    public static final Ip4Address V_STRING_MAPPED = new Ip4Address("101.102.103.104");
    public static final String     V_CONST =  "cc";
    
    public static final MixedData MIXED_DATA_STANDARD = new MixedData(V_STRING, V_INT, V_LONG, V_BOOL, V_FLOAT, V_DOUBLE, V_ENUM, V_STRING_MAPPED, V_CONST);
    public static final MixedData MIXED_DATA_EMPTY = new MixedData(null, null, null, null, null, null, null, null, null);
    public static final MixedData MIXED_DATA_NO_S = new MixedData(null, V_INT, V_LONG, V_BOOL, V_FLOAT, V_DOUBLE, V_ENUM, V_STRING_MAPPED, V_CONST);
    public static final MixedData MIXED_DATA_NO_I = new MixedData(V_STRING, null, V_LONG, V_BOOL, V_FLOAT, V_DOUBLE, V_ENUM, V_STRING_MAPPED, V_CONST);
    public static final MixedData MIXED_DATA_NO_L = new MixedData(V_STRING, V_INT, null, V_BOOL, V_FLOAT, V_DOUBLE, V_ENUM, V_STRING_MAPPED, V_CONST);
    public static final MixedData MIXED_DATA_NO_B = new MixedData(V_STRING, V_INT, V_LONG, null, V_FLOAT, V_DOUBLE, V_ENUM, V_STRING_MAPPED, V_CONST);
    public static final MixedData MIXED_DATA_NO_F = new MixedData(V_STRING, V_INT, V_LONG, V_BOOL, null, V_DOUBLE, V_ENUM, V_STRING_MAPPED, V_CONST);
    public static final MixedData MIXED_DATA_NO_D = new MixedData(V_STRING, V_INT, V_LONG, V_BOOL, V_FLOAT, null, V_ENUM, V_STRING_MAPPED, V_CONST);
    public static final MixedData MIXED_DATA_NO_E = new MixedData(V_STRING, V_INT, V_LONG, V_BOOL, V_FLOAT, V_DOUBLE, null, V_STRING_MAPPED, V_CONST);
    public static final MixedData MIXED_DATA_NO_M = new MixedData(V_STRING, V_INT, V_LONG, V_BOOL, V_FLOAT, V_DOUBLE, V_ENUM, null, V_CONST);
    public static final MixedData MIXED_DATA_NO_C = new MixedData(V_STRING, V_INT, V_LONG, V_BOOL, V_FLOAT, V_DOUBLE, V_ENUM, V_STRING_MAPPED, null);

    public static final int       V_INT_2      = (int)Short.MIN_VALUE - 1;
    public static final long      V_LONG_2     = (long)Integer.MIN_VALUE - 1;
    public static final float     V_FLOAT_2    = -V_FLOAT;
    public static final double    V_DOUBLE_2   = -V_DOUBLE;
    public static final Colour    V_ENUM_2     = Colour.Blue;
    public static final boolean   V_BOOL_2     = false;
    public static final MixedData MIXED_DATA_2 = new MixedData(V_STRING, V_INT_2, V_LONG_2, V_BOOL_2, V_FLOAT_2, V_DOUBLE_2, V_ENUM_2, V_STRING_MAPPED, V_CONST);
    
    public static final IntRange    R_INT    = new IntRange(V_INT_2, V_INT);
    public static final LongRange   R_LONG   = new LongRange(V_LONG_2, V_LONG);
    public static final FloatRange  R_FLOAT  = new FloatRange(V_FLOAT_2, V_FLOAT);
    public static final DoubleRange R_DOUBLE = new DoubleRange(V_DOUBLE_2, V_DOUBLE);
    
    public static final String MIXED_DATA_XML_OK =
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>";
    
    public static final String[] MIXED_DATA_XML_BAD_TYPE_SET =
    {
        "<mixedData><s>ss</s><i>2147483648</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>32768</i><l>xxx</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>yes</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>xxx</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><d>xxx</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.333</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>xx</c></mixedData>"
    };
    
    public static final String[] MIXED_DATA_XML_OUTSIDE_SET_SET =
    {
        "<mixedData><s>ss</s><i>32769</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>-40000</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>32768</i><l>2147483649</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>32768</i><l>-2147483650</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>1.6</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.6E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
        "<mixedData><s>ss:</s><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>",
    };
    
    public static final String MIXED_DATA_XML_OK_2 =
        "<mixedData><s>ss</s><i>-32769</i><l>-2147483649</l><b>false</b><f>-1.5</f><d>-4.5E38</d><e>Blue</e><m>101.102.103.104</m><c>cc</c></mixedData>";
    
    public static final String MIXED_DATA_XML_EMPTY =
        "<mixedData></mixedData>";
    
    public static final String MIXED_DATA_XML_NO_S =
        "<mixedData><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>";
    
    public static final String MIXED_DATA_XML_NO_I =
        "<mixedData><s>ss</s><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>";
    
    public static final String MIXED_DATA_XML_NO_L =
        "<mixedData><s>ss</s><i>32768</i><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>";
    
    public static final String MIXED_DATA_XML_NO_B =
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>";
    
    public static final String MIXED_DATA_XML_NO_F =
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><d>4.5E38</d><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>";
    
    public static final String MIXED_DATA_XML_NO_D =
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><e>Red</e><m>101.102.103.104</m><c>cc</c></mixedData>";
    
    public static final String MIXED_DATA_XML_NO_E =
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><m>101.102.103.104</m><c>cc</c></mixedData>";
    
    public static final String MIXED_DATA_XML_NO_M =
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><c>cc</c></mixedData>";
    
    public static final String MIXED_DATA_XML_NO_C =
        "<mixedData><s>ss</s><i>32768</i><l>2147483648</l><b>true</b><f>1.5</f><d>4.5E38</d><e>Red</e><m>101.102.103.104</m></mixedData>";
    
    public static final String[] MIXED_DATA_SOMETHING_MISSING_SET =
    {
        MIXED_DATA_XML_EMPTY,
        MIXED_DATA_XML_NO_S,
        MIXED_DATA_XML_NO_I,
        MIXED_DATA_XML_NO_L,
        MIXED_DATA_XML_NO_B,
        MIXED_DATA_XML_NO_F,
        MIXED_DATA_XML_NO_D,
        MIXED_DATA_XML_NO_E,
        MIXED_DATA_XML_NO_M,
        MIXED_DATA_XML_NO_C,
    };
    
    public static final String MIXED_DATA_XML_REVERSE =
        "<mixedData><c>cc</c><m>101.102.103.104</m><e>Red</e><d>4.5E38</d><f>1.5</f><b>true</b><l>2147483648</l><i>32768</i><s>ss</s></mixedData>";
    
    public static final String[] MIXED_DATA_XML_OK_ANY_ORDER_SET = { MIXED_DATA_XML_OK, MIXED_DATA_XML_REVERSE };
    
    public static final String MIXED_DATA_XML_NS_OK =
        "<x:mixedData xmlns:x=\"NS1\"><x:s>ss</x:s><x:i>32768</x:i><x:l>2147483648</x:l><x:b>true</x:b><x:f>1.5</x:f><x:d>4.5E38</x:d><x:e>Red</x:e><x:m>101.102.103.104</x:m><x:c>cc</x:c></x:mixedData>";
    
    public static final String MIXED_DATA_XML_NS_REVERSE =
        "<x:mixedData xmlns:x=\"NS1\"><x:c>cc</x:c><x:m>101.102.103.104</x:m><x:e>Red</x:e><x:d>4.5E38</x:d><x:f>1.5</x:f><x:b>true</x:b><x:l>2147483648</x:l><x:i>32768</x:i><x:s>ss</x:s></x:mixedData>";
    
    public static final String[] MIXED_DATA_XML_NS_OK_ANY_ORDER_SET = { MIXED_DATA_XML_NS_OK, MIXED_DATA_XML_NS_REVERSE };
    
    public static final String MIXED_DATA_XML_NS_EMPTY =
        "<x:mixedData xmlns:x=\"NS1\"></x:mixedData>";
    
    public static final String[] MIXED_DATA_XML_NS_OUTSIDE_SET_SET =
    {
        "<x:mixedData xmlns:x=\"NS1\"><x:s>ss</x:s><x:i>32769</x:i><x:l>2147483648</x:l><x:b>true</x:b><x:f>1.5</x:f><x:d>4.5E38</x:d><x:e>Red</x:e><x:m>101.102.103.104</x:m><x:c>cc</x:c></x:mixedData>",
        "<x:mixedData xmlns:x=\"NS1\"><x:s>ss</x:s><x:i>-40000</x:i><x:l>2147483648</x:l><x:b>true</x:b><x:f>1.5</x:f><x:d>4.5E38</x:d><x:e>Red</x:e><x:m>101.102.103.104</x:m><x:c>cc</x:c></x:mixedData>",
        "<x:mixedData xmlns:x=\"NS1\"><x:s>ss</x:s><x:i>32768</x:i><x:l>2147483649</x:l><x:b>true</x:b><x:f>1.5</x:f><x:d>4.5E38</x:d><x:e>Red</x:e><x:m>101.102.103.104</x:m><x:c>cc</x:c></x:mixedData>",
        "<x:mixedData xmlns:x=\"NS1\"><x:s>ss</x:s><x:i>32768</x:i><x:l>-2147483650</x:l><x:b>true</x:b><x:f>1.5</x:f><x:d>4.5E38</x:d><x:e>Red</x:e><x:m>101.102.103.104</x:m><x:c>cc</x:c></x:mixedData>",
        "<x:mixedData xmlns:x=\"NS1\"><x:s>ss</x:s><x:i>32768</x:i><x:l>2147483648</x:l><x:b>true</x:b><x:f>1.6</x:f><x:d>4.5E38</x:d><x:e>Red</x:e><x:m>101.102.103.104</x:m><x:c>cc</x:c></x:mixedData>",
        "<x:mixedData xmlns:x=\"NS1\"><x:s>ss</x:s><x:i>32768</x:i><x:l>2147483648</x:l><x:b>true</x:b><x:f>1.5</x:f><x:d>4.6E38</x:d><x:e>Red</x:e><x:m>101.102.103.104</x:m><x:c>cc</x:c></x:mixedData>",
        "<x:mixedData xmlns:x=\"NS1\"><x:s>ss:</x:s><x:i>32768</x:i><x:l>2147483648</x:l><x:b>true</x:b><x:f>1.5</x:f><x:d>4.5E38</x:d><x:e>Red</x:e><x:m>101.102.103.104</x:m><x:c>cc</x:c></x:mixedData>",
    };
    
    public static final String MIXED_DATA_XML_ATTR_OK =
        "<mixedData s=\"ss\" i=\"32768\" l=\"2147483648\" b=\"true\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>";
    
    public static final String MIXED_DATA_XML_ATTR_NO_S =
        "<mixedData i=\"32768\" l=\"2147483648\" b=\"true\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>";
    
    public static final String MIXED_DATA_XML_ATTR_NO_I =
        "<mixedData s=\"ss\" l=\"2147483648\" b=\"true\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>";
    
    public static final String MIXED_DATA_XML_ATTR_NO_L =
        "<mixedData s=\"ss\" i=\"32768\" b=\"true\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>";
    
    public static final String MIXED_DATA_XML_ATTR_NO_B =
        "<mixedData s=\"ss\" i=\"32768\" l=\"2147483648\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>";
    
    public static final String MIXED_DATA_XML_ATTR_NO_F =
        "<mixedData s=\"ss\" i=\"32768\" l=\"2147483648\" b=\"true\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>";
    
    public static final String MIXED_DATA_XML_ATTR_NO_D =
        "<mixedData s=\"ss\" i=\"32768\" l=\"2147483648\" b=\"true\" f=\"1.5\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>";
    
    public static final String MIXED_DATA_XML_ATTR_NO_E =
        "<mixedData s=\"ss\" i=\"32768\" l=\"2147483648\" b=\"true\" f=\"1.5\" d=\"4.5E38\" m=\"101.102.103.104\" c=\"cc\"/>";
    
    public static final String MIXED_DATA_XML_ATTR_NO_M =
        "<mixedData s=\"ss\" i=\"32768\" l=\"2147483648\" b=\"true\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" c=\"cc\"/>";
    
    public static final String MIXED_DATA_XML_ATTR_NO_C =
        "<mixedData s=\"ss\" i=\"32768\" l=\"2147483648\" b=\"true\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\"/>";
    
    public static final String[] MIXED_DATA_XML_MISSING_ATTRIBUTE_SET =
    {
        MIXED_DATA_XML_ATTR_NO_S,
        MIXED_DATA_XML_ATTR_NO_I,
        MIXED_DATA_XML_ATTR_NO_L,
        MIXED_DATA_XML_ATTR_NO_B,
        MIXED_DATA_XML_ATTR_NO_F,
        MIXED_DATA_XML_ATTR_NO_D,
        MIXED_DATA_XML_ATTR_NO_E,
        MIXED_DATA_XML_ATTR_NO_M,
        MIXED_DATA_XML_ATTR_NO_C,
        MIXED_DATA_XML_EMPTY,
    };
    
    public static final String[] MIXED_DATA_XML_ATTRIBUTE_OUTSIDE_SET_SET =
    {
        "<mixedData s=\"ss\" i=\"32769\" l=\"2147483648\" b=\"true\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>",
        "<mixedData s=\"ss\" i=\"-40000\" l=\"2147483648\" b=\"true\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>",
        "<mixedData s=\"ss\" i=\"32768\" l=\"2147483649\" b=\"true\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>",
        "<mixedData s=\"ss\" i=\"32768\" l=\"-2147483650\" b=\"true\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>",
        "<mixedData s=\"ss\" i=\"32768\" l=\"2147483648\" b=\"true\" f=\"1.6\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>",
        "<mixedData s=\"ss\" i=\"32768\" l=\"2147483648\" b=\"true\" f=\"1.5\" d=\"4.6E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>",
        "<mixedData s=\"ss*\" i=\"32768\" l=\"2147483648\" b=\"true\" f=\"1.5\" d=\"4.5E38\" e=\"Red\" m=\"101.102.103.104\" c=\"cc\"/>",
    };
    
    public static final String MIXED_DATA_XML_ATTR_NS_OK =
        "<mixedData x:s=\"ss\" x:i=\"32768\" x:l=\"2147483648\" x:b=\"true\" x:f=\"1.5\" x:d=\"4.5E38\" x:e=\"Red\" x:m=\"101.102.103.104\" x:c=\"cc\" xmlns:x=\"NS1\"/>";
    
    public static enum Colour
    {
        Red,
        Green,
        Blue
    }
    
    public static class MixedData
    {
        public final String     _s;
        public final Integer    _i;
        public final Long       _l;
        public final Boolean    _b;
        public final Float      _f;
        public final Double     _d;
        public final Colour     _e;
        public final Ip4Address _m;
        public final String     _c;
        
        public final String  _string;
        
        public
        MixedData(String     s,
                  Integer    i,
                  Long       l,
                  Boolean    b,
                  Float      f,
                  Double     d,
                  Colour     e,
                  Ip4Address m,
                  String     c)
        {
            _s = s;
            _i = i;
            _l = l;
            _b = b;
            _f = f;
            _d = d;
            _e = e;
            _m = m;
            _c = c;
            _string = N_STRING + " = " + _s + ", " +
                      N_INT + " = " + _i + ", " +
                      N_LONG + " = " + _l + ", " +
                      N_BOOL + " = " + _b + ", " +
                      N_FLOAT + " = " + _f + ", " +
                      N_DOUBLE + " = " + _d + ", " +
                      N_ENUM + " = " + _e + ", " +
                      N_STRING_MAPPED + " = " + _m + ", " +
                      N_CONST + " = " + _c;
        }
        
        @Override
        public String
        toString()
        {
            return _string;
        }
        
        @Override
        public boolean
        equals(Object other)
        {
            return ((MixedData)other)._string.equals(_string);
        }
    }
    
    private static final XmlSubTest XML1_STRICT = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.nextTextFieldE(N_STRING);
            int i = rootNode.nextIntFieldE(N_INT);
            long l = rootNode.nextLongFieldE(N_LONG);
            boolean b = rootNode.nextBoolFieldE(N_BOOL);
            float f = rootNode.nextFloatFieldE(N_FLOAT);
            double d = rootNode.nextDoubleFieldE(N_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldE(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldE(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.nextConstFieldE(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "strict1";
        }
    };
    
    private static final XmlSubTest WITH_DEFAULTS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.nextTextFieldD(N_STRING, V_STRING);
            int i = rootNode.nextIntFieldD(N_INT, V_INT);
            long l = rootNode.nextLongFieldD(N_LONG, V_LONG);
            boolean b = rootNode.nextBoolFieldD(N_BOOL, V_BOOL);
            float f = rootNode.nextFloatFieldD(N_FLOAT, V_FLOAT);
            double d = rootNode.nextDoubleFieldD(N_DOUBLE, V_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldD(N_ENUM, Colour.class, Colour.Red);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldD(N_STRING_MAPPED, Ip4Address.class, V_STRING_MAPPED);
            String c = rootNode.nextConstFieldD(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "with defaults";
        }
    };
    
    private static final XmlSubTest ALLOW_NULLS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.nextTextFieldN(N_STRING);
            Integer i = rootNode.nextIntFieldN(N_INT);
            Long l = rootNode.nextLongFieldN(N_LONG);
            Boolean b = rootNode.nextBoolFieldN(N_BOOL);
            Float f = rootNode.nextFloatFieldN(N_FLOAT);
            Double d = rootNode.nextDoubleFieldN(N_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldN(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldN(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.nextConstFieldN(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "allow nulls";
        }
    };
    
    private static final XmlSubTest ID_STRICT = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.nextTextFieldE(new TagNodeId(N_STRING, ns));
            int i = rootNode.nextIntFieldE(new TagNodeId(N_INT, ns));
            long l = rootNode.nextLongFieldE(new TagNodeId(N_LONG, ns));
            boolean b = rootNode.nextBoolFieldE(new TagNodeId(N_BOOL, ns));
            float f = rootNode.nextFloatFieldE(new TagNodeId(N_FLOAT, ns));
            double d = rootNode.nextDoubleFieldE(new TagNodeId(N_DOUBLE, ns));
            Colour e = (Colour)rootNode.nextEnumFieldE(new TagNodeId(N_ENUM, ns), Colour.class);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldE(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class);
            String c = rootNode.nextConstFieldE(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "TagNodeId, strict";
        }
    };
    
    private static final XmlSubTest ID_WITH_DEFAULTS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.nextTextFieldD(new TagNodeId(N_STRING, ns), V_STRING);
            int i = rootNode.nextIntFieldD(new TagNodeId(N_INT, ns), V_INT);
            long l = rootNode.nextLongFieldD(new TagNodeId(N_LONG, ns), V_LONG);
            boolean b = rootNode.nextBoolFieldD(new TagNodeId(N_BOOL, ns), V_BOOL);
            float f = rootNode.nextFloatFieldD(new TagNodeId(N_FLOAT, ns), V_FLOAT);
            double d = rootNode.nextDoubleFieldD(new TagNodeId(N_DOUBLE, ns), V_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldD(new TagNodeId(N_ENUM, ns), Colour.class, Colour.Red);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldD(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class, V_STRING_MAPPED);
            String c = rootNode.nextConstFieldD(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "TagNodeId, with defaults";
        }
    };
    
    private static final XmlSubTest ID_ALLOW_NULLS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.nextTextFieldN(new TagNodeId(N_STRING, ns));
            Integer i = rootNode.nextIntFieldN(new TagNodeId(N_INT, ns));
            Long l = rootNode.nextLongFieldN(new TagNodeId(N_LONG, ns));
            Boolean b = rootNode.nextBoolFieldN(new TagNodeId(N_BOOL, ns));
            Float f = rootNode.nextFloatFieldN(new TagNodeId(N_FLOAT, ns));
            Double d = rootNode.nextDoubleFieldN(new TagNodeId(N_DOUBLE, ns));
            Colour e = (Colour)rootNode.nextEnumFieldN(new TagNodeId(N_ENUM, ns), Colour.class);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldN(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class);
            String c = rootNode.nextConstFieldN(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "TagNodeId, allow nulls";
        }
    };
    
    private static final XmlSubTest BAD_ENUM_CLASS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.nextTextFieldE(N_STRING);
            int i = rootNode.nextIntFieldE(N_INT);
            long l = rootNode.nextLongFieldE(N_LONG);
            boolean b = rootNode.nextBoolFieldE(N_BOOL);
            float f = rootNode.nextFloatFieldE(N_FLOAT);
            double d = rootNode.nextDoubleFieldE(N_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldE(N_ENUM, String.class);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldE(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.nextConstFieldE(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "bad enum class";
        }
    };
    
    private static class ClassWithoutStringConstructor
    {
    }
    
    private static final XmlSubTest NO_STRING_CONSTRUCTOR_CLASS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.nextTextFieldE(N_STRING);
            int i = rootNode.nextIntFieldE(N_INT);
            long l = rootNode.nextLongFieldE(N_LONG);
            boolean b = rootNode.nextBoolFieldE(N_BOOL);
            float f = rootNode.nextFloatFieldE(N_FLOAT);
            double d = rootNode.nextDoubleFieldE(N_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldE(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldE(N_STRING_MAPPED, ClassWithoutStringConstructor.class);
            String c = rootNode.nextConstFieldE(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "no string cnstructor class";
        }
    };
    
    private static final XmlSubTest PRIVATE_STRING_CONSTRUCTOR_CLASS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.nextTextFieldE(N_STRING);
            int i = rootNode.nextIntFieldE(N_INT);
            long l = rootNode.nextLongFieldE(N_LONG);
            boolean b = rootNode.nextBoolFieldE(N_BOOL);
            float f = rootNode.nextFloatFieldE(N_FLOAT);
            double d = rootNode.nextDoubleFieldE(N_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldE(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldE(N_STRING_MAPPED, ClassWithPrivateStringConstructor.class);
            String c = rootNode.nextConstFieldE(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "private string constructor class";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_STRICT = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.anyOrderTextFieldE(N_STRING);
            int i = rootNode.anyOrderIntFieldE(N_INT);
            long l = rootNode.anyOrderLongFieldE(N_LONG);
            boolean b = rootNode.anyOrderBoolFieldE(N_BOOL);
            float f = rootNode.anyOrderFloatFieldE(N_FLOAT);
            double d = rootNode.anyOrderDoubleFieldE(N_DOUBLE);
            Colour e = (Colour)rootNode.anyOrderEnumFieldE(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldE(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.anyOrderConstFieldE(N_CONST, V_CONST);
            rootNode.verifyNoOtherChildren(N_STRING, N_INT, N_LONG, N_BOOL, N_FLOAT, N_DOUBLE, N_ENUM, N_STRING_MAPPED, N_CONST);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "any order, strict";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_WITH_DEFAULTS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.anyOrderTextFieldD(N_STRING, V_STRING);
            int i = rootNode.anyOrderIntFieldD(N_INT, V_INT);
            long l = rootNode.anyOrderLongFieldD(N_LONG, V_LONG);
            boolean b = rootNode.anyOrderBoolFieldD(N_BOOL, V_BOOL);
            float f = rootNode.anyOrderFloatFieldD(N_FLOAT, V_FLOAT);
            double d = rootNode.anyOrderDoubleFieldD(N_DOUBLE, V_DOUBLE);
            Colour e = (Colour)rootNode.anyOrderEnumFieldD(N_ENUM, Colour.class, Colour.Red);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldD(N_STRING_MAPPED, Ip4Address.class, V_STRING_MAPPED);
            String c = rootNode.anyOrderConstFieldD(N_CONST, V_CONST);
            rootNode.verifyNoOtherChildren(FIELD_NAMES);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "any order with defaults";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_ALLOW_NULLS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.anyOrderTextFieldN(N_STRING);
            Integer i = rootNode.anyOrderIntFieldN(N_INT);
            Long l = rootNode.anyOrderLongFieldN(N_LONG);
            Boolean b = rootNode.anyOrderBoolFieldN(N_BOOL);
            Float f = rootNode.anyOrderFloatFieldN(N_FLOAT);
            Double d = rootNode.anyOrderDoubleFieldN(N_DOUBLE);
            Colour e = (Colour)rootNode.anyOrderEnumFieldN(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldN(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.anyOrderConstFieldN(N_CONST, V_CONST);
            rootNode.verifyNoOtherChildren(FIELD_NAMES);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "any order, allow nulls";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_ID_STRICT = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.anyOrderTextFieldE(new TagNodeId(N_STRING, ns));
            int i = rootNode.anyOrderIntFieldE(new TagNodeId(N_INT, ns));
            long l = rootNode.anyOrderLongFieldE(new TagNodeId(N_LONG, ns));
            boolean b = rootNode.anyOrderBoolFieldE(new TagNodeId(N_BOOL, ns));
            float f = rootNode.anyOrderFloatFieldE(new TagNodeId(N_FLOAT, ns));
            double d = rootNode.anyOrderDoubleFieldE(new TagNodeId(N_DOUBLE, ns));
            Colour e = (Colour)rootNode.anyOrderEnumFieldE(new TagNodeId(N_ENUM, ns), Colour.class);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldE(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class);
            String c = rootNode.anyOrderConstFieldE(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoOtherChildren(ns, FIELD_NAMES);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "anyOrder, TagNodeId, strict";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_ID_WITH_DEFAULTS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.anyOrderTextFieldD(new TagNodeId(N_STRING, ns), V_STRING);
            int i = rootNode.anyOrderIntFieldD(new TagNodeId(N_INT, ns), V_INT);
            long l = rootNode.anyOrderLongFieldD(new TagNodeId(N_LONG, ns), V_LONG);
            boolean b = rootNode.anyOrderBoolFieldD(new TagNodeId(N_BOOL, ns), V_BOOL);
            float f = rootNode.anyOrderFloatFieldD(new TagNodeId(N_FLOAT, ns), V_FLOAT);
            double d = rootNode.anyOrderDoubleFieldD(new TagNodeId(N_DOUBLE, ns), V_DOUBLE);
            Colour e = (Colour)rootNode.anyOrderEnumFieldD(new TagNodeId(N_ENUM, ns), Colour.class, Colour.Red);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldD(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class, V_STRING_MAPPED);
            String c = rootNode.anyOrderConstFieldD(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoOtherChildren(ns, FIELD_NAMES);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "any order, TagNodeId, with defaults";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_ID_ALLOW_NULLS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.anyOrderTextFieldN(new TagNodeId(N_STRING, ns));
            Integer i = rootNode.anyOrderIntFieldN(new TagNodeId(N_INT, ns));
            Long l = rootNode.anyOrderLongFieldN(new TagNodeId(N_LONG, ns));
            Boolean b = rootNode.anyOrderBoolFieldN(new TagNodeId(N_BOOL, ns));
            Float f = rootNode.anyOrderFloatFieldN(new TagNodeId(N_FLOAT, ns));
            Double d = rootNode.anyOrderDoubleFieldN(new TagNodeId(N_DOUBLE, ns));
            Colour e = (Colour)rootNode.anyOrderEnumFieldN(new TagNodeId(N_ENUM, ns), Colour.class);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldN(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class);
            String c = rootNode.anyOrderConstFieldN(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoOtherChildren(ns, FIELD_NAMES);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "any order, TagNodeId, allow nulls";
        }
    };
    
    private static final XmlSubTest XML1_STRICT_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.nextTextFieldE(N_STRING, DefinedCharSets.LATIN_LETTERS);
            int i = rootNode.nextIntFieldE(N_INT, R_INT);
            long l = rootNode.nextLongFieldE(N_LONG, R_LONG);
            boolean b = rootNode.nextBoolFieldE(N_BOOL);
            float f = rootNode.nextFloatFieldE(N_FLOAT, R_FLOAT);
            double d = rootNode.nextDoubleFieldE(N_DOUBLE, R_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldE(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldE(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.nextConstFieldE(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "strict1, set";
        }
    };
    
    private static final XmlSubTest WITH_DEFAULTS_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.nextTextFieldD(N_STRING, V_STRING, DefinedCharSets.LATIN_LETTERS);
            int i = rootNode.nextIntFieldD(N_INT, V_INT, R_INT);
            long l = rootNode.nextLongFieldD(N_LONG, V_LONG, R_LONG);
            boolean b = rootNode.nextBoolFieldD(N_BOOL, V_BOOL);
            float f = rootNode.nextFloatFieldD(N_FLOAT, V_FLOAT, R_FLOAT);
            double d = rootNode.nextDoubleFieldD(N_DOUBLE, V_DOUBLE, R_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldD(N_ENUM, Colour.class, Colour.Red);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldD(N_STRING_MAPPED, Ip4Address.class, V_STRING_MAPPED);
            String c = rootNode.nextConstFieldD(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "with defaults, set";
        }
    };
    
    private static final XmlSubTest ALLOW_NULLS_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.nextTextFieldN(N_STRING, DefinedCharSets.LATIN_LETTERS);
            Integer i = rootNode.nextIntFieldN(N_INT, R_INT);
            Long l = rootNode.nextLongFieldN(N_LONG, R_LONG);
            Boolean b = rootNode.nextBoolFieldN(N_BOOL);
            Float f = rootNode.nextFloatFieldN(N_FLOAT, R_FLOAT);
            Double d = rootNode.nextDoubleFieldN(N_DOUBLE, R_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldN(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldN(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.nextConstFieldN(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "allow nulls, set";
        }
    };
    
    private static final XmlSubTest ID_STRICT_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.nextTextFieldE(new TagNodeId(N_STRING, ns), DefinedCharSets.LATIN_LETTERS);
            int i = rootNode.nextIntFieldE(new TagNodeId(N_INT, ns), R_INT);
            long l = rootNode.nextLongFieldE(new TagNodeId(N_LONG, ns), R_LONG);
            boolean b = rootNode.nextBoolFieldE(new TagNodeId(N_BOOL, ns));
            float f = rootNode.nextFloatFieldE(new TagNodeId(N_FLOAT, ns), R_FLOAT);
            double d = rootNode.nextDoubleFieldE(new TagNodeId(N_DOUBLE, ns), R_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldE(new TagNodeId(N_ENUM, ns), Colour.class);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldE(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class);
            String c = rootNode.nextConstFieldE(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "TagNodeId, strict, set";
        }
    };
    
    private static final XmlSubTest ID_WITH_DEFAULTS_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.nextTextFieldD(new TagNodeId(N_STRING, ns), V_STRING, DefinedCharSets.LATIN_LETTERS);
            int i = rootNode.nextIntFieldD(new TagNodeId(N_INT, ns), V_INT, R_INT);
            long l = rootNode.nextLongFieldD(new TagNodeId(N_LONG, ns), V_LONG, R_LONG);
            boolean b = rootNode.nextBoolFieldD(new TagNodeId(N_BOOL, ns), V_BOOL);
            float f = rootNode.nextFloatFieldD(new TagNodeId(N_FLOAT, ns), V_FLOAT, R_FLOAT);
            double d = rootNode.nextDoubleFieldD(new TagNodeId(N_DOUBLE, ns), V_DOUBLE, R_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldD(new TagNodeId(N_ENUM, ns), Colour.class, Colour.Red);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldD(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class, V_STRING_MAPPED);
            String c = rootNode.nextConstFieldD(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "TagNodeId, with defaults, set";
        }
    };
    
    private static final XmlSubTest ID_ALLOW_NULLS_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.nextTextFieldN(new TagNodeId(N_STRING, ns), DefinedCharSets.LATIN_LETTERS);
            Integer i = rootNode.nextIntFieldN(new TagNodeId(N_INT, ns), R_INT);
            Long l = rootNode.nextLongFieldN(new TagNodeId(N_LONG, ns), R_LONG);
            Boolean b = rootNode.nextBoolFieldN(new TagNodeId(N_BOOL, ns));
            Float f = rootNode.nextFloatFieldN(new TagNodeId(N_FLOAT, ns), R_FLOAT);
            Double d = rootNode.nextDoubleFieldN(new TagNodeId(N_DOUBLE, ns), R_DOUBLE);
            Colour e = (Colour)rootNode.nextEnumFieldN(new TagNodeId(N_ENUM, ns), Colour.class);
            Ip4Address m = (Ip4Address)rootNode.nextStringMappedFieldN(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class);
            String c = rootNode.nextConstFieldN(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "TagNodeId, allow nulls, set";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_STRICT_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.anyOrderTextFieldE(N_STRING, DefinedCharSets.LATIN_LETTERS);
            int i = rootNode.anyOrderIntFieldE(N_INT, R_INT);
            long l = rootNode.anyOrderLongFieldE(N_LONG, R_LONG);
            boolean b = rootNode.anyOrderBoolFieldE(N_BOOL);
            float f = rootNode.anyOrderFloatFieldE(N_FLOAT, R_FLOAT);
            double d = rootNode.anyOrderDoubleFieldE(N_DOUBLE, R_DOUBLE);
            Colour e = (Colour)rootNode.anyOrderEnumFieldE(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldE(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.anyOrderConstFieldE(N_CONST, V_CONST);
            rootNode.verifyNoOtherChildren(N_STRING, N_INT, N_LONG, N_BOOL, N_FLOAT, N_DOUBLE, N_ENUM, N_STRING_MAPPED, N_CONST);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "any order, strict";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_WITH_DEFAULTS_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.anyOrderTextFieldD(N_STRING, V_STRING, DefinedCharSets.LATIN_LETTERS);
            int i = rootNode.anyOrderIntFieldD(N_INT, V_INT, R_INT);
            long l = rootNode.anyOrderLongFieldD(N_LONG, V_LONG, R_LONG);
            boolean b = rootNode.anyOrderBoolFieldD(N_BOOL, V_BOOL);
            float f = rootNode.anyOrderFloatFieldD(N_FLOAT, V_FLOAT, R_FLOAT);
            double d = rootNode.anyOrderDoubleFieldD(N_DOUBLE, V_DOUBLE, R_DOUBLE);
            Colour e = (Colour)rootNode.anyOrderEnumFieldD(N_ENUM, Colour.class, Colour.Red);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldD(N_STRING_MAPPED, Ip4Address.class, V_STRING_MAPPED);
            String c = rootNode.anyOrderConstFieldD(N_CONST, V_CONST);
            rootNode.verifyNoOtherChildren(FIELD_NAMES);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "any order with defaults";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_ALLOW_NULLS_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.anyOrderTextFieldN(N_STRING, DefinedCharSets.LATIN_LETTERS);
            Integer i = rootNode.anyOrderIntFieldN(N_INT, R_INT);
            Long l = rootNode.anyOrderLongFieldN(N_LONG, R_LONG);
            Boolean b = rootNode.anyOrderBoolFieldN(N_BOOL);
            Float f = rootNode.anyOrderFloatFieldN(N_FLOAT, R_FLOAT);
            Double d = rootNode.anyOrderDoubleFieldN(N_DOUBLE, R_DOUBLE);
            Colour e = (Colour)rootNode.anyOrderEnumFieldN(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldN(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.anyOrderConstFieldN(N_CONST, V_CONST);
            rootNode.verifyNoOtherChildren(FIELD_NAMES);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "any order, allow nulls";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_ID_STRICT_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.anyOrderTextFieldE(new TagNodeId(N_STRING, ns), DefinedCharSets.LATIN_LETTERS);
            int i = rootNode.anyOrderIntFieldE(new TagNodeId(N_INT, ns), R_INT);
            long l = rootNode.anyOrderLongFieldE(new TagNodeId(N_LONG, ns), R_LONG);
            boolean b = rootNode.anyOrderBoolFieldE(new TagNodeId(N_BOOL, ns));
            float f = rootNode.anyOrderFloatFieldE(new TagNodeId(N_FLOAT, ns), R_FLOAT);
            double d = rootNode.anyOrderDoubleFieldE(new TagNodeId(N_DOUBLE, ns), R_DOUBLE);
            Colour e = (Colour)rootNode.anyOrderEnumFieldE(new TagNodeId(N_ENUM, ns), Colour.class);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldE(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class);
            String c = rootNode.anyOrderConstFieldE(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoOtherChildren(ns, FIELD_NAMES);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "anyOrder, TagNodeId, strict";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_ID_WITH_DEFAULTS_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.anyOrderTextFieldD(new TagNodeId(N_STRING, ns), V_STRING, DefinedCharSets.LATIN_LETTERS);
            int i = rootNode.anyOrderIntFieldD(new TagNodeId(N_INT, ns), V_INT, R_INT);
            long l = rootNode.anyOrderLongFieldD(new TagNodeId(N_LONG, ns), V_LONG, R_LONG);
            boolean b = rootNode.anyOrderBoolFieldD(new TagNodeId(N_BOOL, ns), V_BOOL);
            float f = rootNode.anyOrderFloatFieldD(new TagNodeId(N_FLOAT, ns), V_FLOAT, R_FLOAT);
            double d = rootNode.anyOrderDoubleFieldD(new TagNodeId(N_DOUBLE, ns), V_DOUBLE, R_DOUBLE);
            Colour e = (Colour)rootNode.anyOrderEnumFieldD(new TagNodeId(N_ENUM, ns), Colour.class, Colour.Red);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldD(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class, V_STRING_MAPPED);
            String c = rootNode.anyOrderConstFieldD(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoOtherChildren(ns, FIELD_NAMES);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "any order, TagNodeId, with defaults";
        }
    };
    
    private static final XmlSubTest ANY_ORDER_ID_ALLOW_NULLS_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            XmlNameSpace ns = new XmlNameSpace("y", "NS1");
            TagNode rootNode = xmlReader.getRootNode(new TagNodeId(N_MIXED_DATA, ns));
            String s = rootNode.anyOrderTextFieldN(new TagNodeId(N_STRING, ns), DefinedCharSets.LATIN_LETTERS);
            Integer i = rootNode.anyOrderIntFieldN(new TagNodeId(N_INT, ns), R_INT);
            Long l = rootNode.anyOrderLongFieldN(new TagNodeId(N_LONG, ns), R_LONG);
            Boolean b = rootNode.anyOrderBoolFieldN(new TagNodeId(N_BOOL, ns));
            Float f = rootNode.anyOrderFloatFieldN(new TagNodeId(N_FLOAT, ns), R_FLOAT);
            Double d = rootNode.anyOrderDoubleFieldN(new TagNodeId(N_DOUBLE, ns), R_DOUBLE);
            Colour e = (Colour)rootNode.anyOrderEnumFieldN(new TagNodeId(N_ENUM, ns), Colour.class);
            Ip4Address m = (Ip4Address)rootNode.anyOrderStringMappedFieldN(new TagNodeId(N_STRING_MAPPED, ns), Ip4Address.class);
            String c = rootNode.anyOrderConstFieldN(new TagNodeId(N_CONST, ns), V_CONST);
            rootNode.verifyNoOtherChildren(ns, FIELD_NAMES);
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "any order, TagNodeId, allow nulls";
        }
    };
    
    private static final XmlSubTest ATTR_STRICT = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.attributeValueE(N_STRING);
            int i = rootNode.attributeIntE(N_INT);
            long l = rootNode.attributeLongE(N_LONG);
            boolean b = rootNode.attributeBoolE(N_BOOL);
            float f = rootNode.attributeFloatE(N_FLOAT);
            double d = rootNode.attributeDoubleE(N_DOUBLE);
            Colour e = (Colour)rootNode.attributeEnumE(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.attributeStringMappedE(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.attributeConstE(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "attributes, strict";
        }
    };
    
    private static final XmlSubTest ATTR_ALLOW_NULLS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.attributeValueN(N_STRING);
            Integer i = rootNode.attributeIntN(N_INT);
            Long l = rootNode.attributeLongN(N_LONG);
            Boolean b = rootNode.attributeBoolN(N_BOOL);
            Float f = rootNode.attributeFloatN(N_FLOAT);
            Double d = rootNode.attributeDoubleN(N_DOUBLE);
            Colour e = (Colour)rootNode.attributeEnumN(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.attributeStringMappedN(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.attributeConstN(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "attributes, allow nulls";
        }
    };
    
    private static final XmlSubTest ATTR_WITH_DEFAULTS = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.attributeValueD(N_STRING, V_STRING);
            int i = rootNode.attributeIntD(N_INT, V_INT);
            long l = rootNode.attributeLongD(N_LONG, V_LONG);
            boolean b = rootNode.attributeBoolD(N_BOOL, V_BOOL);
            float f = rootNode.attributeFloatD(N_FLOAT, V_FLOAT);
            double d = rootNode.attributeDoubleD(N_DOUBLE, V_DOUBLE);
            Colour e = (Colour)rootNode.attributeEnumD(N_ENUM, Colour.class, V_ENUM);
            Ip4Address m =
                    (Ip4Address)rootNode.attributeStringMappedD(
                            N_STRING_MAPPED, Ip4Address.class, V_STRING_MAPPED);
            String c = rootNode.attributeConstD(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "attributes, with default";
        }
    };
    
    private static final XmlSubTest ATTR_STRICT_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.attributeValueE(N_STRING, DefinedCharSets.LATIN_LETTERS);
            int i = rootNode.attributeIntE(N_INT, R_INT);
            long l = rootNode.attributeLongE(N_LONG, R_LONG);
            boolean b = rootNode.attributeBoolE(N_BOOL);
            float f = rootNode.attributeFloatE(N_FLOAT, R_FLOAT);
            double d = rootNode.attributeDoubleE(N_DOUBLE, R_DOUBLE);
            Colour e = (Colour)rootNode.attributeEnumE(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.attributeStringMappedE(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.attributeConstE(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "attributes, strict, set";
        }
    };
    
    private static final XmlSubTest ATTR_ALLOW_NULLS_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.attributeValueN(N_STRING, DefinedCharSets.LATIN_LETTERS);
            Integer i = rootNode.attributeIntN(N_INT, R_INT);
            Long l = rootNode.attributeLongN(N_LONG, R_LONG);
            Boolean b = rootNode.attributeBoolN(N_BOOL);
            Float f = rootNode.attributeFloatN(N_FLOAT, R_FLOAT);
            Double d = rootNode.attributeDoubleN(N_DOUBLE, R_DOUBLE);
            Colour e = (Colour)rootNode.attributeEnumN(N_ENUM, Colour.class);
            Ip4Address m = (Ip4Address)rootNode.attributeStringMappedN(N_STRING_MAPPED, Ip4Address.class);
            String c = rootNode.attributeConstN(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "attributes, allow nulls, set";
        }
    };
    
    private static final XmlSubTest ATTR_WITH_DEFAULTS_SET = new XmlSubTest()
    {    
        @Override
        public Object
        read(XmlReader xmlReader)
        {
            TagNode rootNode = xmlReader.getRootNode(N_MIXED_DATA);
            String s = rootNode.attributeValueD(N_STRING, V_STRING, DefinedCharSets.LATIN_LETTERS);
            int i = rootNode.attributeIntD(N_INT, V_INT, R_INT);
            long l = rootNode.attributeLongD(N_LONG, V_LONG, R_LONG);
            boolean b = rootNode.attributeBoolD(N_BOOL, V_BOOL);
            float f = rootNode.attributeFloatD(N_FLOAT, V_FLOAT, R_FLOAT);
            double d = rootNode.attributeDoubleD(N_DOUBLE, V_DOUBLE, R_DOUBLE);
            Colour e = (Colour)rootNode.attributeEnumD(N_ENUM, Colour.class, V_ENUM);
            Ip4Address m =
                    (Ip4Address)rootNode.attributeStringMappedD(
                            N_STRING_MAPPED, Ip4Address.class, V_STRING_MAPPED);
            String c = rootNode.attributeConstD(N_CONST, V_CONST);
            rootNode.verifyNoMoreChildren();
            
            MixedData md = new MixedData(s, i, l, b, f, d, e, m, c);
            printTestPartSeparator();
            return md;
        }
        
        @Override
        public String
        toString()
        {
            return "attributes, with default, set";
        }
    };
    
    private XmlSubTest[] STRAIGHT_NAME_ORDERED_SUBTESTS =
    {
        XML1_STRICT, WITH_DEFAULTS, ALLOW_NULLS,
        XML1_STRICT_SET, WITH_DEFAULTS_SET, ALLOW_NULLS_SET,
    };
    
    private XmlSubTest[] STRAIGHT_NAME_STRICT_ORDERED_SUBTESTS =
    {
        XML1_STRICT,
        XML1_STRICT_SET
    };
    
    private XmlSubTest[] STRAIGHT_NAME_WITH_DEFAULTS_ORDERED_SUBTESTS =
    {
        WITH_DEFAULTS,
        WITH_DEFAULTS_SET
    };
    
    private XmlSubTest[] STRAIGHT_NAME_ALLOW_NULLS_ORDERED_SUBTESTS =
    {
        ALLOW_NULLS,
        ALLOW_NULLS_SET
    };
    
    private XmlSubTest[] STRAIGHT_NAME_STRICT_ANY_ORDER_SUBTESTS =
    {
        ANY_ORDER_STRICT,
        ANY_ORDER_STRICT_SET
    };
    
    private XmlSubTest[] STRAIGHT_NAME_WITH_DEFAULTS_ANY_ORDER_SUBTESTS =
    {
        ANY_ORDER_WITH_DEFAULTS,
        ANY_ORDER_WITH_DEFAULTS_SET
    };
    
    private XmlSubTest[] STRAIGHT_NAME_ALLOW_NULLS_ANY_ORDER_SUBTESTS =
    {
        ANY_ORDER_ALLOW_NULLS,
        ANY_ORDER_ALLOW_NULLS_SET
    };
    
    private XmlSubTest[] STRAIGHT_NAME_ANY_ORDER_SUBTESTS =
    {
        ANY_ORDER_STRICT, ANY_ORDER_WITH_DEFAULTS, ANY_ORDER_ALLOW_NULLS,
        ANY_ORDER_STRICT_SET, ANY_ORDER_WITH_DEFAULTS_SET, ANY_ORDER_ALLOW_NULLS_SET,
    };
    
    private XmlSubTest[] STRAIGHT_NAME_SET_ORDERED_SUBTESTS =
    {
        XML1_STRICT_SET, WITH_DEFAULTS_SET, ALLOW_NULLS_SET,
    };
    
    private XmlSubTest[] STRAIGHT_NAME_SET_ANY_ORDER_SUBTESTS =
    {
        ANY_ORDER_STRICT_SET, ANY_ORDER_WITH_DEFAULTS_SET, ANY_ORDER_ALLOW_NULLS_SET,
    };
    
    //--
    
    private XmlSubTest[] ID_ORDERED_SUBTESTS =
    {
        ID_STRICT, ID_WITH_DEFAULTS, ID_ALLOW_NULLS,
        ID_STRICT_SET, ID_WITH_DEFAULTS_SET, ID_ALLOW_NULLS_SET,
    };
    
    private XmlSubTest[] ID_STRICT_ORDERED_SUBTESTS =
    {
        ID_STRICT,
        ID_STRICT_SET
    };
    
    private XmlSubTest[] ID_WITH_DEFAULTS_ORDERED_SUBTESTS =
    {
        ID_WITH_DEFAULTS,
        ID_WITH_DEFAULTS_SET
    };
    
    private XmlSubTest[] ID_ALLOW_NULLS_ORDERED_SUBTESTS =
    {
        ID_ALLOW_NULLS,
        ID_ALLOW_NULLS_SET
    };
    
    private XmlSubTest[] ID_STRICT_ANY_ORDER_SUBTESTS =
    {
        ANY_ORDER_ID_STRICT,
        ANY_ORDER_ID_STRICT_SET
    };
    
    private XmlSubTest[] ID_WITH_DEFAULTS_ANY_ORDER_SUBTESTS =
    {
        ANY_ORDER_ID_WITH_DEFAULTS,
        ANY_ORDER_ID_WITH_DEFAULTS_SET
    };
    
    private XmlSubTest[] ID_ALLOW_NULLS_ANY_ORDER_SUBTESTS =
    {
        ANY_ORDER_ID_ALLOW_NULLS,
        ANY_ORDER_ID_ALLOW_NULLS_SET
    };
    
    private XmlSubTest[] ID_ANY_ORDER_SUBTESTS =
    {
        ANY_ORDER_ID_STRICT, ANY_ORDER_ID_WITH_DEFAULTS, ANY_ORDER_ID_ALLOW_NULLS,
        ANY_ORDER_ID_STRICT_SET, ANY_ORDER_ID_WITH_DEFAULTS_SET, ANY_ORDER_ID_ALLOW_NULLS_SET,
    };
    
    private XmlSubTest[] ID_SET_ORDERED_SUBTESTS =
    {
        ID_STRICT_SET, ID_WITH_DEFAULTS_SET, ID_ALLOW_NULLS_SET,
    };
    
    private XmlSubTest[] ID_SET_ANY_ORDER_SUBTESTS =
    {
        ANY_ORDER_ID_STRICT_SET, ANY_ORDER_ID_WITH_DEFAULTS_SET, ANY_ORDER_ID_ALLOW_NULLS_SET,
    };
    
    private XmlSubTest[] ATTR_STRICT_SUBTESTS =
    {
        ATTR_STRICT, ATTR_STRICT_SET,
    };
    
    private XmlSubTest[] ATTR_ALLOW_NULLS_SUBTESTS =
    {
        ATTR_ALLOW_NULLS, ATTR_ALLOW_NULLS_SET,
    };
    
    private XmlSubTest[] ATTR_WITH_DEFAULTS_SUBTESTS =
    {
        ATTR_WITH_DEFAULTS, ATTR_WITH_DEFAULTS_SET,
    };
    
    private XmlSubTest[] ATTR_SET_SUBTESTS =
    {
        ATTR_STRICT_SET, ATTR_ALLOW_NULLS_SET, ATTR_WITH_DEFAULTS_SET
    };
    
    public void
    testOk(XmlSubTest[]   subTests,
           String[]       xmls,
           XmlReadOptions readOptions,
           Object         expectedObject)
    {
        for (String xml : xmls)
        {
            testOk(subTests, xml, readOptions, expectedObject);
        }
    }
    
    public void
    testOk(XmlSubTest[]   subTests,
           String         xml,
           XmlReadOptions readOptions,
           Object         expectedObject)
    {
        for (XmlSubTest subTest : subTests)
        {
            subTest.testOk(xml, readOptions, expectedObject);
        }
    }
    
    public void
    testFail(XmlSubTest[]   subTests,
             String[]       xmls,
             XmlReadOptions readOptions,
             Class<?>       expectedEClass)
    {
        for (String xml : xmls)
        {
            testFail(subTests, xml, readOptions, expectedEClass);
        }
    }
    
    public void
    testFail(XmlSubTest[]   subTests,
             String         xml,
             XmlReadOptions readOptions,
             Class<?>       expectedEClass)
    {
        for (XmlSubTest subTest : subTests)
        {
            subTest.testFail(xml, readOptions, expectedEClass);
        }
    }
    
    public void
    test()
    {
        announceTestClassStart();
        
        testOk(STRAIGHT_NAME_ORDERED_SUBTESTS, MIXED_DATA_XML_OK, STRICT_READ_OPTIONS, MIXED_DATA_STANDARD);
        testOk(STRAIGHT_NAME_ORDERED_SUBTESTS, MIXED_DATA_XML_OK_2, STRICT_READ_OPTIONS, MIXED_DATA_2);
        testFail(STRAIGHT_NAME_STRICT_ORDERED_SUBTESTS, MIXED_DATA_XML_NO_C, STRICT_READ_OPTIONS, MissingNodeException.class);
        testFail(STRAIGHT_NAME_STRICT_ORDERED_SUBTESTS, MIXED_DATA_XML_NO_I, STRICT_READ_OPTIONS, WrongNodeException.class);
        testOk(STRAIGHT_NAME_WITH_DEFAULTS_ORDERED_SUBTESTS, MIXED_DATA_SOMETHING_MISSING_SET, STRICT_READ_OPTIONS, MIXED_DATA_STANDARD);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ORDERED_SUBTESTS, MIXED_DATA_XML_NO_S, STRICT_READ_OPTIONS, MIXED_DATA_NO_S);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ORDERED_SUBTESTS, MIXED_DATA_XML_NO_I, STRICT_READ_OPTIONS, MIXED_DATA_NO_I);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ORDERED_SUBTESTS, MIXED_DATA_XML_NO_L, STRICT_READ_OPTIONS, MIXED_DATA_NO_L);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ORDERED_SUBTESTS, MIXED_DATA_XML_NO_B, STRICT_READ_OPTIONS, MIXED_DATA_NO_B);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ORDERED_SUBTESTS, MIXED_DATA_XML_NO_F, STRICT_READ_OPTIONS, MIXED_DATA_NO_F);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ORDERED_SUBTESTS, MIXED_DATA_XML_NO_D, STRICT_READ_OPTIONS, MIXED_DATA_NO_D);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ORDERED_SUBTESTS, MIXED_DATA_XML_NO_E, STRICT_READ_OPTIONS, MIXED_DATA_NO_E);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ORDERED_SUBTESTS, MIXED_DATA_XML_NO_M, STRICT_READ_OPTIONS, MIXED_DATA_NO_M);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ORDERED_SUBTESTS, MIXED_DATA_XML_EMPTY, STRICT_READ_OPTIONS, MIXED_DATA_EMPTY);
        testFail(STRAIGHT_NAME_ORDERED_SUBTESTS, MIXED_DATA_XML_BAD_TYPE_SET, STRICT_READ_OPTIONS, XmlTextFieldDoesNotMatchTypeException.class);
        testFail(STRAIGHT_NAME_SET_ORDERED_SUBTESTS, MIXED_DATA_XML_OUTSIDE_SET_SET, STRICT_READ_OPTIONS, XmlTextFieldNotInSetException.class);
        BAD_ENUM_CLASS.testFail(MIXED_DATA_XML_OK, STRICT_READ_OPTIONS, SuppliedClassNotEnumException.class);
        NO_STRING_CONSTRUCTOR_CLASS.testFail(MIXED_DATA_XML_OK, STRICT_READ_OPTIONS, SuppliedClassNotStringMappedException.class);
        PRIVATE_STRING_CONSTRUCTOR_CLASS.testFail(MIXED_DATA_XML_OK, STRICT_READ_OPTIONS, SuppliedClassNotStringMappedException.class);
        
        testOk(STRAIGHT_NAME_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_OK_ANY_ORDER_SET, ALLOW_UNORDERED_ACCESS, MIXED_DATA_STANDARD);
        testOk(STRAIGHT_NAME_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_OK_2, ALLOW_UNORDERED_ACCESS, MIXED_DATA_2);
        testFail(STRAIGHT_NAME_STRICT_ANY_ORDER_SUBTESTS, MIXED_DATA_SOMETHING_MISSING_SET, ALLOW_UNORDERED_ACCESS, MissingNodeException.class);
        testOk(STRAIGHT_NAME_WITH_DEFAULTS_ANY_ORDER_SUBTESTS, MIXED_DATA_SOMETHING_MISSING_SET, ALLOW_UNORDERED_ACCESS, MIXED_DATA_STANDARD);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NO_S, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_S);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NO_I, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_I);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NO_L, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_L);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NO_B, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_B);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NO_F, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_F);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NO_D, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_D);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NO_E, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_E);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NO_M, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_M);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_EMPTY, ALLOW_UNORDERED_ACCESS, MIXED_DATA_EMPTY);
        testOk(STRAIGHT_NAME_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_REVERSE, ALLOW_UNORDERED_ACCESS, MIXED_DATA_STANDARD);
        testFail(STRAIGHT_NAME_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_BAD_TYPE_SET, ALLOW_UNORDERED_ACCESS, XmlTextFieldDoesNotMatchTypeException.class);
        testFail(STRAIGHT_NAME_SET_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_OUTSIDE_SET_SET, ALLOW_UNORDERED_ACCESS, XmlTextFieldNotInSetException.class);
        
        testOk(ID_ORDERED_SUBTESTS, MIXED_DATA_XML_NS_OK, STRICT_READ_OPTIONS, MIXED_DATA_STANDARD);
        testFail(ID_STRICT_ORDERED_SUBTESTS, MIXED_DATA_XML_NS_EMPTY, STRICT_READ_OPTIONS, MissingNodeException.class);
        testFail(ID_STRICT_ORDERED_SUBTESTS, MIXED_DATA_XML_NS_REVERSE, STRICT_READ_OPTIONS, WrongNodeException.class);
        testOk(ID_WITH_DEFAULTS_ORDERED_SUBTESTS, MIXED_DATA_XML_NS_EMPTY, STRICT_READ_OPTIONS, MIXED_DATA_STANDARD);
        testOk(ID_ALLOW_NULLS_ORDERED_SUBTESTS, MIXED_DATA_XML_NS_EMPTY, STRICT_READ_OPTIONS, MIXED_DATA_EMPTY);
        testFail(ID_SET_ORDERED_SUBTESTS, MIXED_DATA_XML_NS_OUTSIDE_SET_SET, STRICT_READ_OPTIONS, XmlTextFieldNotInSetException.class);
        
        testOk(ID_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NS_OK_ANY_ORDER_SET, ALLOW_UNORDERED_ACCESS, MIXED_DATA_STANDARD);
        testFail(ID_STRICT_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NS_EMPTY, ALLOW_UNORDERED_ACCESS, MissingNodeException.class);
        testOk(ID_WITH_DEFAULTS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NS_OK, ALLOW_UNORDERED_ACCESS, MIXED_DATA_STANDARD);
        testOk(ID_WITH_DEFAULTS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NS_EMPTY, ALLOW_UNORDERED_ACCESS, MIXED_DATA_STANDARD);
        testOk(ID_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NS_OK, ALLOW_UNORDERED_ACCESS, MIXED_DATA_STANDARD);
        testOk(ID_ALLOW_NULLS_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NS_EMPTY, ALLOW_UNORDERED_ACCESS, MIXED_DATA_EMPTY);
        testFail(ID_SET_ANY_ORDER_SUBTESTS, MIXED_DATA_XML_NS_OUTSIDE_SET_SET, ALLOW_UNORDERED_ACCESS, XmlTextFieldNotInSetException.class);
        
        testOk(ATTR_STRICT_SUBTESTS, MIXED_DATA_XML_ATTR_OK, STRICT_READ_OPTIONS, MIXED_DATA_STANDARD);
        //testOk(ATTR_STRICT_SUBTESTS, MIXED_DATA_XML_ATTR_NS_OK, STRICT_READ_OPTIONS, MIXED_DATA_STANDARD);
        testFail(ATTR_STRICT_SUBTESTS, MIXED_DATA_XML_MISSING_ATTRIBUTE_SET, STRICT_READ_OPTIONS, XmlMissingAttributeException.class);
        ATTR_ALLOW_NULLS.testOk(MIXED_DATA_XML_ATTR_OK, STRICT_READ_OPTIONS, MIXED_DATA_STANDARD);
        testOk(ATTR_WITH_DEFAULTS_SUBTESTS, MIXED_DATA_XML_ATTR_OK, STRICT_READ_OPTIONS, MIXED_DATA_STANDARD);
        testOk(ATTR_WITH_DEFAULTS_SUBTESTS, MIXED_DATA_XML_MISSING_ATTRIBUTE_SET, STRICT_READ_OPTIONS, MIXED_DATA_STANDARD);
        testOk(ATTR_ALLOW_NULLS_SUBTESTS, MIXED_DATA_XML_ATTR_NO_S, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_S);
        testOk(ATTR_ALLOW_NULLS_SUBTESTS, MIXED_DATA_XML_ATTR_NO_I, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_I);
        testOk(ATTR_ALLOW_NULLS_SUBTESTS, MIXED_DATA_XML_ATTR_NO_L, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_L);
        testOk(ATTR_ALLOW_NULLS_SUBTESTS, MIXED_DATA_XML_ATTR_NO_B, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_B);
        testOk(ATTR_ALLOW_NULLS_SUBTESTS, MIXED_DATA_XML_ATTR_NO_F, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_F);
        testOk(ATTR_ALLOW_NULLS_SUBTESTS, MIXED_DATA_XML_ATTR_NO_D, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_D);
        testOk(ATTR_ALLOW_NULLS_SUBTESTS, MIXED_DATA_XML_ATTR_NO_E, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_E);
        testOk(ATTR_ALLOW_NULLS_SUBTESTS, MIXED_DATA_XML_ATTR_NO_M, ALLOW_UNORDERED_ACCESS, MIXED_DATA_NO_M);
        testOk(ATTR_ALLOW_NULLS_SUBTESTS, MIXED_DATA_XML_EMPTY, ALLOW_UNORDERED_ACCESS, MIXED_DATA_EMPTY);
        testFail(ATTR_SET_SUBTESTS, MIXED_DATA_XML_ATTRIBUTE_OUTSIDE_SET_SET, STRICT_READ_OPTIONS, XmlAttributeNotInSetException.class);
        
        announceTestClassPass();
    }
    
    public static void
    staticTest()
    {
        XmlReaderSuiteTypes t = new XmlReaderSuiteTypes();
        t.test();
    }
    
    public static void
    main(String[] args)
    {
        staticTest();
    }
}
