/*
 * Copyright Rasmus Torkel 2012
 * Author    Rasmus Torkel 2012
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.xml_basic.read.impl;

import java.io.Reader;
import java.io.StringReader;

import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;

public abstract class XmlSubTest extends UnitTest
{
    public abstract Object
    read(XmlReader reader);
    
    @Override
    public String
    toString()
    {
        return getClass().getName();
    }
    
    public void
    testOk(String      xml,
           XmlReadOptions readOptions,
           Object      expectedObject)
    {
        System.out.println(this + ", testOK");
        Object readObject;
        try
        {
            readObject = read(xml, readOptions);
        }
        catch (Exception e)
        {
            e.printStackTrace(System.out);
            try
            {
                Thread.sleep(100);
            }
            catch (InterruptedException e1)
            {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }
            throw new UnitTestError(getClass());
        }

        System.out.println("Object is " + readObject);
        if (   readObject.getClass().equals(expectedObject.getClass())
            && readObject.equals(expectedObject))
        {
            System.out.println("as expected");
        }
        else
        {
            fail("Retrieved " + readObject + " but expected " + expectedObject);
        }
        
        printEndTest();
    }
    
    public Object
    read(String xml,
         XmlReadOptions readOptions)
    {
        System.out.println(readOptions.toString());
        System.out.println(xml);
        printTestPartSeparator();
        printRuler(xml);
        
        Reader reader = new StringReader(xml);
        XmlReader xmlReader = new XmlReader(reader, "Unit test", readOptions);
        return read(xmlReader);
    }
    
    public void
    testFail(String      xml,
             XmlReadOptions readOptions,
             Class<?>    expectedEClass)
    {
        System.out.println(this + ", testFail");
        try
        {
            Object object = read(xml, readOptions);
            fail("Unpectedly succeeded in parsing XML, object is " + object);
        }
        catch (Exception e)
        {
            Class<?> actualEClass = e.getClass();
            if (actualEClass != expectedEClass)
            {
                e.printStackTrace(System.out);
                fail("Got " + actualEClass.getName() + " but expected " + expectedEClass.getName());
            }
            else
            {
                System.out.println(e.toString());
                System.out.println("Exception was as expected");
            }
        }
        printEndTest();
    }
    
    public void
    testOk(String[]    xmls,
           XmlReadOptions readOptions,
           Object      expectedObject)
    {
        for (String xml : xmls)
        {
            testOk(xml, readOptions, expectedObject);
        }
    }
    
    public void
    testFail(String[]    xmls,
             XmlReadOptions readOptions,
             Class<?>    expectedEClass)
    {
        for (String xml : xmls)
        {
            testFail(xml, readOptions, expectedEClass);
        }
    }
}
