package rasmus_torkel.test.xml_basic.read.impl;

import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.exception.XmlSyntaxException;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;

public class XmlUndeclarePrefixTest extends UnitTest
{
    public static void
    main(String[] args)
    {
        testOk();
        testBad(BAD_XML_TAG);
        testBad(BAD_XML_ATTR);
    }
    
    private static final XmlNameSpace NS1 = new XmlNameSpace("ns", "NS1");
    private static final String NS1_DECLARATION = NS1.makeDeclaration();
    
    private static final String UP_TO_E21_XML =
            "<root " + NS1_DECLARATION + ">\n" +
            "  <ns:e1>x1</ns:e1>\n" +
            "  <e2 xmlns:ns=\"\">\n" +
            "    <e21>x21</e21>\n";
    
    private static final String E221_XML =
            "      <ns:e221>x221</ns:e221>\n";
    
    private static final String FROM_E23_XML =
            "    <e23>x23</e23>\n" +
            "  </e2>\n" +
            "  <ns:e3>x3</ns:e3>\n" +
            "</root>\n";
    
    private static final String OK_XML =
            UP_TO_E21_XML +
            "    <ns:e22 " + NS1_DECLARATION + ">\n" +
            E221_XML +
            "    </ns:e22>\n" +
            FROM_E23_XML;
    
    private static final String BAD_XML_TAG =
            UP_TO_E21_XML +
            "    <ns:e22>\n" +
            E221_XML +
            "    </ns:e22>\n" +
            FROM_E23_XML;
    
    private static final String BAD_XML_ATTR =
            UP_TO_E21_XML +
            "    <e22 ns:attr=\"xx\">\n" +
            E221_XML +
            "    </e22>\n" +
            FROM_E23_XML;
    
    private static class TestNodes
    {
        TagNode rootNode;
        TagNode e2Node;
        TagNode e22Node;
    }
    
    public static void
    testOk()
    {
        readAll(OK_XML);
        System.out.println("Succeeded as expected");
        printEndTest();
    }
    
    public static void
    testBad(String xml)
    {
        TestNodes testNodes = readToE21(xml);
        try
        {
            testNodes.e22Node = testNodes.e2Node.nextChildE("e22", XmlNameSpace.ANY);
            System.out.println("Unexpectedly succeeded");
            throw new UnitTestError();
        }
        catch (XmlSyntaxException e)
        {
            System.out.println("Got exception as expected:" + e);
            printEndTest();
        }
        catch (Throwable e)
        {
            System.out.println("Got wrong exception");
            e.printStackTrace(System.out);
            throw new UnitTestError();
        }
        finally
        {
            testNodes.rootNode.closeReader();
        }
    }
    
    public static void
    readAll(String xml)
    {
        TestNodes testNodes = readToE21(xml);
        testNodes.e22Node = testNodes.e2Node.nextChildE("e22", XmlNameSpace.ANY);
        testNodes.e22Node.nextConstFieldE("e221", NS1, "x221");
        testNodes.e22Node.verifyNoMoreChildren();
        testNodes.e2Node.nextConstFieldE("e23", "x23");
        testNodes.e2Node.verifyNoMoreChildren();
        testNodes.rootNode.nextConstFieldE("e3", NS1, "x3");
        testNodes.rootNode.verifyNoMoreChildren();
        System.out.println("read all");
        printTestPartSeparator();
    }
    
    public static TestNodes
    readToE21(String xml)
    {
        System.out.print(xml);
        printTestPartSeparator();
        TestNodes testNodes = new TestNodes();
        testNodes.rootNode = XmlReader.xmlStringToRoot(xml, "unit test (undeclare name space)", XmlReadOptions.DEFAULT);
        testNodes.rootNode.nextConstFieldE("e1", NS1, "x1");
        testNodes.e2Node = testNodes.rootNode.nextChildE("e2");
        testNodes.e2Node.nextConstFieldE("e21", "x21");
        System.out.println("read to e21");
        return testNodes;
    }
}
