package rasmus_torkel.test.xml_basic.simple;

import java.io.File;

import rasmus_torkel.text.read.exception.ReadException;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.MiscXmlFunctions;
import rasmus_torkel.xml_basic.write.XmlSink;

public class SimpleXmlDemo
{
    public static void
    main(String[] args)
    {
        String xmlString = makeXml();
        readXml(xmlString);
        
        xmlString = "<" + xmlString;
        System.out.println(xmlString);
        MiscXmlFunctions.showXmlForHtml(xmlString);
        readBadXml(xmlString);
        
        xmlString = makeXmlWrongRoot();
        readBadXml(xmlString);
        
        xmlString = makeXmlWithoutName();
        readBadXml(xmlString);
        
        xmlString = makeXmlBadInt();
        readBadXml(xmlString);
    }
    
    public static String
    makeXml()
    {
        String name = "Rasmus Torkel";
        int numberOfFingers = 10;
        File file = new File("./data/xx.txt");
        printSampleData(name, numberOfFingers, file);
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("sample");
        xmlSink.sinkSimpleNode("name", name);
        xmlSink.sinkSimpleNode("numberOfFingers", numberOfFingers);
        xmlSink.sinkSimpleNode("file", file);
        xmlSink.closeNode();
        String xmlString = xmlSink.toString();
        System.out.println(xmlString);
        MiscXmlFunctions.showXmlForHtml(xmlString);
        return xmlString;
    }
    
    public static String
    makeXmlWrongRoot()
    {
        String name = "Rasmus Torkel";
        int numberOfFingers = 10;
        File file = new File("./data/xx.txt");
        printSampleData(name, numberOfFingers, file);
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("example");
        xmlSink.sinkSimpleNode("name", name);
        xmlSink.sinkSimpleNode("numberOfFingers", numberOfFingers);
        xmlSink.sinkSimpleNode("file", file);
        xmlSink.closeNode();
        String xmlString = xmlSink.toString();
        System.out.println(xmlString);
        MiscXmlFunctions.showXmlForHtml(xmlString);
        return xmlString;
    }
    
    public static String
    makeXmlWithoutName()
    {
        String name = "Rasmus Torkel";
        int numberOfFingers = 10;
        File file = new File("./data/xx.txt");
        printSampleData(name, numberOfFingers, file);
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("sample");
        xmlSink.sinkSimpleNode("numberOfFingers", numberOfFingers);
        xmlSink.sinkSimpleNode("file", file);
        xmlSink.closeNode();
        String xmlString = xmlSink.toString();
        System.out.println(xmlString);
        MiscXmlFunctions.showXmlForHtml(xmlString);
        return xmlString;
    }
    
    public static String
    makeXmlBadInt()
    {
        String name = "Rasmus Torkel";
        File file = new File("./data/xx.txt");
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("sample");
        xmlSink.sinkSimpleNode("name", name);
        xmlSink.sinkSimpleNode("numberOfFingers", "ten");
        xmlSink.sinkSimpleNode("file", file);
        xmlSink.closeNode();
        String xmlString = xmlSink.toString();
        System.out.println(xmlString);
        MiscXmlFunctions.showXmlForHtml(xmlString);
        return xmlString;
    }
    
    public static void
    readBadXml(String xmlString)
    {
        try
        {
            readXml(xmlString);
        }
        catch (ReadException e)
        {
            System.out.println(e);
        }
    }
    
    public static void
    readXml(String xmlString)
    {
        TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.DEFAULT, "sample");
        String name = sampleNode.nextTextFieldE("name");
        int numberOfFingers = sampleNode.nextIntFieldD("numberOfFingers", 10);
        File file = (File)sampleNode.nextStringMappedFieldN("file", File.class);
        sampleNode.verifyNoMoreChildren();
        printSampleData(name, numberOfFingers, file);
    }
    
    public static void
    printSampleData(String name,
                    int    numberOfFingers,
                    File   file)
    {
        System.out.println("name = " + name);
        System.out.println("numberOfFingers = " + numberOfFingers);
        System.out.println("file = " + file);
        System.out.println();
    }
}
