package rasmus_torkel.test.xml_basic.simple;

import java.io.File;

import rasmus_torkel.text.read.exception.ReadException;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.MiscXmlFunctions;
import rasmus_torkel.xml_basic.write.XmlSink;

public class SimpleXmlExtraTextDemo
{
    public static void
    main(String[] args)
    {
        String xmlString = makeXml();
        readXmlIncludingExtraText(xmlString);
        readXmlExcludingExtraText(xmlString);
        
        readXmlBadOptionCatchException(xmlString);
    }
    
    public static String
    makeXml()
    {
        String name = "Rasmus Torkel";
        int numberOfFingers = 10;
        File file = new File("./data/xx.txt");
        SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("sample");
        xmlSink.sinkText("aa");
        xmlSink.sinkSimpleNode("name", name);
        xmlSink.sinkText("bb");
        xmlSink.sinkSimpleNode("numberOfFingers", numberOfFingers);
        xmlSink.sinkText("cc");
        xmlSink.sinkSimpleNode("file", file);
        xmlSink.sinkText("dd");
        xmlSink.closeNode();
        String xmlString = xmlSink.toString();
        System.out.println(xmlString);
        MiscXmlFunctions.showXmlForHtml(xmlString);
        return xmlString;
    }
    
    public static void
    readXmlBadOptionCatchException(String xmlString)
    {
        try
        {
            readXmlBadOption(xmlString);
        }
        catch (ReadException e)
        {
            System.out.println(e);
        }
    }
    
    public static void
    readXmlBadOption(String xmlString)
    {
        System.out.println("readXmlBadOption");
        TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.DEFAULT, "sample");
        String name = sampleNode.nextTextFieldE("name");
        int numberOfFingers = sampleNode.nextIntFieldD("numberOfFingers", 10);
        File file = (File)sampleNode.nextStringMappedFieldN("file", File.class);
        sampleNode.verifyNoMoreChildren();
        SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
    }
    
    public static void
    readXmlExcludingExtraText(String xmlString)
    {
        System.out.println("readXmlExcludingExtraText");
        TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.ALLOW_SUBNODES_AND_TEXT, "sample");
        String name = sampleNode.nextTextFieldE("name");
        int numberOfFingers = sampleNode.nextIntFieldD("numberOfFingers", 10);
        File file = (File)sampleNode.nextStringMappedFieldN("file", File.class);
        sampleNode.verifyNoMoreChildren();
        SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
    }
    
    public static void
    readXmlIncludingExtraText(String xmlString)
    {
        System.out.println("readXmlIncludingExtraText");
        TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.ALLOW_SUBNODES_AND_TEXT, "sample");
        String text1 = sampleNode.nextTextN();
        String name = sampleNode.nextTextFieldE("name");
        String text2 = sampleNode.nextTextN();
        int numberOfFingers = sampleNode.nextIntFieldD("numberOfFingers", 10);
        String text3 = sampleNode.nextTextN();
        File file = (File)sampleNode.nextStringMappedFieldN("file", File.class);
        String text4 = sampleNode.nextTextN();
        sampleNode.verifyNoMoreChildren();
        System.out.println("text1 = \"" + text1 + "\"");
        System.out.println("text2 = \"" + text2 + "\"");
        System.out.println("text3 = \"" + text3 + "\"");
        System.out.println("text4 = \"" + text4 + "\"");
        SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
    }
}
