/*
 * Copyright Rasmus Torkel 2015
 * Author    Rasmus Torkel 2015
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.xml_basic.simple;

import java.io.File;

import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.MiscXmlFunctions;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;
import rasmus_torkel.xml_basic.write.XmlSink;

public class SimpleXmlNameSpaceDemo
{
    public static void
    main(String[] args)
    {
        String xmlString = makeXmlNs();
        readXml(xmlString);
        makeXmlNsSamePrefix();
    }
    
    public static String
    makeXmlNs()
    {
        String name = "Rasmus Torkel";
        int numberOfFingers = 10;
        File file = new File("./data/xx.txt");
        printSampleData(name, numberOfFingers, file);
        XmlNameSpace ns = new XmlNameSpace("x", "http://rasmustorkel.com/ns1/");
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("sample", ns);
        xmlSink.sinkSimpleNode("name", ns, name);
        xmlSink.sinkSimpleNode("numberOfFingers", ns, numberOfFingers);
        xmlSink.sinkSimpleNode("file", ns, file);
        xmlSink.closeNode();
        String xmlString = xmlSink.toString();
        System.out.println(xmlString);
        MiscXmlFunctions.showXmlForHtml(xmlString);
        return xmlString;
    }
    
    public static String
    makeXmlNsSamePrefix()
    {
        String name = "Rasmus Torkel";
        int numberOfFingers = 10;
        File file = new File("./data/xx.txt");
        printSampleData(name, numberOfFingers, file);
        XmlNameSpace ns1 = new XmlNameSpace("x", "http://rasmustorkel.com/ns1/");
        XmlNameSpace ns2 = new XmlNameSpace("x", "http://rasmustorkel.com/ns2/");
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("sample");
        xmlSink.sinkSimpleNode("name", ns1, name);
        xmlSink.sinkSimpleNode("numberOfFingers", ns2, numberOfFingers);
        xmlSink.sinkSimpleNode("file", file);
        xmlSink.closeNode();
        String xmlString = xmlSink.toString();
        System.out.println(xmlString);
        MiscXmlFunctions.showXmlForHtml(xmlString);
        return xmlString;
    }
    
    public static void
    readXml(String xmlString)
    {
        XmlNameSpace ns = new XmlNameSpace("y", "http://rasmustorkel.com/ns1/");
        TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.DEFAULT, "sample", ns);
        String name = sampleNode.nextTextFieldE("name", ns);
        int numberOfFingers = sampleNode.nextIntFieldD("numberOfFingers", ns, 10);
        File file = (File)sampleNode.nextStringMappedFieldN("file", ns, File.class);
        sampleNode.verifyNoMoreChildren();
        printSampleData(name, numberOfFingers, file);
    }
    
    public static void
    printSampleData(String name,
                    int    numberOfFingers,
                    File   file)
    {
        System.out.println("name = " + name);
        System.out.println("numberOfFingers = " + numberOfFingers);
        System.out.println("file = " + file);
        System.out.println();
    }
}

