package rasmus_torkel.test.xml_basic.simple;

import java.io.File;

import rasmus_torkel.set.ints.IntRange;
import rasmus_torkel.set.strings.StringPatternSet;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.MiscXmlFunctions;
import rasmus_torkel.xml_basic.write.XmlSink;

public class SimpleXmlReadMoreValidationDemo
{
    public static void
    main(String[] args)
    {
        String negativeFingersXml = makeXml("Rasmus Torkel", -1);
        readXmlWithPostValidation(negativeFingersXml);
        readXmlWithSetValidation(negativeFingersXml);
        String singleNameXml = makeXml("Rasmus", -1);
        readXmlWithSetValidation(singleNameXml);
    }
    
    public static String
    makeXml(String name,
            int    numberOfFingers)
    {
        File file = new File("./data/xx.txt");
        SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("sample");
        xmlSink.sinkSimpleNode("name", name);
        xmlSink.sinkSimpleNode("numberOfFingers", numberOfFingers);
        xmlSink.sinkSimpleNode("file", file);
        xmlSink.closeNode();
        String xmlString = xmlSink.toString();
        System.out.println(xmlString);
        MiscXmlFunctions.showXmlForHtml(xmlString);
        return xmlString;
    }
    
    public static void
    readXmlWithPostValidation(String xmlString)
    {
        System.out.println("Reading with post validation");
        try
        {
            TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.DEFAULT, "sample");
            String name = sampleNode.nextTextFieldE("name");
            int numberOfFingers = sampleNode.nextIntFieldD("numberOfFingers", 10);
            File file = (File)sampleNode.nextStringMappedFieldN("file", File.class);
            sampleNode.verifyNoMoreChildren();
            SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
            if (numberOfFingers < 0 || numberOfFingers > 14)
            {
                throw new RuntimeException(
                        sampleNode + " contains numberOfFingers field which has value " +
                        numberOfFingers + " which is not in the range from 0 to 14");
            }
        }
        catch (Exception e)
        {
            System.out.println(e);
        }
    }
    
    public static void
    readXmlWithSetValidation(String xmlString)
    {
        System.out.println("Reading with set validation");
        try
        {
            StringPatternSet nameSet = new StringPatternSet("^[A-Z][a-z]+ [A-Z][a-z]+$");
            IntRange numberOfFingersRange = new IntRange(0, 14);
            TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.DEFAULT, "sample");
            String name = sampleNode.nextTextFieldE("name", nameSet);
            int numberOfFingers = sampleNode.nextIntFieldD("numberOfFingers", 10, numberOfFingersRange);
            File file = (File)sampleNode.nextStringMappedFieldN("file", File.class);
            sampleNode.verifyNoMoreChildren();
            SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
        }
        catch (Exception e)
        {
            System.out.println(e);
        }
    }
}
