package rasmus_torkel.test.xml_basic.simple;

import java.io.File;

import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.MiscXmlFunctions;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;
import rasmus_torkel.xml_basic.write.XmlSink;

public class SimpleXmlTagNodeIdDemo
{
    public static final XmlNameSpace NS_SAMPLE = new XmlNameSpace("x", "http://rasmustorkel.com/ns1/");
    public static final TagNodeId ID_SAMPLE =            new TagNodeId("sample", NS_SAMPLE);
    public static final TagNodeId ID_NAME =              new TagNodeId("name", NS_SAMPLE);
    public static final TagNodeId ID_NUMBER_OF_FINGERS = new TagNodeId("numberOfFingers", NS_SAMPLE);
    public static final TagNodeId ID_FILE =              new TagNodeId("file", NS_SAMPLE);
    
    public static void
    main(String[] args)
    {
        String xmlString = makeXmlNs();
        readXml(xmlString);
    }
    
    public static String
    makeXmlNs()
    {
        String name = "Rasmus Torkel";
        int numberOfFingers = 10;
        File file = new File("./data/xx.txt");
        SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode(ID_SAMPLE);
        xmlSink.sinkSimpleNode(ID_NAME, name);
        xmlSink.sinkSimpleNode(ID_NUMBER_OF_FINGERS, numberOfFingers);
        xmlSink.sinkSimpleNode(ID_FILE, file);
        xmlSink.closeNode();
        String xmlString = xmlSink.toString();
        System.out.println(xmlString);
        MiscXmlFunctions.showXmlForHtml(xmlString);
        return xmlString;
    }
    
    public static void
    readXml(String xmlString)
    {
        TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.DEFAULT, ID_SAMPLE);
        String name = sampleNode.nextTextFieldE(ID_NAME);
        int numberOfFingers = sampleNode.nextIntFieldD(ID_NUMBER_OF_FINGERS, 10);
        File file = (File)sampleNode.nextStringMappedFieldN(ID_FILE, File.class);
        sampleNode.verifyNoMoreChildren();
        SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
    }
}
