package rasmus_torkel.test.xml_basic.simple;

import java.io.File;

import rasmus_torkel.text.read.exception.ReadException;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.MiscXmlFunctions;
import rasmus_torkel.xml_basic.rw.TagNodeId;
import rasmus_torkel.xml_basic.write.XmlSink;

public class SimpleXmlUnorderedRetrievalDemo
{
    public static void
    main(String[] args)
    {
        String xmlString = makeXml();
        readXml(xmlString);
        readXmlReadRest(xmlString);
        readUnorderedXmlOrdered(xmlString);
        
        readXmlBadOptionCatchException(xmlString);
    }
    
    public static String
    makeXml()
    {
        String name = "Rasmus Torkel";
        int numberOfFingers = 10;
        File file = new File("./data/xx.txt");
        SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("sample");
        xmlSink.sinkSimpleNode("numberOfFingers", numberOfFingers);
        xmlSink.sinkSimpleNode("file", file);
        xmlSink.sinkSimpleNode("name", name);
        xmlSink.closeNode();
        String xmlString = xmlSink.toString();
        System.out.println(xmlString);
        MiscXmlFunctions.showXmlForHtml(xmlString);
        return xmlString;
    }
    
    public static void
    readXmlBadOptionCatchException(String xmlString)
    {
        try
        {
            readXmlBadOption(xmlString);
        }
        catch (ReadException e)
        {
            System.out.println(e);
        }
    }
    
    public static void
    readXmlBadOption(String xmlString)
    {
        System.out.println("readXmlBadOption");
        TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.DEFAULT, "sample");
        String name = sampleNode.anyOrderTextFieldE("name");
        int numberOfFingers = sampleNode.anyOrderIntFieldD("numberOfFingers", 10);
        File file = (File)sampleNode.anyOrderStringMappedFieldN("file", File.class);
        sampleNode.verifyNoOtherChildren("name", "numberOfFingers", "file");
        SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
    }
    
    public static void
    readXml(String xmlString)
    {
        System.out.println("readXml");
        TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.ANY_ORDER_RETRIEVAL, "sample");
        String name = sampleNode.anyOrderTextFieldE("name");
        int numberOfFingers = sampleNode.anyOrderIntFieldD("numberOfFingers", 10);
        File file = (File)sampleNode.anyOrderStringMappedFieldN("file", File.class);
        sampleNode.verifyNoOtherChildren("name", "numberOfFingers", "file");
        SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
    }
    
    public static void
    readXmlReadRest(String xmlString)
    {
        System.out.println("readXmlReadRest");
        TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.ANY_ORDER_RETRIEVAL, "sample");
        String name = sampleNode.anyOrderTextFieldE("name");
        int numberOfFingers = sampleNode.anyOrderIntFieldD("numberOfFingers", 10);
        File file = (File)sampleNode.anyOrderStringMappedFieldN("file", File.class);
        sampleNode.readRest();
        SimpleXmlDemo.printSampleData(name, numberOfFingers, file);
    }
    
    public static void
    readUnorderedXmlOrdered(String xmlString)
    {
        System.out.println("readUnorderedXmlOrdered");
        TagNode sampleNode = XmlReader.xmlStringToRoot(xmlString, "demo", XmlReadOptions.DEFAULT, "sample");
        String name = null;
        String numberOfFingersStr = null;
        String fileStr = null;
        while (sampleNode.peepChildN() != null)
        {
            TagNode childNode = sampleNode.nextChildE();
            TagNodeId id = childNode._id;
            if (id._nameSpace != null)
            {
                throw new RuntimeException(childNode + " has name space");
            }
            if (id._relativeName.equals("name"))
            {
                if (name != null)
                {
                    throw new RuntimeException(childNode + " is the second name node");
                }
                name = childNode.onlyText();
            }
            else if (id._relativeName.equals("numberOfFingers"))
            {
                if (numberOfFingersStr != null)
                {
                    throw new RuntimeException(childNode + " is the second numberOfFingers node");
                }
                numberOfFingersStr = childNode.onlyText();
            }
            else if (id._relativeName.equals("file"))
            {
                if (fileStr != null)
                {
                    throw new RuntimeException(childNode + " is the second file node");
                }
                fileStr = childNode.onlyText();
            }
            else
            {
                throw new RuntimeException(childNode + " has unexpected name");
            }
        }
        SimpleXmlDemo.printSampleData(name, Integer.parseInt(numberOfFingersStr), new File(fileStr));
    }
}
