/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.xml_basic.tool;

import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.xml_basic.tool.XmlComparer;

public class XmlComparerSuite extends UnitTest
{
    public static final String NO_CONTENT_ROOT = "<root/>";
    public static final String EMPTY_ROOT = "<root></root>";
    
    public static final String REF =
        "<root ra1=\"rr11\" ra2=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22></n22></n2></root>";
    
    public static final String[] DIFF_SET =
    {
      //"<root ra1=\"rr11\" ra2=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22></n22></n2></root>",
        "<root ra1=\"rr11\" ra2=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22>xx</n22></n2></root>",
        "<root ra1=\"rr11\" ra2=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22/>     </n2></root>",
        "<root ra1=\"rr11\" ra2=\"rr2x\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22></n22></n2></root>",
        "<root ra1=\"rr11\" rax=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22></n22></n2></root>",
        "<root ra1=\"rr11\" ra2=\"rr22\" ra2=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22></n22></n2></root>",
        "<root ra1=\"rr11\" ra1=\"rr11\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22></n22></n2></root>",
        "<root ra1=\"rr11\" ra2=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n23></n23></n2></root>",
        "<rootx ra1=\"rr11\" ra2=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22></n22></n2></root>",
        "<root ra1=\"rr11\" ra2=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21>           </n2></root>",
        "<root ra1=\"rr11\" ra2=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2>                <n22></n22></n2></root>",
        //"<root ra1=\"rr11\" ra2=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22></n22></n2></root>",
    };
    
    public static final String[] EQUIV_SET =
    {
        "<root ra2=\"rr22\" ra1=\"rr11\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22></n22></n2></root>",
        "<root ra1=\"rr11\" ra2=\"rr22\"><n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1><n2><n21>n21xx</n21><n22>  </n22></n2></root>",
        "<root ra1=\"rr11\" ra2=\"rr22\">\n" +
        "  <n1 n1a1=\"n1a11\" n1a2=\"n1a22\">n1xx</n1>\n" +
        "  <n2>\n" +
        "    <n21>n21xx</n21>\n" +
        "    <n22></n22>\n"+
        "  </n2>\n" +
        "</root>\n",
    };
    
    public void
    test(String  xml1,
         String  xml2,
         boolean expectEquivalent)
    {
        String context1 = "Unit test XML string 1";
        String context2 = "Unit test XML string 2";
        test(xml1, context1, xml2, context2, expectEquivalent);
        test(xml2, context1, xml1, context2, expectEquivalent);
    }
    
    public void
    test(String  xml1,
         String  context1,
         String  xml2,
         String  context2,
         boolean expectEquivalent)
    {
        System.out.println("First XML");
        System.out.println("Context = " + context1 + ", xml below");
        System.out.println(xml1);
        printRuler(xml1);
        printTestPartSeparator();
        
        System.out.println("Second XML");
        System.out.println("Context = " + context2 + ", xml below");
        System.out.println(xml2);
        printRuler(xml2);
        printTestPartSeparator();
        
        boolean areEquivalent = XmlComparer.areEquivalent(xml1, context1, xml2, context2, System.out);
        printTestPartSeparator();
        
        if (areEquivalent)
        {
            System.out.println("Equivalent");
        }
        else
        {
            System.out.println("Not equivalent");
        }
        
        if (areEquivalent != expectEquivalent)
        {
            System.out.println("areEquivalent returned unexpected result");
            throw new UnitTestError(getClass());
        }
        printEndTest();
    }
    
    public void
    test()
    {
        announceTestClassStart();
        test("<root><n1></n1></root>", "<root><n1>xx</n1></root>", false);
        test(NO_CONTENT_ROOT, NO_CONTENT_ROOT, true);
        test(EMPTY_ROOT, EMPTY_ROOT, true);
        test(EMPTY_ROOT, NO_CONTENT_ROOT, false);
        test(EMPTY_ROOT, null, NO_CONTENT_ROOT, null, false);
        test(EMPTY_ROOT, null, "<root>xx</root>", null, false);
        for (String equalXml : EQUIV_SET)
        {
            test(REF, equalXml, true);
        }
        for (String diffXml : DIFF_SET)
        {
            test(REF, diffXml, false);
        }
        announceTestClassPass();
    }
    
    public static void
    main(String[] args)
    {
        XmlComparerSuite t = new XmlComparerSuite();
        t.test();
    }
}
