/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.xml_basic.tool;

import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.xml_basic.tool.XmlComparer;
import rasmus_torkel.xml_basic.tool.XmlReformatter;

public class XmlReformatterSuite extends UnitTest
{
    public void
    test(String inXml)
    {
        System.out.println(inXml);
        printRuler(inXml);
        printTestPartSeparator();
        String outXml = XmlReformatter.reformat(inXml, "Unit Test, first reformat");
        System.out.print(outXml);
        printRuler(outXml);
        printTestPartSeparator();
        if (XmlComparer.areEquivalent(inXml, "input XML", outXml, "output XML", System.out))
        {
            System.out.println("The reformatted XML is equivalent");
        }
        else
        {
            System.out.println("The reformatted XML is not equivalent");
            throw new UnitTestError(getClass());
        }
        String repeatOutXml = XmlReformatter.reformat(outXml, "Unit Test, reformatting reformatted");
        if (!repeatOutXml.equals(outXml))
        {
            printRuler(repeatOutXml);
            System.out.print(repeatOutXml);
            throw new UnitTestError(getClass());
        }
        printEndTest();
    }
    
    public void
    test()
    {
        announceTestClassStart();
        test("<root><n1 an1=\"11\" an2=\"22\"/></root>");
        test("<root><n1 an1=\"11\" an2=\"22\">xx</n1></root>");
        test("<root/>");
        test("<root></root>");
        test("<root>xx</root>");
        test("<root a1=\"11\"/>");
        test("<root a1=\"11\"></root>");
        test("<root a1=\"11\">xx</root>");
        test("<root a1=\"11\" a2=\"22\">xx</root>");
        test("<root a1=\"11\" a2=\"22\">xx</root>");
        test("<root a1=\"11\" a2=\"22\"><nn1 an1=\"11\" an2=\"22\"/></root>");
        test("<root><n1/><n2/></root>");
        test("<root a1=\"11\" a2=\"22\"><nn1 an1=\"11\" an2=\"22\"/><nn2>xx</nn2><nn3><nn31/><nn32>32x</nn32></nn3><nn4/><nn5/></root>");
        test("<x:person xmlns:x=\"NS1\"><x:name><x:firstName>Johann</x:firstName><x:surname>Torkel</x:surname></x:name><x:occupation>Accountant</x:occupation></x:person>");
        test("<x:person xmlns:x=\"NS1\"><x:name x:type=\"name\"><x:firstName>Johann</x:firstName><x:surname>Torkel</x:surname></x:name><x:occupation>Accountant</x:occupation></x:person>");
        test("<x:person xmlns:x=\"NS1\"><name x:type=\"name\"><firstName>Johann</firstName><surname>Torkel</surname></name><occupation>Accountant</occupation></x:person>");
        test("<root xmlns:x=\"NS1\"><x:a/></root>");
        test("<root xmlns:x=\"NS1\"><a x:attr=\"vv\"/></root>");
        announceTestClassPass();
    }
    
    public static void
    main(String[] args)
    {
        XmlReformatterSuite t = new XmlReformatterSuite();
        t.test();
    }
}
