/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.xml_basic.tool;

import java.io.File;

import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.xml_basic.tool.XmlComparer;
import rasmus_torkel.xml_basic.tool.XmlReformatter;

public class XmlReformatterTest extends UnitTest
{
    public static void
    main(String[] args)
    {
        announceTestClassStart(XmlReformatterTest.class);
        boolean areEquivalent;
        if (args.length == 1)
        {
            String inFileName = args[0];
            File inFile = new File(inFileName);
            String outXml = XmlReformatter.reformat(inFile);
            System.out.print(outXml);
            printTestPartSeparator();
            areEquivalent = XmlComparer.areEquivalent(inFile, outXml, "reformatted XML", System.out);
        }
        else if (args.length == 2)
        {
            String inFileName = args[0];
            String outFileName = args[1];
            File inFile = new File(inFileName);
            File outFile = new File(outFileName);
            XmlReformatter.reformat(inFile, outFile);
            printTestPartSeparator();
            areEquivalent = XmlComparer.areEquivalent(inFile, outFile, System.out);
        }
        else
        {
            System.out.println("Need one or two arguments, inputFileName and optional outputFileName");
            return;
        }
        printTestPartSeparator();
        if (areEquivalent)
        {
            System.out.println("The reformatting produced equivalent results");
            printEndTest();
            announceTestClassPass(XmlReformatterTest.class);
        }
        else
        {
            System.out.println("The reformatting failed as the resulting XML is not equivalent to the original");
            printEndTest();
            throw new UnitTestError(XmlReformatterTest.class);
        }
    }
}
