package rasmus_torkel.test.xml_basic.write;

import rasmus_torkel.misc.ObjectUtil;
import rasmus_torkel.set.chars.CharSet;
import rasmus_torkel.set.chars.DefinedCharSets;
import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.text.encode.CharEncodeOptions;
import rasmus_torkel.xml_basic.read.TagNode;
import rasmus_torkel.xml_basic.read.XmlReadOptions;
import rasmus_torkel.xml_basic.read.impl.XmlReader;
import rasmus_torkel.xml_basic.rw.MiscXmlFunctions;
import rasmus_torkel.xml_basic.rw.XmlAttribute;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;
import rasmus_torkel.xml_basic.write.XmlSink;
import rasmus_torkel.xml_basic.write.XmlWriteOptions;

public class XmlWriteCharTest extends UnitTest
{
    public static void
    main(String[] args)
    {
        XmlWriteCharTest t = new XmlWriteCharTest();
        announceTestClassStart(t.getClass());
        try
        {
            t.test();
            announceTestClassPass(t.getClass());
        }
        catch (Exception e)
        {
            e.printStackTrace(System.out);
            t.fail("Unexpected " + e); 
        }
    }
    
    public void
    test()
    {
        testNameCharSet(DefinedCharSets.LATIN_LOWER_CASE_LETTERS, "Name");
        testNameCharSet(DefinedCharSets.LATIN_LOWER_CASE_LETTERS, "namE");
        testName("name");
        testName("x.x");
        testNameErr("111");
        testValue("value");
        testValue("<>&\"'");
        testValue("\t\r\n");
        testValueErr(XmlWriteOptions.DEFAULT, "" + (char)195 + "," + (char)0xffff);
        CharEncodeOptions encodeOptionsLower = CharEncodeOptions.DEFAULT.diffAllChars(DefinedCharSets.LATIN_LOWER_CASE_LETTERS);
        XmlWriteOptions writeOptionsLower = XmlWriteOptions.DEFAULT.diffEncodeOptions(encodeOptionsLower, false);
        testValueErr(writeOptionsLower, "valuE");
        CharEncodeOptions encodeOptionsCoreLowerPlusUpper =
                new CharEncodeOptions(DefinedCharSets.LATIN_LOWER_CASE_LETTERS, DefinedCharSets.LATIN_LETTERS);
        XmlWriteOptions writeOptionsCoreLowerPlusUpper = XmlWriteOptions.DEFAULT.diffEncodeOptions(encodeOptionsCoreLowerPlusUpper, false);
        testValue(writeOptionsCoreLowerPlusUpper, "Value");
        testValueErr(writeOptionsCoreLowerPlusUpper, "the_value");
    }
    
    public void
    testNameCharSet(CharSet nameCharSet,
                    String  name)
    {
        testName(name);
        XmlWriteOptions options = XmlWriteOptions.DEFAULT.diffNameCharSet(nameCharSet);
        testNameErr(options, name);
    }
    
    public void
    testName(String          name)
    {
        testName(XmlWriteOptions.DEFAULT, name);
    }
    
    public void
    testName(XmlWriteOptions options,
             String          name)
    {
        System.out.println("nameCharSet is " + options._nameCharSet);
        test(options, name, null, name, "v", "v");
    }
    
    public void
    testValueCharSet(CharSet valueCharSet,
                     String  value)
    {
        testValue(value);
        CharEncodeOptions encodeOption = CharEncodeOptions.DEFAULT.diffAllChars(valueCharSet);
        XmlWriteOptions options = XmlWriteOptions.DEFAULT.diffEncodeOptions(encodeOption, false);
        testValueErr(options, value);
    }
    
    public void
    testValue(String value)
    {
        testValue(XmlWriteOptions.DEFAULT, value);
    }
    
    public void
    testValue(XmlWriteOptions options,
              String          value)
    {
        test(options, "n", null, "a", value, value);
    }
    
    public void
    testNameErr(String          name)
    {
        testNameErr(XmlWriteOptions.DEFAULT, name);
    }
    
    public void
    testNameErr(XmlWriteOptions options,
                String          name)
    {
        System.out.println("nameCharSet is " + options._nameCharSet);
        testErr(options, name, null, "a", "v", "v");
        System.out.println("nameCharSet is " + options._nameCharSet);
        testErr(options, "n", null, name, "v", "v");
    }
    
    public void
    testValueErr(XmlWriteOptions options,
                 String          value)
    {
        System.out.println("valueCharSet is " + options._encodeOptions._allChars);
        testErr(options, "n", null, "a", "v", value);
        System.out.println("valueCharSet is " + options._encodeOptions._allChars);
        testErr(options, "n", null, "a", value, "v");
    }
    
    public void
    test(XmlWriteOptions options,
         String          nodeRelativeName,
         XmlNameSpace    nodeNameSpace,
         String          attributeName,
         String          attributeValue,
         String          nodeText)
    {
        System.out.println(nodeRelativeName + " " + nodeNameSpace + " " + attributeName + " " + attributeValue + " " + nodeText);
        XmlSink sink = new XmlSink(options);
        sink.startNode(nodeRelativeName, nodeNameSpace);
        sink.sinkAttribute(attributeName, attributeValue);
        sink.sinkText(nodeText);
        sink.closeNode();
        printTestPartSeparator();
        
        String xml = sink.toString();
        System.out.print(xml);
        printTestPartSeparator();
        
        String html = MiscXmlFunctions.prepareXmlForHtml(xml);
        System.out.print(html);
        printTestPartSeparator();
        
        TagNode node = XmlReader.xmlStringToRoot(xml, "Unit Test", XmlReadOptions.DEFAULT);
        int expectedAttributeIndex = 0;
        if (nodeNameSpace != null)
        {
            expectedAttributeIndex++;
        }
        int expectedAttributeQty = 1 + expectedAttributeIndex;
        int attributeQty = node.attributeQty();
        if (expectedAttributeQty != attributeQty)
        {
            fail("Attribute qty differs, expected " + expectedAttributeQty + ", read " + attributeQty);
        }
        XmlAttribute attribute = node.attribute(expectedAttributeIndex);
        String nodeText2 = node.nextTextE();
        System.out.println(node._id._relativeName + " " + node._id._nameSpace + " " + attribute._name + " " + attribute._value + " " + nodeText2);
        if (!ObjectUtil.areEqual(node._id._relativeName, nodeRelativeName))
        {
            fail("Node relativeName differs, wrote " + nodeRelativeName + ", read " + node._id._relativeName);
        }
        if (!ObjectUtil.areEqual(node._id._nameSpace, nodeNameSpace))
        {
            fail("Node relativeName differs, wrote " + nodeNameSpace + ", read " + node._id._nameSpace);
        }
        if (!attribute._name.equals(attributeName))
        {
            fail("attributeName differs, wrote " + attributeName + ", read " + attribute._name);
        }
        if (!attribute._value.equals(attributeValue))
        {
            fail("attributeValue differs, wrote " + attributeName + ", read " + attribute._value);
        }
        if (!nodeText2.equals(nodeText))
        {
            fail("node text differs, wrote " + nodeText + ", read " + nodeText2);
        }
        printEndTest();
    }
    
    public void
    testErr(XmlWriteOptions options,
            String          nodeRelativeName,
            XmlNameSpace    nodeNameSpace,
            String          attributeName,
            String          attributeValue,
            String          nodeText)
    {
        System.out.println(nodeRelativeName + " " + nodeNameSpace + " " + attributeName + " " + attributeValue + " " + nodeText);
        XmlSink sink = new XmlSink(options);
        try
        {
            sink.startNode(nodeRelativeName, nodeNameSpace);
            sink.sinkAttribute(attributeName, attributeValue);
            sink.sinkText(nodeText);
            sink.closeNode();
            System.out.println("Expected exception but succeeded");
            throw new UnitTestError(this.getClass());
        }
        catch (RuntimeException e)
        {
            System.out.println("Got exception as expected: " + e);
        }
        printEndTest();
    }
}
