package rasmus_torkel.test.xml_basic.write;

import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.xml_basic.rw.XmlAttribute;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;
import rasmus_torkel.xml_basic.write.XmlSink;

public class XmlWriteErrorTest extends UnitTest
{
    private static final String       NS_NAME_1 = "rtorkel.com/namespace_1";
    private static final XmlNameSpace NS_1A = new XmlNameSpace("_1a", NS_NAME_1);
    private static final XmlNameSpace NS_1B = new XmlNameSpace("_1b", NS_NAME_1);
    
    public static void
    main(String[] args)
    {
        announceTestClassStart(XmlWriteErrorTest.class);
        testNonUniqueAttributesNoNs();
        testNonUniqueAttributesNs();
        testPreRootNsDeclaration();
        testPostRootNsDeclaration();
        testAttributeWithColonDeclaration();
        testNameSpaceAsAttributeDeclaration();
        announceTestClassPass(XmlWriteErrorTest.class);
    }
    
    public static void
    testNonUniqueAttributesNoNs()
    {
        System.out.println("Trying non-unique attributes without name space");
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("root");
        xmlSink.sinkAttribute("a", "1");
        try
        {
            System.out.println("Trying second attribute with same name");
            xmlSink.sinkAttribute("a", "2");
            System.out.println("Unexpected success");
            xmlSink.closeNode();
            String xmlString = xmlSink.toString();
            System.out.println(xmlString);
            throw new UnitTestError();
        }
        catch (Exception e)
        {
            System.out.println("Caught exception as expected: " + e);
            printEndTest();
        }
    }
    
    public static void
    testNonUniqueAttributesNs()
    {
        System.out.println("Trying non-unique attributes with name space");
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("root");
        xmlSink.sinkAttribute(new XmlAttribute("a", NS_1A, "1"));
        try
        {
            System.out.println("Trying second attribute with same name");
            xmlSink.sinkAttribute(new XmlAttribute("a", NS_1B, "2"));
            System.out.println("Unexpected success");
            xmlSink.closeNode();
            String xmlString = xmlSink.toString();
            System.out.println(xmlString);
            throw new UnitTestError();
        }
        catch (Exception e)
        {
            System.out.println("Caught exception as expected: " + e);
            printEndTest();
        }
    }
    
    public static void
    testPreRootNsDeclaration()
    {
        System.out.println("Trying declaration of name space before root");
        XmlSink xmlSink = new XmlSink();
        try
        {
            xmlSink.declareNameSpace(NS_1A);
            System.out.println("Unexpected success declaring namespace before root opened");
            throw new UnitTestError();
        }
        catch (Exception e)
        {
            System.out.println("Caught exception as expected: " + e);
            printEndTest();
        }
    }
    
    public static void
    testPostRootNsDeclaration()
    {
        System.out.println("Trying declaration of name space after root");
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("root");
        xmlSink.closeNode();
        try
        {
            xmlSink.declareNameSpace(NS_1A);
            System.out.println("Unexpected success declaring namespace after root closed");
            throw new UnitTestError();
        }
        catch (Exception e)
        {
            System.out.println("Caught exception as expected: " + e);
            printEndTest();
        }
    }
    
    private static void
    testAttributeWithColonDeclaration()
    {
        System.out.println("Trying declaration of attribute which is really namespace");
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("root");
        try
        {
            xmlSink.sinkAttribute("a:a", "rtorkel.com/namespace_1");
            System.out.println("Unexpected success declaring namespaced attribute like regular attribute");
            throw new UnitTestError();
        }
        catch (Exception e)
        {
            System.out.println("Caught exception as expected: " + e);
            printEndTest();
        }
        finally
        {
            xmlSink.abandon();
        }
    }
    
    private static void
    testNameSpaceAsAttributeDeclaration()
    {
        System.out.println("Trying declaration of attribute which is really namespace");
        XmlSink xmlSink = new XmlSink();
        xmlSink.startNode("root");
        try
        {
            XmlAttribute attr = new XmlAttribute("a", XmlNameSpace.XMLNS, "rtorkel.com/namespace_1");
            xmlSink.sinkAttribute(attr);
            System.out.println("Unexpected success declaring namespace just like a regular attribute");
            throw new UnitTestError();
        }
        catch (Exception e)
        {
            System.out.println("Caught exception as expected: " + e);
            printEndTest();
        }
        finally
        {
            xmlSink.abandon();
        }
    }
}
