package rasmus_torkel.test.xml_basic.write;

import rasmus_torkel.set.chars.DefinedCharSets;
import rasmus_torkel.test_framework.UnitTest;
import rasmus_torkel.test_framework.UnitTestError;
import rasmus_torkel.text.encode.CharEncodeOptions;
import rasmus_torkel.text.encode.SpaceEscapeOption;
import rasmus_torkel.text.write.Indent;
import rasmus_torkel.text.write.IndentingLineSinkOptions;
import rasmus_torkel.text.write.LineEndOption;
import rasmus_torkel.xml_basic.rw.XmlNameSpace;
import rasmus_torkel.xml_basic.write.AttributeLineOptions;
import rasmus_torkel.xml_basic.write.XmlSink;
import rasmus_torkel.xml_basic.write.XmlWriteOptions;

public class XmlWriteOptionsTest extends UnitTest
{
    public static void
    doOne(XmlWriteOptions options)
    {
        System.out.print(options.toXml());
        printTestPartSeparator();

        XmlNameSpace utNs = new XmlNameSpace("ut", "Name Space for XmlWriteOptionsTest");
        XmlSink xmlSink = new XmlSink(options);
        xmlSink.startNode("root", utNs);
        xmlSink.declareNameSpace(utNs);
        {
            xmlSink.sinkSimpleNode("n0", utNs, "This is value V0");
        }
        {
            xmlSink.startNode("n1", utNs);
            xmlSink.sinkAttribute("a1", "av1");
            xmlSink.sinkText("This is value V1");
            xmlSink.closeNode();
        }
        {
            xmlSink.startNode("n2", utNs);
            xmlSink.sinkAttribute("a1", "av1");
            xmlSink.sinkAttribute("a2", "av2");
            xmlSink.sinkText("This is value V2");
            xmlSink.closeNode();
        }
        {
            xmlSink.startNode("n3", utNs);
            xmlSink.sinkAttribute("a3", "av3");
            xmlSink.sinkAttribute("a2", "av2");
            xmlSink.sinkAttribute("a1", "av1");
            xmlSink.sinkText("This is value V3." + LineEndOption.SYSTEM_PROPERTY._lineEndStr + "And this is the second line.");
            xmlSink.closeNode();
        }
        xmlSink.closeNode();
        System.out.println(xmlSink.toString());
        printEndTest();
    }
    
    public static void
    doErrorCase(XmlWriteOptions options)
    {
        try
        {
            System.out.print(options.toXml(XmlWriteOptions.DEFAULT));
            printTestPartSeparator();
            doOne(options);
            System.out.println("Failed to get expected exception");
            throw new UnitTestError(XmlWriteOptionsTest.class);
        }
        catch (Exception e)
        {
            System.out.println("Got exception as expected: " + e);
        }
    }
    
    public static void
    main(String[] args)
    {
        doOne(XmlWriteOptions.DEFAULT.diffIndentingLineSinkOptions(IndentingLineSinkOptions.DEFAULT.diffLineEndOption(LineEndOption.MAC)));
        doOne(XmlWriteOptions.DEFAULT.diffIndentingLineSinkOptions(IndentingLineSinkOptions.DEFAULT.diffIndent(Indent.TAB)));
        doOne(XmlWriteOptions.DEFAULT.diffIndentingLineSinkOptions(IndentingLineSinkOptions.DEFAULT.diffIndent(Indent.FOUR_SPACES)));
        doOne(XmlWriteOptions.DEFAULT.diffAttributeSortOption(true));
        doOne(XmlWriteOptions.DEFAULT);
        doOne(XmlWriteOptions.DEFAULT.diffAttributeLineOptions(new AttributeLineOptions(false, false, 2)));
        doOne(XmlWriteOptions.DEFAULT.diffAttributeLineOptions(new AttributeLineOptions(true, true, 2)));
        doOne(XmlWriteOptions.DEFAULT.diffAttributeLineOptions(new AttributeLineOptions(false, true, 3)));
        doOne(XmlWriteOptions.ONE_LINE_PER_TAG);
        doOne(XmlWriteOptions.ONE_LINE_PER_ATTRIBUTE);
        doOne(XmlWriteOptions.DEFAULT.diffEncodeOptions(CharEncodeOptions.DONT_ESCAPE_SPACE, false));
        doOne(XmlWriteOptions.DEFAULT.diffEncodeOptions(CharEncodeOptions.DONT_ESCAPE_SPACE.diffSpaceEscapeOption(SpaceEscapeOption.ESCAPE_NON_SIMPLE), false));
        doOne(XmlWriteOptions.DEFAULT.diffEncodeOptions(CharEncodeOptions.ENGLISH_KEYBOARD_GENERAL_TEXT, false));
        CharEncodeOptions coreLower = CharEncodeOptions.DEFAULT.diffCoreChars(DefinedCharSets.LATIN_LOWER_CASE_LETTERS);
        doOne(XmlWriteOptions.DEFAULT.diffEncodeOptions(coreLower, false));
        CharEncodeOptions allLower = coreLower.diffAllChars(DefinedCharSets.LATIN_LOWER_CASE_LETTERS);
        doErrorCase(XmlWriteOptions.DEFAULT.diffEncodeOptions(allLower, true));
    }
}
