/*
 * Copyright Rasmus Torkel 2013
 * Author    Rasmus Torkel 2013
 * 
 * You may only use this software subject to the terms and conditions which either you or your
 * organisation would have downloaded with this software and which should have been placed
 * somewhere accessible to you. These conditions are likely to
 * -- allow you or your organisation to determine whether you or your organisation
 *    may use this software for free or not and if not how to pay and how much;
 * -- prohibit or restrict republishing of this software and transmission to other organisations,
 *    even related companies within the same group;
 * -- prohibit the renaming of this class and the containing package.
 *    
 * Neither you nor your organisation may modify this header. However, you or your organisation
 * may modify anything below this header. If so, you or your organisation must place a comment
 * to that effect immediately below this header.
 */
package rasmus_torkel.test.xml_basic.write;

import rasmus_torkel.set.chars.DefinedCharSets;
import rasmus_torkel.text.encode.CharEncodeOptions;
import rasmus_torkel.text.encode.SpaceEscapeOption;
import rasmus_torkel.text.encode.UnsupportedCharacterEncodeException;
import rasmus_torkel.text.write.Indent;
import rasmus_torkel.text.write.IndentingLineSinkOptions;
import rasmus_torkel.xml_basic.rw.XmlAttribute;
import rasmus_torkel.xml_basic.tool.XmlReformatterWriteOptions;
import rasmus_torkel.xml_basic.write.XmlSink;
import rasmus_torkel.xml_basic.write.XmlWriteOptions;

public class XmlWriteTest1 extends XmlWriteTest
{
    private static final String TRICKY_CHARS_STRING = "lt< gt> quot\" apos\' amp& rn\r\na9" + (char)0xa9;
    
    private XmlSink
    startTest(XmlWriteOptions options)
    {
        return startTest(options, options);
    }
    
    private XmlSink
    startTest(XmlWriteOptions optionsOptions,
              XmlWriteOptions options)
    {
        System.out.println("options:");
        System.out.println();
        //System.out.println(options.toString());
        System.out.println(options.toXml(optionsOptions));
        printTestPartSeparator();
        return new XmlSink(options);
    }
    
    private void
    endTest(XmlSink xmlSink)
    {
        //String xml = xmlSink.toString();
        //System.out.println(xml);
        checkXmlSyntax(xmlSink);
        printEndTest();
    }
    
    public void
    test2WithSentence(String          optionsDescription,
                      XmlWriteOptions options,
                      boolean         expectSuccess)
    {
        testSentence(optionsDescription, options, expectSuccess);
        testAttributeWithSentence(optionsDescription, options, expectSuccess);
    }
    
    public void
    testSentence(String          optionsDescription,
                 XmlWriteOptions options,
                 boolean         expectSuccess)
    {
        System.out.println("Options description: " + optionsDescription);
        XmlSink xmlSink = startTest(XmlWriteOptions.DEFAULT, options);
        try
        {
            xmlSink.sinkSimpleNode("root", "This is a sentence.");
            if (!expectSuccess)
            {
                throw fail("Unexpected success while sinking XML");
            }
            endTest(xmlSink);
        }
        catch (UnsupportedCharacterEncodeException e)
        {
            System.out.println(e);
            System.out.println("Position = " + e._position);
            System.out.println("Generated text so far:");
            System.out.println(e._textSoFar);
            if (expectSuccess)
            {
                throw fail("Unexpected exception while sinking XML " + e);
            }
            printEndTest();
        }
    }
    
    public void
    testAttributeWithSentence(String          optionsDescription,
                              XmlWriteOptions options,
                              boolean         expectSuccess)
    {
        System.out.println("Options description: " + optionsDescription);
        XmlSink xmlSink = startTest(XmlWriteOptions.DEFAULT, options);
        try
        {
            xmlSink.startNode("root");
            xmlSink.sinkAttribute("aa", "This is a sentence.");
            xmlSink.sinkText("xx");
            xmlSink.closeNode();
            if (!expectSuccess)
            {
                String xml = xmlSink.toString();
                System.out.println(xml);
                throw fail("Unexpected success while sinking XML");
            }
            endTest(xmlSink);
        }
        catch (UnsupportedCharacterEncodeException e)
        {
            System.out.println(e);
            System.out.println("Position = " + e._position);
            System.out.println("Generated text so far:");
            System.out.println(e._textSoFar);
            if (expectSuccess)
            {
                throw fail("Unexpected exception while sinking XML " + e);
            }
            printEndTest();
        }
    }
    
    public void
    testSimpleNodeWithTrickyChars(String          optionsDescription,
                                  XmlWriteOptions options)
    {
        System.out.println("testSimpleNodeWithTrickyChars");
        System.out.println("Options description: " + optionsDescription);
        XmlSink xmlSink = startTest(options);
        xmlSink.sinkSimpleNode("root", TRICKY_CHARS_STRING);
        endTest(xmlSink);
    }
    
    public void
    testTrickyChars(XmlWriteOptions options)
    {
        System.out.println("testTrickyChars");
        XmlSink xmlSink = startTest(options);
        xmlSink.startNode("_123");
        xmlSink.sinkAttribute("_", TRICKY_CHARS_STRING);
        xmlSink.sinkAttribute("a", "\"");
        xmlSink.sinkAttribute("b", "\"\'\"");
        xmlSink.sinkAttribute("c", "\'");
        xmlSink.sinkAttribute("d", "\'\"\'");
        xmlSink.sinkText(TRICKY_CHARS_STRING);
        xmlSink.closeNode();
        endTest(xmlSink);
    }
    /*
    public void
    testXmlInXml(XmlWriteOptions options)
    {
        System.out.println("testXmlInXml");
        XmlSink xmlSink1 = startTest(options);
        xmlSink1.sinkOpenTag("root");
        xmlSink1.sinkSimpleNode("node", "Hello");
        xmlSink1.closeNode();
        String xml1 = xmlSink1.toString();
        System.out.println(xml1);
        printTestPartSeparator();
        
        XmlSink xmlSink2 = new XmlSink(options);
        xmlSink2.sinkSimpleNode("contains_xml_1", xml1);
        endTest(xmlSink2);
    }*/
    
    public void
    testVaried(XmlWriteOptions options)
    {
        System.out.println("testVaried");
        XmlSink xmlSink = startTest(options);
        xmlSink.startNode("root");
        xmlSink.sinkAttribute(new XmlAttribute("n1root", "v1root"));
        {
            sinkSelfClosingTags(xmlSink, 5);
            sinkEmptyNodes(xmlSink, 5);
            sinkTextNodes(xmlSink, 5);
            sinkNestedNodes(xmlSink, 5);
        }
        xmlSink.closeNode();
        endTest(xmlSink);
    }
    
    private void
    sinkSelfClosingTags(XmlSink xmlSink,
                        int     qty)
    {
        for (int i = 0; i <= qty; i++)
        {
            String nodeName = "sst" + i;
            xmlSink.startNode(nodeName);
            sinkAttributes(xmlSink, nodeName, i);
            xmlSink.closeTagAndNode();
        }
    }
    
    private void
    sinkEmptyNodes(XmlSink xmlSink,
                   int     qty)
    {
        for (int i = 0; i <= qty; i++)
        {
            String nodeName = "empty" + i;
            xmlSink.startNode(nodeName);
            sinkAttributes(xmlSink, nodeName, i);
            xmlSink.closeNode();
        }
    }
    
    private void
    sinkTextNodes(XmlSink xmlSink,
                  int     qty)
    {
        for (int i = 0; i <= qty; i++)
        {
            String nodeName = "simple" + i;
            xmlSink.startNode(nodeName);
            sinkAttributes(xmlSink, nodeName, i);
            xmlSink.sinkText("This is text node text");
            xmlSink.closeNode();
        }
    }
    
    private void
    sinkNestedNodes(XmlSink xmlSink,
                    int     qty)
    {
        for (int i = 0; i <= qty; i++)
        {
            String nodeName = "nested" + i;
            xmlSink.startNode(nodeName);
            sinkAttributes(xmlSink, nodeName, i);
            xmlSink.sinkSimpleNode("simple1", "Simple node 1");
            xmlSink.sinkSimpleNode("simple2", "Simple node 2");
            xmlSink.closeNode();
        }
    }
    
    private void
    sinkAttributes(XmlSink xmlSink,
                   String  nodeName,
                   int     qty)
    {
        for (int i = qty; i > 0; i--)
        {
            xmlSink.sinkAttribute("n" + i + nodeName, "v" + i + nodeName);
        }
    }
    
    public void
    test()
    {
        announceTestClassStart();
        
        testTrickyChars(XmlWriteOptions.DEFAULT);
        
        testVaried(XmlWriteOptions.DEFAULT);
        
        testVaried(XmlReformatterWriteOptions.DEFAULT);
        
        IndentingLineSinkOptions ilsOptions_emptyIndent =
            IndentingLineSinkOptions.DEFAULT.diffIndent(new Indent(""));
        XmlWriteOptions xwOptions_emptyIndent =
            XmlWriteOptions.DEFAULT.diffIndentingLineSinkOptions(ilsOptions_emptyIndent);
        testVaried(xwOptions_emptyIndent);
        
        IndentingLineSinkOptions ilsOptions_tabIndent =
            IndentingLineSinkOptions.DEFAULT.diffIndent(new Indent("\t"));
        XmlWriteOptions xwOptions_tabIndent =
            XmlWriteOptions.DEFAULT.diffIndentingLineSinkOptions(ilsOptions_tabIndent);
        testVaried(xwOptions_tabIndent);
        
        CharEncodeOptions ceOptions_lessCoreChars =
            CharEncodeOptions.DEFAULT.diffCoreChars(DefinedCharSets.LATIN_LETTERS);
        CharEncodeOptions ceOptions_lessAllChars =
            CharEncodeOptions.DEFAULT.diffAllChars(DefinedCharSets.LATIN_LETTERS);
        XmlWriteOptions xwOptions_lessCoreChars =
            XmlWriteOptions.DEFAULT.diffEncodeOptions(ceOptions_lessCoreChars, true);
        XmlWriteOptions xwOptions_lessAllChars =
            XmlWriteOptions.DEFAULT.diffEncodeOptions(ceOptions_lessAllChars, true);
        test2WithSentence("default", XmlWriteOptions.DEFAULT, true);
        test2WithSentence("core chars are letters", xwOptions_lessCoreChars, true);
        test2WithSentence("all chars are letters", xwOptions_lessAllChars, false);
        
        testSimpleNodeWithTrickyChars("default", XmlWriteOptions.DEFAULT);

        // By default, non-simple space chars aren't in the core set anyway, so for
        // the space options to make any difference, we have to first include some
        // non-simple space chars in the core set.
        CharEncodeOptions ceOptions_generalTextCoreChars =
            CharEncodeOptions.DEFAULT.diffCoreChars(DefinedCharSets.ENGLISH_KEYBOARD_GENERAL_TEXT);
        CharEncodeOptions ceOptions_dontEscapeSpace =
            ceOptions_generalTextCoreChars.diffSpaceEscapeOption(SpaceEscapeOption.DONT_ESCAPE);
        CharEncodeOptions ceOptions_escapeNonSimpleSpace =
            ceOptions_generalTextCoreChars.diffSpaceEscapeOption(SpaceEscapeOption.ESCAPE_NON_SIMPLE);
        XmlWriteOptions xwOptions_dontEscapeSpace =
            XmlWriteOptions.DEFAULT.diffEncodeOptions(ceOptions_dontEscapeSpace, false);
        XmlWriteOptions xwOptions_escapeNonSimpleSpace =
            XmlWriteOptions.DEFAULT.diffEncodeOptions(ceOptions_escapeNonSimpleSpace, false);
        testSimpleNodeWithTrickyChars("general text, don't escape space", xwOptions_dontEscapeSpace);
        testSimpleNodeWithTrickyChars("general text, escape non-simple space", xwOptions_escapeNonSimpleSpace);
    }
    
    public static void
    main(String[] args)
    {
        XmlWriteTest1 t = new XmlWriteTest1();
        t.test();
    }
}
